/***************************************************************************
                               awaymessage.cpp
                             -------------------
    begin                : Sun Feb 20 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 * A simple Dialog box that tells the user that there they are currently set
 * to away and to click on the back button to be back.  While this window is
 * open auto response messages are sent.
 ***************************************************************************/

#include "awaymessage.h"
#include "kaimapplication.h"

/***************************************************************************
 * Constructor
 ***************************************************************************/
AwayMessage::AwayMessage(QWidget *parent, const char *name, settings *aimSettings) : QWidget(parent,name){
  // Load image
  this->setIcon(Icon("kaim_small"));
  // Create GUI
  initDialog();
  // Load setings
  i_aimSettings = aimSettings;
  // Initialize variables and GUI settings
  //message->setReadOnly(true);
  oldSoundSettings = i_aimSettings->SoundMode;
  if (i_aimSettings->away_disable_sound == true)
    no_sound->setChecked(true);
  // Connect button
  connect(Back, SIGNAL(clicked()), this, SLOT(back()));
	connect(Update, SIGNAL(clicked()), this, SLOT(updateClicked()));
}

/***************************************************************************
 * Deconstructor
 ***************************************************************************/
AwayMessage::~AwayMessage(){
}

/***************************************************************************
 * Create GUi 
 ***************************************************************************/
void  AwayMessage::initDialog(){
  this->resize(280,215);
  message= new QMultiLineEdit(this,"NoName");
  message->setWordWrap (QMultiLineEdit::WidgetWidth );
  message->setGeometry(10,60,260,110);
  message->setMinimumSize(0,0);

  Back= new QPushButton(this,"NoName");
  Back->setGeometry(170,180,100,25);
  Back->setMinimumSize(0,0);
  Back->setText("Back");

	Update= new QPushButton(this,"NoName");
  Update->setGeometry(110,180,60,25);
  Update->setMinimumSize(0,0);
  Update->setText("Update");
  Update->show();
	
	/*
  text1= new QLabel(this,"NoName");
  text1->setGeometry(10,10,260,20);
  text1->setMinimumSize(0,0);
  text1->setText(tr("The following message will be sent to anyone"));

  text2= new QLabel(this,"NoName");
  text2->setGeometry(10,30,230,20);
  text2->setMinimumSize(0,0);
  text2->setText(tr("that tries to IM you:"));
	*/

  no_sound= new QCheckBox(this,"NoName");
  no_sound->setGeometry(10,180,110,25);
  no_sound->setMinimumSize(0,0);
  no_sound->setText("Disable Sounds");
  no_sound->hide();

}

void AwayMessage::resizeEvent(QResizeEvent *){
	message->setGeometry(0, 10, this->width(), this->height()-60);
	Back->setGeometry(this->width()-50-5, this->height()-40, 50, 25);
	Update->setGeometry(5, this->height()-40, 55, 25);
	no_sound->setGeometry(10, 10 + this->height()-60 + 5, 110, 25);
}

/***************************************************************************
 * If AutoLogin is checked the other one is automaticly checked and disabled
 ***************************************************************************/
void AwayMessage::toggle(bool on){
  if (on == true){
    // Set sound to nosound
    oldSoundSettings = i_aimSettings->SoundMode;
    i_aimSettings->SoundMode = 2;
  }
  else{
    // Set sound mode back to old setting
    i_aimSettings->SoundMode = oldSoundSettings;
    // Save chage in settings
    i_aimSettings->away_disable_sound = false;
  }
}

/***************************************************************************
 * Sets the away message equal to the QString
 ***************************************************************************/
void AwayMessage::setText(QString text){
  message->clear();
  message->setText(text);
}

/***************************************************************************
 * Closes the window
 * Occurs when the Back Button is hit.
 ***************************************************************************/
void AwayMessage::back(){
  i_aimSettings->SoundMode = oldSoundSettings;
  emit backClicked();
}

/***************************************************************************
 * Closes the window
 * Occurs when the Update Button is hit.
 ***************************************************************************/
void AwayMessage::updateClicked(){
  emit update(message->text());
}


// awaymessage.cpp
#include "awaymessage.moc"
