/***************************************************************************
                             buddylistwindow.cpp
                             -------------------
    begin                : Sun Feb 13 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : ben@meyerhome.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "buddylistwindow.h"
#include "kaimapplication.h"

#define DEFAULT_AWAYWIDGET_HEIGHT 170

//These delays are in seconds!!
#define POST_BUDDY_SIGNOFF_DELAY 10
#define POST_BUDDY_SIGNON_DELAY 10
#define POST_AWAY_MESSAGE_SENT_DELAY 300

#define TWINDOW_AIM_SANITY_CHECK if(connection->getStatus() == TAIM_OFFLINE) {\
                                   QMessageBox::critical(0, QString("kaim"), QString("You are not online."));\
                                   return; }

/***************************************************************************
 * Constructor
 ***************************************************************************/
BuddyListWindow::BuddyListWindow(QWidget *parent, const char *name, settings *aimSettings, TAim *i_connection ) : QWidget(parent,name){
  viewAll = true;
	sendingPounce = true;
	autoResizing = false;
	i_aimSettings = aimSettings;
  connection = i_connection;
 	idleTimer = new QTimer(this, "idleTimer");
	connect(idleTimer, SIGNAL(timeout()), this, SLOT(setAutoAway()));
	idleTimer->stop();

  connect(connection, SIGNAL(userInfoReceived(QString)), this, SLOT(slotBuddyInfoRecieved(QString)));
  connect(connection, SIGNAL(IMReceived(QString, QString, bool)), this, SLOT(receivedIM(QString, QString, bool)));
  connect(connection, SIGNAL(buddyChanged(int)), this, SLOT(buddyChanged(int)));
  connect(connection, SIGNAL(userSignedOn(QString)), this, SLOT(buddySignedOn(QString)));
  connect(connection, SIGNAL(userNotAway(QString)), this, SLOT(checkPounceReturnAway(QString)));
  connect(connection, SIGNAL(userNotIdle(QString)), this, SLOT(checkPounceReturnIdle(QString)));	      
  // Create the menu
  _FileMenu = new QPopupMenu(this, "FileMenu");
  _ToolsMenu = new QPopupMenu(this, "ToolsMenu");
	_AwayMenu = new QPopupMenu(this, "AwayMenu");
  _ViewMenu = new QPopupMenu(this, "ViewMenu");
  _PounceMenu = new QPopupMenu(this, "PounceMenu");
  _PounceDelete = new QPopupMenu(this, "PounceDelete");
	connect(_PounceMenu, SIGNAL(aboutToShow()), this, SLOT(pounceDeleteMenuAboutToShow()));
  _BuddyMenu = new QPopupMenu(this, "Buddy");
	_FolderMenu = new QPopupMenu(this, "Folder");
  _HelpMenu = new QPopupMenu(this, "HelpMenu");
  _importMenu = new QPopupMenu(this, "ImportMenu");
  // The menu bar for the menus
  _menuBar= new QMenuBar(this, "NoName");
  initMenuBar();
  initWindow();
	connect(_AwayMenu, SIGNAL(activated(int)), this, SLOT(Away(int)));
  connect(treeList->buddyEdit, SIGNAL(returnPressed()), this, SLOT(changeBuddyName()));
  messageWindows.setAutoDelete(false);
	this->setMinimumSize(150,200);
	_TreeMenu = new QPopupMenu(this, "NoName");
}

/***************************************************************************
 * Deconstructor
 ***************************************************************************/
BuddyListWindow::~BuddyListWindow(){
}

/***************************************************************************
 * A buddy has been moved from one folder to another.  Move it. :)
 ***************************************************************************/
void BuddyListWindow::moveBuddy(QListViewItem* buddy, QListViewItem* folder){
	if (buddy != NULL && folder != NULL){
	  TBuddyList list = connection->buddyList();
		int folderNumber = list.getNumGroup(folder->text(0));
		list.setGroup(list.getNum(buddy->text(0)),folderNumber);
		connection->setBuddyList(list);
		loadList();
	}
}
			
/******************************************************************************
 * Init anything in the main window that would change. 
 ******************************************************************************/
void BuddyListWindow::initOptions(){
	if(awayRightNow == true){
		Back();
	}	
	if (i_aimSettings->main_noIcons == true){
	  chatButton->hide();
	  awayButton->hide();
		infoButton->hide();
	}
	else{
		chatButton->show();
	  awayButton->show();
		infoButton->show();
	}
  if (i_aimSettings->main_browser != "netscape"){
		browser->otherBrowser = i_aimSettings->main_browser;
		browser->setBrowser(2);
	}
	resetIdleTimer();  //this will take care of seeing if auto_away is enabled.

	emit(set_window(this->width(), this->height()+1));
  loadList();
}

/******************************************************************************
 * Close out the client.
 ******************************************************************************/
void BuddyListWindow::closeEvent(QCloseEvent *) {
  // Close window
  emit quit();
}

/***************************************************************************
 * init the global chat window settings.  Then set up them.
 ***************************************************************************/
void BuddyListWindow::initChatSettings(){
  emit( updateChatSettings(i_aimSettings->chatSettings) );
  emit( updateChatColorSettings(i_aimSettings->chatColorSettings) );
  emit( updateChatSoundSettings(i_aimSettings->chatSoundSettings) );
  emit( updateChatToolBarSettings(i_aimSettings->chatToolBarSettings) );
}

/***************************************************************************
 * init for the main objects, button, tree
 ***************************************************************************/
void BuddyListWindow::initWindow(){
  treeList= new KaimListView(this,"NoName");
  treeList->setGeometry(5,100,130,210);
  treeList->setFont(QFont("helvetica", 12, 50));
  treeList->addColumn("Screen Name", -1);
  //treeList->setBackgroundPixmap(QPixmap("/home/ben/background.gif"));
  connect( treeList, SIGNAL(moveItem(QListViewItem *, QListViewItem *)), this, SLOT(moveBuddy(QListViewItem *, QListViewItem *)));
  connect( treeList, SIGNAL(expanded (QListViewItem *)), this, SLOT(folderExpanded (QListViewItem *)));
  connect( treeList, SIGNAL(collapsed(QListViewItem *)), this, SLOT(folderCollapsed(QListViewItem *)));

	lastSelectedItem = 0;
  connect( treeList, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(TreeSelected(QListViewItem *)) );
  connect( treeList, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(TreeSelected(QListViewItem *)) );
  connect( treeList, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)), this, SLOT(clickedTree(QListViewItem *, const QPoint &, int)) );

  chatButton= new QPushButton(this,"NoName");
  chatButton->setGeometry(5,315,40,40);
  chatButton->setPixmap(Icon("chat_big"));
  QToolTip::add(chatButton,"Chat");

  infoButton= new QPushButton(this,"NoName");
  infoButton->setGeometry(95,315,40,40);
	infoButton->setPixmap(Icon("info_big"));
  QToolTip::add(infoButton,"Get Buddy Info");

  awayButton= new QPushButton(this,"NoName");
  awayButton->setGeometry(50,315,40,40);
  awayButton->setPixmap(Icon("away_big"));
  QToolTip::add(awayButton,"Away");
	
  
  connect(_PounceDelete, SIGNAL(activated(int)), this, SLOT(slotRemovePounce(int)));
  connect(_PounceMenu, SIGNAL(activated(int)), this, SLOT(editPounce(int)));
  // The Three Button Connections.
  connect(chatButton, SIGNAL(clicked()), this, SLOT(ButtonChatClicked()));
  connect(awayButton, SIGNAL(pressed()), this, SLOT(ButtonShowAwayList()));
  connect(infoButton, SIGNAL(clicked()), this, SLOT(slotGetBuddyInfo()));

	awayWidget = new AwayMessage(this, "AwayDialogBox", i_aimSettings);
	connect(awayWidget, SIGNAL(backClicked()), this, SLOT(Back()));	
	connect(awayWidget, SIGNAL(update(QString)), this, SLOT(updateAway(QString)));
	awayWidget->hide();
}

/***************************************************************************
 * Builds the menu bars
 ***************************************************************************/
void BuddyListWindow::initMenuBar(){
  _FileMenu->clear();
  
	_FileMenu->insertItem(Icon("buddy_oscar"), "&Buddy", _BuddyMenu);
    _BuddyMenu->clear();
		_BuddyMenu->insertItem(Icon("add"), "&Add Buddy...", this, SLOT(slotAddBuddy()));
		_BuddyMenu->insertItem(Icon("remove"), "&Remove Buddy...", this, SLOT(slotRemoveBuddy()));
    _BuddyMenu->insertItem(Icon("edit"), "&Edit Name", this, SLOT(slotEditBuddy()));
    _BuddyMenu->insertItem(Icon("chat_small"), "&New Instant Message", this, SLOT(ButtonChatClicked()));
	 	_BuddyMenu->insertItem(Icon("info"), "&Get Buddy Info", this, SLOT(slotGetBuddyInfo()));
	 	_BuddyMenu->insertItem(Icon("log"),"&Open Logs folder", this, SLOT(slotOpenLogs()));
	
	_FileMenu->insertItem(Icon("folder"), "&Folder", _FolderMenu);
	  _FolderMenu->clear();
    _FolderMenu->insertItem("&Add Folder", this, SLOT(slotAddFolder()));
    _FolderMenu->insertItem("&Remove Folder...", this, SLOT(slotRemoveFolder()));
    _FolderMenu->insertItem(Icon("edit"), "&Edit Name", this, SLOT(slotEditBuddy()));
						
	_FileMenu->insertItem(Icon("import"), "&Import", _importMenu);
    _importMenu->clear();
    _importMenu->insertItem(Icon("icon_gaim"), "&Gaim...", this, SLOT(slotImportFromGaim()));
    _importMenu->insertItem(Icon("icon_tik"), "&Tik...", this, SLOT(slotImportFromTik()));
    _importMenu->insertItem(Icon("kaim_small"), "&Kaim...", this, SLOT(slotImportFromKaim()));
  _FileMenu->insertSeparator();
  _FileMenu->insertItem(Icon("configure"),"&Options", this, SIGNAL(showOptions()), ALT+Key_O);
  _FileMenu->insertSeparator();
  _FileMenu->insertItem(Icon("buddy_offline"), "&Log Off", this, SIGNAL(logOff()), ALT+Key_L);
  _FileMenu->insertItem(Icon("exit"),"&Quit", this, SIGNAL(quit()), ALT+Key_Q);
  
	_ToolsMenu->insertItem(Icon("buddy_away"), "&Away", _AwayMenu);
    initAwayList();
  _ToolsMenu->insertItem(Icon("view"), "&View", _ViewMenu);
    initViewAllSetting();
  _ToolsMenu->insertItem(Icon("pounce"), "&Pounce", _PounceMenu);
    _PounceMenu->clear();
     _PounceMenu->insertItem("&Add pounce...", this, SLOT(slotAddPounceDialog()));
    _PounceMenu->insertItem("&Remove pounce", _PounceDelete);
    _PounceMenu->insertSeparator();
  
  _HelpMenu->clear();
  _HelpMenu->insertItem(Icon("gohome"), "Open Kaim Web Site", this, SLOT(MenuSlotOpenWebSite()));
  _HelpMenu->insertItem(Icon("check_for_update"), "Check For New Release", this, SIGNAL(checkForNewRelease()));
	_HelpMenu->insertSeparator();
  _HelpMenu->insertItem(Icon("kaim_small"), "&About...", this, SLOT(MenuSlotAbout()));
  
  _menuBar->clear();
  _menuBar->insertItem("&Program", _FileMenu);
  _menuBar->insertItem("&Tools", _ToolsMenu);
  _menuBar->insertItem("&Help", _HelpMenu);
}

/***************************************************************************
 * Loads the Awaylist under the list.
 ***************************************************************************/
void BuddyListWindow::initAwayList(){
  _AwayMenu->clear();
	QMap <QString, QString>::Iterator it;
	int count = 1;
	for(it = i_aimSettings->mapAwayMessages->begin(); 
			it != i_aimSettings->mapAwayMessages->end(); ++it){
		_AwayMenu->insertItem(it.key(), count++);
	}
	/*
  message *top = i_aimSettings->listOfMessages->TheMessages;
  int count = 1;
  while (top != NULL){
    _AwayMenu->insertItem(top->Name, count);
    connect(_AwayMenu, SIGNAL(activated(int)), this, SLOT(Away(int)));
    count++;
    top = top->next;
  }
	*/
  _AwayMenu->insertSeparator();
  _AwayMenu->insertItem("&New Message...", this, SLOT(slotAddAwayMessage()));
}

/***************************************************************************
 * Loads the ViewAll settings.
 ***************************************************************************/
void BuddyListWindow::initViewAllSetting(){
  _ViewMenu->clear();
  _ViewMenu->setCheckable(true);
  if (viewAll == true){
    viewAll = false;
    _ViewMenu->insertItem("&All Buddies", this, SLOT(ChangeViewAllSetting()),0,1);
  }
  else{
    viewAll = true;
    _ViewMenu->insertItem("&On-Line Buddies", this, SLOT(ChangeViewAllSetting()),0,1);
  }
  _ViewMenu->insertItem("&LastOn", this, SLOT(viewLastOn()),0,2);
  _ViewMenu->insertItem("&Idle", this, SLOT(viewIdle()),0,3);
  _ViewMenu->setItemChecked(2,i_aimSettings->main_showLastOn);
  _ViewMenu->setItemChecked(3,i_aimSettings->main_showIdle);
}


/***************************************************************************
 * Changes the ViewAll setting and updates the list
 ***************************************************************************/
void BuddyListWindow::ChangeViewAllSetting(){
  initViewAllSetting();
  loadList();
}

/***************************************************************************
 * Turn on/off the LastOn colume
 ***************************************************************************/
void BuddyListWindow::viewLastOn(){
  if (i_aimSettings->main_showLastOn == true)
    i_aimSettings->main_showLastOn = false;
  else
    i_aimSettings->main_showLastOn = true;
  _ViewMenu->setItemChecked(2,i_aimSettings->main_showLastOn);
  loadList();
}

/***************************************************************************
 * Turn on/off the idle colume
 ***************************************************************************/
void BuddyListWindow::viewIdle(){
  if (i_aimSettings->main_showIdle == true)
    i_aimSettings->main_showIdle = false;
  else
    i_aimSettings->main_showIdle = true;
  _ViewMenu->setItemChecked(3,i_aimSettings->main_showIdle);
  loadList();
}

/***************************************************************************
 * The function will parse through all the names and put them up on the Tree
 ***************************************************************************/
void BuddyListWindow::loadList() {
  treeList->clear();
  // remove all columns
  while((treeList->header())->count() > 1)
    treeList->removeColumn(1);
  
  if(connection->getStatus() == TAIM_OFFLINE) return;
  // we're online, so get going...
	
  // add columns as necessary, and headers
  if ((i_aimSettings->main_showIdle) == true)
    treeList->addColumn("Idle", -1);
  if ((i_aimSettings->main_showLastOn) == true)
    treeList->addColumn("Last On", -1);
  
  // get list to use...
  TBuddyList list = connection->buddyList();
  // Set the user list to be the same as
 	
  // Add the groups
  QString groupName = "";
  for (int i=0; i < list.getCountGroup(); i++) {
    groupName = list.getNameGroup(i);
    QListViewItem *groupItem = new QListViewItem (treeList, groupName);
    groupItem->setPixmap(0, Icon("folder"));
		
    // Add all the buddies in this group
    QString buddyName = "";
    for (int i2=0; i2 < list.getCount(); i2++) {
      TBuddy *tempBuddy = list.getByNum(i2);
      if (tempBuddy != NULL){
        // Go through the budies seeing if they are in the group.
        int buddyGroup = tempBuddy->group;
        if (buddyGroup == i){
          buddyName = list.getName(i2);
          TBuddy curBud;
	  list.get(&curBud, i2);
	  if (tempBuddy->status == TAIM_ONLINE || viewAll == true){
            BuddyListWindowItem *buddyItem = new BuddyListWindowItem(groupItem, QString(""));
						buddyItem->setText(0, buddyName);
	          loadTreeBuddy(tempBuddy);
          }
        }
      }
    }
    groupItem->setOpen(true);	
  }
  if (list.getCountGroup() == 0){
    list.addGroup(QString("Buddies"));
    // Update server & local list
    connection->setBuddyList(list);
    //connection->setServerBuddyList(list);
    viewAll = true;
    initViewAllSetting();  
    // Add the folder to the gui and bring up the edit box on it.
    QListViewItem *groupItem = new QListViewItem (treeList, "Buddies");
    groupItem->setPixmap(0, Icon("folder"));
    treeList->setCurrentItem(groupItem);
  }
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  autoResize();
}

/***************************************************************************
 * This function will fill the tree from the current buddy
 ***************************************************************************/
void BuddyListWindow::loadTreeBuddy(TBuddy *buddyInfo){
  QListViewItem *parent = treeList->firstChild();
  QListViewItem *item = 0;
  QListViewItem *item2 = 0;

  if(parent == 0)
    return;
  // Find the buddy in the tree
  while( parent != 0 ){
    item = parent->firstChild();
    while( item != 0 ){
      if( item->text(0) == buddyInfo->name ){
	item2 = item;
  	break;
      }
      item = item->nextSibling();
    }
    parent = parent->nextSibling();
  }
  
  
  if (item2 != 0 ){
  
    if (i_aimSettings->main_showIdle == true){
      int minute = buddyInfo->idleTime;
      int hour=0;
      while (minute > 59){
        hour++;
        minute = minute-60;
      }
			// Default of 0:0
      QString idleText = "";
			if (minute > 0)
			if (hour == 0){
			  // Check for 0 minutes
			  if (minute > 0){		
			   // There is only minutes
			   if (i_aimSettings->main_idleLayout == 1)
				   idleText = QString("0:%1").arg(minute);
			   else
	         idleText = QString("%1m").arg(minute);
			  }
			}
			else{
				// There must be hours so do both	
			  if (i_aimSettings->main_idleLayout == 1)
			    idleText = QString("%1:%2").arg(hour).arg(minute);
			  else
	       idleText = QString("%1h %2m").arg(hour).arg(minute);
			}
      if (treeList->columnText(1) == "Idle")
 	      item2->setText(1,idleText);
      if (treeList->columnText(2) == "Idle")
	      item2->setText(2,idleText);
    }
    
    if (i_aimSettings->main_showLastOn == true){
      QTime time(0,0,0,0);
      QString CurrentTime = "0";
  
      if (buddyInfo->status ==  TAIM_ONLINE){
        CurrentTime = "Now";
      }
      else  
      {
        QDate date(1970,1,1);
        QDateTime dateTime;
        dateTime.setTime_t( buddyInfo->lastOn );
        time = dateTime.time();
        //printf("date:%s\n", ((dateTime.date()).toString()).data());
        date = dateTime.date();
    
    
        if (time.minute() < 10)
          CurrentTime = QString("%1 %2:0%3").arg(date.dayName(date.dayOfWeek())).arg(time.hour()).arg(time.minute());
        else	
          CurrentTime = QString("%1 %2:%3").arg(date.dayName(date.dayOfWeek())).arg(time.hour()).arg(time.minute());
        if (buddyInfo->lastOn == 0){
          CurrentTime = "Unknown";
        }	
      }
      if (treeList->columnText(2) == "Last On")
        item2->setText(2,CurrentTime); 
      if (treeList->columnText(1) == "Last On")
        item2->setText(1,CurrentTime);
    }
  
    if (buddyInfo->status !=  TAIM_ONLINE){
			/*
			pm = createBuddyPixmapWithIcon(Icon("buddy_offline"),
						FONT_STYLE_ITALICS, Qt::darkGray, 
						colorGroup().base(), item2->text(HIDDEN_COLUMN) );
			*/
			((BuddyListWindowItem *)item2)->setDecoration(BuddyListWindowItem::Italics);
			item2->setPixmap(0, Icon("buddy_offline"));
      item2->setText(1,QString(""));
    }
    else{
      if (buddyInfo->signedOn == 1){
			  //item2->setPixmap(0, Icon("login_icon"));
				/*
				pm = createBuddyPixmapWithIcon(Icon("login_icon"),
							FONT_STYLE_BOLD, colorGroup().text(), colorGroup().base(), 
							item2->text(HIDDEN_COLUMN) );
				*/
				//item2->setText(HIDDEN_COLUMN, QString(""));
				item2->setPixmap(0, Icon("login_icon"));
				((BuddyListWindowItem *)item2)->setDecoration(BuddyListWindowItem::Bold);
				//delete pm;
      }
			else{
        switch(buddyInfo->userClass){
        case TOC_USER_ADMIN:
          item2->setPixmap(0, Icon("buddy_admin"));
          break;
        case TOC_USER_AOL:
          item2->setPixmap(0, Icon("buddy_aol"));
          break;
        case TOC_USER_UNCONFIRMED:
          item2->setPixmap(0, Icon("buddy_unknown"));
          break;
        case TOC_USER_NORMAL:
          item2->setPixmap(0, Icon("buddy_oscar"));
          break;
        case TOC_USER_AWAY:
          item2->setPixmap(0, Icon("buddy_away"));
          break;
        default:
          item2->setPixmap(0, Icon("buddy_unknown"));
          break;
        }
				((BuddyListWindowItem *)item2)->setDecoration(BuddyListWindowItem::Normal);
			}
    }
 	}
  else{
    // The buddy must have just signed on.
    loadList();
    if((i_aimSettings->SoundMode == 1) &&
       (i_aimSettings->sound_enable_login == true)){
      printf("\a");
      fflush(stdout);
      //printf("Buddy's name:%s\n",(buddyInfo->name).data() );
    }
    if((i_aimSettings->SoundMode == 0) &&
       (i_aimSettings->sound_enable_login == true)){
      system(QString("play ") + i_aimSettings->sound_file_login + " &");
    }
  }
}

/***************************************************************************
 * 5 minutes after a away message is sent the send away message again is
 * turned back on. 
 ***************************************************************************/
void BuddyListWindow::postAwayMessageSent( ){
  QString *bud;
  bud = awayMessageSent.first();

  if(!bud)
    return;
  
	QString normedName = tocNormalize(*bud);
  QString ChatWindowName = (i_aimSettings->login_name) + normedName;
  ChatWindow *targetWindow = messageWindows[normedName];
  if(targetWindow){
    targetWindow->sentAwayMessageToBuddy = false;
  }    

  if (awayMessageSent.isEmpty() != true){
		awayMessageSent.remove( awayMessageSent.begin() );
	}
	
	//dont leak memory:
  if(bud){
    delete bud;
  }
}

/***************************************************************************
 * Removes a buddy from the tree.  Called a little bit after they sign off.
 ***************************************************************************/
void BuddyListWindow::postBuddySignoff( ){
  QString *bud;
  bud = signedOff.first();

  if(!bud)
    return;

	// If the buddy is actually on then don't remove him.
	TBuddyList list = connection->buddyList();
  TBuddy *buddy = list.getByNum(list.getNum(*bud));
  // Only remove him if he is not online right now
	if (buddy->status != TAIM_ONLINE){
	  if(!viewAll){ //remove the item from view only if we're not in view-All mode
	    QListViewItemIterator it( treeList );
	    for ( ; it.current(); it++ ){
	      if( it.current()->text(0) == *bud ){
	        if(it.current()->parent()){
	          it.current()->parent()->removeItem( it.current() );
	        }	
	        break;
	      }
	    }
  	}
	}
	
  if (signedOff.isEmpty() != true){
    signedOff.remove( signedOff.begin() );
  }

  //dont leak memory:
  if(bud){
    delete bud;
  }
}

/***************************************************************************
 * Changes the icon of a buddy from just signed on to something else.
 ***************************************************************************/
void BuddyListWindow::postBuddySignon( ){
  QString *bud;
  bud = signedOn.first();

  if(!bud)
    return;
  
	TBuddyList list = connection->buddyList();
  TBuddy *buddy = list.getByNum(list.getNum(*bud));
  if (buddy){
		buddy->signedOn = 0;
	  loadTreeBuddy(buddy);
		connection->setBuddyList(list);
	}
	if (signedOn.isEmpty() != true){
    signedOn.remove( signedOn.begin() );
  }
 	
	//dont leak memory:
  if(bud){
    delete bud;
  }
}

/***************************************************************************
 * When a folder is expanded give it the open icon
 ***************************************************************************/
void BuddyListWindow::folderExpanded ( QListViewItem* item ){
  item->setPixmap(0, Icon("folder"));
} 

/***************************************************************************
 * When a folder is closed give it the closed icon
 ***************************************************************************/
void BuddyListWindow::folderCollapsed ( QListViewItem* item ){
  item->setPixmap(0, Icon("folder_closed"));
}

/***************************************************************************
 * The server will update a buddy
 ***************************************************************************/
void BuddyListWindow::buddyChanged(int number){
  TBuddyList list = connection->buddyList();
  TBuddy *bud = list.getByNum(number);
  loadTreeBuddy(bud);	
	/*
	QValueList<QString *>::Iterator it;
	QString *p = 0;

  // If signed on, make sure buddy is not in the signedOff list.  This would
  // happen if the buddy signed off, and signed back on _really_ fast before
  // his name got removed from the buddy list.
  if (bud->status == TAIM_ONLINE){
    for(it = signedOff.begin(); it != signedOff.end(); ++it){
      if( (*it)->compare(bud->name) == 0 ){
        p = *it;
        signedOff.remove( it );
        if(p){
          delete p;
        }
      break;
      }
    }
  }
	*/

  // If signed off tell chat window and play sound if needed
  if (bud->status !=  TAIM_ONLINE){
    if((i_aimSettings->SoundMode == 1) &&
       (i_aimSettings->sound_enable_logout == true)){
      printf("\a");
      fflush(stdout);
    }
    if((i_aimSettings->SoundMode == 0) &&
       (i_aimSettings->sound_enable_logout == true)){
      system(QString("play ") + i_aimSettings->sound_file_logout + " &");
    }
    QString normedName = tocNormalize(bud->name);
    QString ChatWindowName = (i_aimSettings->login_name) + normedName;
    ChatWindow *targetWindow = messageWindows[ ChatWindowName ];
    if(targetWindow)
      targetWindow->buddySignedOff();

    //add to the signedOff users list
    signedOff += new QString(bud->name);
    // Timer to remove item
    QTimer::singleShot( POST_BUDDY_SIGNOFF_DELAY *1000, this, SLOT(postBuddySignoff())); 
  }  
  autoResize();	
}

/***************************************************************************
 * Buddy Just signed on tell any chat window that need to know
 ***************************************************************************/
void BuddyListWindow::buddySignedOn(QString buddy){
  QString normedName = tocNormalize(buddy);
  QString ChatWindowName = (i_aimSettings->login_name) + normedName;
  ChatWindow *targetWindow = messageWindows[ ChatWindowName ];
  if(targetWindow)
    targetWindow->buddySignedOn();
  checkPounceSignOn(buddy);
	
	TBuddyList list = connection->buddyList();
  TBuddy *bud = list.getByNum(list.getNum(buddy));
  if (bud){
	  bud->signedOn = 1;
		if (bud->status == TAIM_ONLINE)
		  loadTreeBuddy(bud);
		signedOn += new QString(normedName);
		QTimer::singleShot( POST_BUDDY_SIGNON_DELAY *1000, this, SLOT(postBuddySignon()));
		connection->setBuddyList(list);
	}
} 

/***************************************************************************
 * Check what name is wanted to chat with
 ***************************************************************************/
void BuddyListWindow::ButtonChatClicked(){
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  QString name = "";
  QListViewItem  *cItem = treeList->currentItem();
  if ( cItem != NULL)
    name = cItem->text(0);
  if (CheckIfFolder(cItem) == true)
    name = "";
  ImConferm *confermName = new ImConferm(this, "IMConfirm", name);
  confermName->setCaption("Kaim - IM Buddy");
  connect(confermName, SIGNAL(ChatWithBuddyConfermed(QString)), this, SLOT(ChatWithBuddy(QString)));
  confermName->show();
}

/***************************************************************************
 * Checks if the item that is selected is a folder or not.
 * @Return bool if "selected" is a folder.
 ***************************************************************************/
bool BuddyListWindow::CheckIfFolder(QListViewItem* selected){
  QListViewItem *parent = treeList->firstChild();
  bool isFolder = false;
  if(selected == 0) return false;
  if(parent == 0) return true;
  while( parent != 0 ){
    if (parent == selected)
      isFolder = true;
    parent = parent->nextSibling();
  }
  return isFolder;
}

/***************************************************************************
 * Item in tree was selected/double clicked.  Open chat window
 ***************************************************************************/
void BuddyListWindow::DoubleClicked(){
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());
  TreeSelected(treeList->selectedItem());  
}

/***************************************************************************
 * Item in tree was selected/double clicked.  Open chat window
 ***************************************************************************/
void BuddyListWindow::TreeSelected(QListViewItem* selected){
  // If not a folder
  if (CheckIfFolder(selected) == false){
    const QString buddyName = selected->text(0);
    ChatWindow *targetWindow = messageWindow(buddyName);
    if (targetWindow != 0)
      targetWindow->show();
  }
}

/***************************************************************************
 * Open a chat window with buddyName
 ***************************************************************************/
void BuddyListWindow::ChatWithBuddy(QString buddyName){
  ChatWindow *targetWindow = messageWindow(buddyName);
  if (targetWindow){
    targetWindow->show();
  }
}

/***************************************************************************
 * A message came in. Open window and send massage
 ***************************************************************************/
void BuddyListWindow::receivedIM(QString name, QString message, bool isAuto){
  //printf("MmessageIn: %s\n", message.data());
  ChatWindow *targetWindow = messageWindow(name);
  targetWindow->show();
  // Slot autoin
  if (isAuto){
    message = QString("<B>Auto Response: </B>") + message;
  }
  targetWindow->messageIn(message);
  if (awayRightNow == true){
    if (targetWindow->sentAwayMessageToBuddy == false){
      targetWindow->awayMessage(awayMessage);
      sendAutoIM(awayMessage, name);
      targetWindow->sentAwayMessageToBuddy = true;
			
			awayMessageSent += new QString(name);
			QTimer::singleShot( POST_AWAY_MESSAGE_SENT_DELAY *1000, this, SLOT(postAwayMessageSent()));
    }
  }    
}

/***************************************************************************
 * Settings changed tell things to update
 ***************************************************************************/
void BuddyListWindow::settingsChanged(){
  initOptions();
	emit (updateChatSettings(i_aimSettings->chatSettings));
  emit (updateChatColorSettings(i_aimSettings->chatColorSettings));	
  //emit (updateChatToolBarSettings(i_aimSettings->chatToolBarSettings));
  emit (updateChatSoundSettings(i_aimSettings->chatSoundSettings));
  initAwayList();
}

/***************************************************************************
 * Toolbar Settings changed tell things to update
 ***************************************************************************/
void BuddyListWindow::chatToolbarUpdated(){
  emit (updateChatToolBarSettings(i_aimSettings->chatToolBarSettings));
}



/***************************************************************************
 * Get a Chat Window
 ***************************************************************************/
ChatWindow *BuddyListWindow::messageWindow(const QString &name){
  QString normedName = tocNormalize(name);
  QString ChatWindowName = (i_aimSettings->login_name) + normedName;
  ChatWindow *targetWindow = messageWindows[ ChatWindowName ];
  if(targetWindow == NULL){
    TBuddyList list = connection->buddyList();
    if (list.getNum(normedName) == -1){
      	targetWindow = new ChatWindow(i_aimSettings->login_name, normedName, false);
    
    }
    else
      targetWindow = new ChatWindow(i_aimSettings->login_name, normedName, true);
    messageWindows.insert(ChatWindowName, targetWindow);
    connect(targetWindow, SIGNAL(messageOut(QString, QString)), this, SLOT(sendIM(QString, QString)) );
		connect(targetWindow, SIGNAL(antiIdle()), this, SLOT(resetIdleTimer()));
    messageWindows.setAutoDelete(false);
    targetWindow->browser = browser;
		targetWindow->show();
    targetWindow->setChatSettings(i_aimSettings->chatSettings);
    targetWindow->setChatColorSettings(i_aimSettings->chatColorSettings);
    targetWindow->setChatSoundSettings(i_aimSettings->chatSoundSettings);
    targetWindow->setChatToolBarSettings(i_aimSettings->chatToolBarSettings);
    // Signals out from ChatWindow
    connect(targetWindow, SIGNAL(addBuddy(QString)), this, SLOT(slotAddBuddy(QString)));
    connect(targetWindow, SIGNAL(removeBuddy(QString)), this, SLOT(slotRemoveBuddy(QString)));
    connect(targetWindow, SIGNAL(blockBuddy(QString)), this, SLOT(slotBlockBuddy(QString)));
    connect(targetWindow, SIGNAL(warningOut(QString)), this, SLOT(slotWarnBuddy(QString)) );
    connect(targetWindow, SIGNAL(getInfo(QString)), this, SLOT(slotGetBuddyInfo(QString)) );
    connect(targetWindow, SIGNAL(openHomePage()), this, SLOT(MenuSlotOpenWebSite()));
    connect(targetWindow, SIGNAL(showAbout()), this, SLOT(MenuSlotAbout()));
    // Update Singles to Chat Window
    connect(this, SIGNAL(updateChatSettings(chat_Settings*)), targetWindow, SLOT(setChatSettings(chat_Settings*)));
    connect(this, SIGNAL(updateChatColorSettings(chat_Color_Settings*)), targetWindow, SLOT(setChatColorSettings(chat_Color_Settings*)));
    connect(this, SIGNAL(updateChatSoundSettings(chat_Sound_Settings*)), targetWindow, SLOT(setChatSoundSettings(chat_Sound_Settings*)));
    connect(this, SIGNAL(updateChatToolBarSettings(chat_ToolBar_Settings*)), targetWindow, SLOT(setChatToolBarSettings(chat_ToolBar_Settings*)));
    connect(this, SIGNAL(updateIfIsBuddy(QString,bool)), targetWindow, SLOT(setIfIsBuddy(QString, bool)));

    // Quit signal from Chat Window
    connect(targetWindow, SIGNAL(quiting(QString)), this, SLOT(RemoveChatWindow(QString)));
  }

  return targetWindow;
}

/***************************************************************************
 * When a buddy window dies it is removed
 ***************************************************************************/
void BuddyListWindow::RemoveChatWindow(QString name){
  QString normedName = tocNormalize(name);
  normedName = (i_aimSettings->login_name) + normedName;
  ChatWindow *targetWindow = NULL;
  targetWindow = messageWindows[ normedName ];
  if (targetWindow != NULL)
  {
    messageWindows.remove(normedName);
    delete targetWindow;
  }
}

/***************************************************************************
 * The away button was clicked show the list there.
 ***************************************************************************/
void BuddyListWindow::ButtonShowAwayList(){
  initAwayList();
  QPoint temp(this->parentWidget()->x()+awayButton->x()+20,this->parentWidget()->y()+awayButton->y()+50);
  _AwayMenu->exec(temp);
  awayButton->setDown(false);
}

/***************************************************************************
 * Add a buddy to the list
 ***************************************************************************/
void BuddyListWindow::slotAddBuddy(){
  //Make sure a folder exists and is selected
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  if(!treeList->selectedItem())
  {
    QMessageBox::warning( 0, "Add Buddy",
      "Please select a folder",
      "OK", /*"Quit",*/ 0,
      0, 1 );
    return;
  }
  QString name;
  bool ok = FALSE;
  QString text = QInputDialog::getText( tr( "Enter Buddy's Name" ), tr( "Please enter the buddy's name" ), QString::null, &ok, this );
  if ( ok && !text.isEmpty() )
    slotAddBuddy(tocNormalize(text));// user entered something and pressed ok
  else
    return	;// user entered nothing or pressed cancel
  emit (saveSettings());
}

/***************************************************************************
 * Add a buddy to the list (buddyName)
 ***************************************************************************/
void BuddyListWindow::slotAddBuddy(QString buddyName){
  TBuddyList list = connection->buddyList();
  int groupNumber = 0;
  QListViewItem *cItem = treeList->selectedItem();
  // If window is highlighted get group number.
  if ( cItem != NULL){
    if (CheckIfFolder(cItem) == true){
      groupNumber = list.getNumGroup(treeList->selectedItem()->text(0));
    }
    else{
      //get parent
      groupNumber = list.getNumGroup(((cItem->parent())->text(0)));
    }
  }	
      
  // Update server
  TBuddy *edited = new TBuddy();
  edited->name = buddyName;
  edited->group = groupNumber;
  edited->status = TAIM_OFFLINE;
  edited->userClass = 0;
  edited->signonTime = 0;
  edited->evil = 0;

  list.add(edited);
  connection->setBuddyList(list);
  loadList();
  delete edited;
  
  emit updateIfIsBuddy(buddyName, true);
}

/***************************************************************************
 * Removes a buddy from the list
 * Gets currently selected buddy and calls slotRemoveBuddy(name)
 ***************************************************************************/
void BuddyListWindow::slotRemoveBuddy(){
  QString name = treeList->currentItem()->text(0);
  slotRemoveBuddy(name);
  emit (saveSettings());
}

/******************************************************************************
 * Remove the buddy from the current list
 * @Peram name - Name of the buddy that is to be removed
 *****************************************************************************/
void BuddyListWindow::slotRemoveBuddy(QString name){
  int answer = QMessageBox::warning(0, "Remove Buddy",
    "Do you wish to remove this buddy from your buddylist?",
    "Yes", "Cancel", 0,1);
  if (!answer){
    TBuddyList list = connection->buddyList();
    list.del(list.getNum(name));
    connection->setBuddyList(list);
    //connection->setServerBuddyList(list);
    loadList();
    emit updateIfIsBuddy(name, false);
  }
}

/******************************************************************************
 * Send a warning message against the buddy currently selected in the buddylist
 ******************************************************************************/
void BuddyListWindow::slotWarnBuddy(){
  QString name = treeList->currentItem()->text(0);
  slotWarnBuddy(name);
}

/******************************************************************************
 * Prompt if you really want to warn "name"
 * Prompt if you want to do in anon or not
 * @Peram name - Name of the buddy that is to be Warned
 ******************************************************************************/
void BuddyListWindow::slotWarnBuddy(QString name){
  QString buddyName = "";
  int answer = QMessageBox::warning(0, "Warn",
    QString("Do you wish to warn ") + name + "?",
    "Yes", "Cancel", 0,1);
  if (!answer){
    int anon = QMessageBox::warning(this, "Message",
     "Do you wish to warn anonymously?",
     "Yes", "No", 0,1);
    if (!anon)
      slotSendWarning(name, true);
    else
      slotSendWarning(name, false);
  }	
}

/******************************************************************************
 * Send an IM warning against name
 * @Peram name - Name of the buddy
 * @Peram isAnonymouse - warn anonymouse or not.
 ******************************************************************************/
void BuddyListWindow::slotSendWarning(QString name, bool isAnonymous){
  TWINDOW_AIM_SANITY_CHECK;
  connection->sendWarning(name, isAnonymous);
}

/******************************************************************************
 * Block Current selected buddy from being on my list.
 *****************************************************************************/
void BuddyListWindow::slotBlockBuddy(){
  QString name = treeList->currentItem()->text(0);
  slotWarnBuddy(name);
}

/******************************************************************************
 * block buddy "name"
 * @Peram name - Name of the buddy that is to be Blocked
 *****************************************************************************/
void BuddyListWindow::slotBlockBuddy(QString name){
  QString buddyName = "";
  int answer = QMessageBox::warning(0, "Block Buddy",
    "Do you wish to block this buddy from your buddylist?",
    "Yes", "Cancel", 0,1);
  if (!answer){
    TBuddyList list = connection->denyList();
    TBuddy *edited = new TBuddy();
    edited->name = name;
    edited->group = 0;
    edited->status = TAIM_OFFLINE;
    edited->userClass = 0;
    edited->signonTime = 0;
    edited->evil = 0;
    list.add(edited);
    delete edited;
    connection->setDenyList(list);
    //connection->setServerBuddyList(list);
    //loadList();
    if(connection->permissions() != 4)
      connection->setPermissions(4);
  }	
}

/***************************************************************************
 * Get information button clicked, get current highlighted name and get info
 ***************************************************************************/
void BuddyListWindow::slotGetBuddyInfo(){
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  const QString name = treeList->currentItem()->text(0);
  if (CheckIfFolder(treeList->currentItem()) == false){
    slotGetBuddyInfo( name );
  }
  else{
    bool ok = FALSE;
    QString text = QInputDialog::getText( tr( "Enter Buddy's Name" ), tr( "Please enter the buddy's name" ), QString::null, &ok, this );
    if ( ok && !text.isEmpty() )
      slotGetBuddyInfo( text );
  }
}

/***************************************************************************
 * Get Information on a user
 ***************************************************************************/
void BuddyListWindow::slotGetBuddyInfo(QString name){
  TWINDOW_AIM_SANITY_CHECK;
  if(awayRightNow == true)
    Back();
  connection->requestUserInfo( name );
}

/***************************************************************************
 * Info has been recieved
 * @Peram string - The info that was recieved
 ***************************************************************************/
void BuddyListWindow::slotBuddyInfoRecieved(QString string){
  Info *infoBox = new Info(0, "infoBox");
  infoBox->addToWindow(string);
  infoBox->setCaption("Kaim - info");
  infoBox->setIcon(Icon("kaim_small"));
  infoBox->browser = browser;
	infoBox->show();
}

/***************************************************************************
 * Add a new folder to the list.
 ***************************************************************************/
void BuddyListWindow::slotAddFolder(){
  // Get current list
  TBuddyList list = connection->buddyList();
  list.addGroup(QString("NewFolder"));
  // Update server list
  connection->setBuddyList(list);
  //connection->setServerBuddyList(list);
  viewAll = true;
  initViewAllSetting();  
  // Add the folder to the gui and bring up the edit box on it.
  QListViewItem *groupItem = new QListViewItem (treeList, "NewFolder");
  groupItem->setPixmap(0, Icon("folder"));
  treeList->setCurrentItem(groupItem);
  slotEditBuddy();
  treeList->setSorting(0,1);
  treeList->sort();
  emit (saveSettings());
}

/***************************************************************************
 * Remove the current selected folder
 ***************************************************************************/
void BuddyListWindow::slotRemoveFolder(){
  int answer = QMessageBox::warning(this, "Remove Folder",
    "Do you wish to remove this folder from your buddylist?\nAll buddies in the folder will be lost.",
    "Yes", "Cancel", 0,1);
  if (!answer){
    TBuddyList list = connection->buddyList();
    list.delGroup(list.getNumGroup(treeList->currentItem()->text(0)));
    connection->setBuddyList(list);
    //connection->setServerBuddyList(list);
    loadList();
  }  
  emit (saveSettings());
}

/***************************************************************************
 * Bring up the edit box on the currently selected user.
 ***************************************************************************/
void BuddyListWindow::slotEditBuddy(){
  QString name = "";
  QListViewItem *cItem = treeList->currentItem();
  if ( cItem != NULL){
   	name = cItem->text(0);
	}

  treeList->buddyEdit->setText("");
  treeList->buddyEdit->setText(name);

  //int location = treeList->currentItem()->depth();
  int xx = 16*2 + 3;
  int yy = treeList->currentItem()->itemPos();
  if (CheckIfFolder(treeList->currentItem()) == true){
    xx = xx -19;
  }
  treeList->buddyEdit->setGeometry(xx,yy,(treeList->columnWidth(0)-xx),18);
  treeList->moveChild(treeList->buddyEdit,xx,yy);
	treeList->buddyEdit->setMinimumSize((treeList->columnWidth(0)-xx),18);
  treeList->buddyEdit->setMaximumSize((treeList->columnWidth(0)-xx),18);
		
  // Showing it will give it focus
  treeList->buddyEdit->focus = 0;
  treeList->buddyEdit->show();
  treeList->buddyEdit->setFocus();
  treeList->buddyEdit->setSelection(0,treeList->buddyEdit->maxLength());
}

/***************************************************************************
 * Once the edit window is gone update the name
 ***************************************************************************/
void BuddyListWindow::changeBuddyName(){
  QListViewItem *cItem = treeList->currentItem();
  QString oldName = "";
  if ( cItem != NULL){
   	oldName = cItem->text(0);
	}

  QString newName = treeList->buddyEdit->text();

  if ( cItem != NULL){
   	cItem->setText(0, newName);
	}

  // Update server
  TBuddyList list = connection->buddyList();
  if (CheckIfFolder(treeList->currentItem()) == true)
    list.renameGroup( oldName, newName);
  else{
    TBuddy *edited = list.getByNum(list.getNum(oldName));
    edited->name = newName;
		//loadTreeBuddy(edited);
		loadList();
  }
  connection->setBuddyList(list);
  //connection->setServerBuddyList(list);

  treeList->buddyEdit->hide();
  emit (saveSettings());
}

// MEW LAND

/***************************************************************************
 * Send an auto IM message
 ***************************************************************************/
void BuddyListWindow::sendAutoIM(QString target, QString message){
  TWINDOW_AIM_SANITY_CHECK;
  connection->sendIM(target, message, true, true);
}

/***************************************************************************
 * Send an IM message
 ***************************************************************************/
void BuddyListWindow::sendIM(QString target, QString message){
  TWINDOW_AIM_SANITY_CHECK;
  if (sendingPounce == false){
	  if(awayRightNow == true)
      Back();
    connection->sendIM(target, message, false, true);
  }
	else{
    connection->sendIM(target, message, false, false);
 	}
			
}

/***************************************************************************
 * The auto away timer has signaled, set user as away.
 ***************************************************************************/
void BuddyListWindow::setAutoAway(){
	//set the autoAway to the correct away message as specified by the user
	Away(i_aimSettings->away_auto_default_message + 1); //+1 offset, necessary.
}

/***************************************************************************
 * Reset the idle timer because the user just did something.
 ***************************************************************************/
void BuddyListWindow::resetIdleTimer(){
	if (i_aimSettings->away_enable_auto == true){
		idleTimer->changeInterval(i_aimSettings->away_auto_delay_minutes * 60000);
	} else {
		idleTimer->stop();
	}
}

/***************************************************************************
 * Add a new away message.
 ***************************************************************************/
void BuddyListWindow::slotAddAwayMessage(){
//  QMessageBox::information(this, "Away Messages","Use program/options to add away\nmessages for the time being.", QMessageBox::Ok);
  NewAwayMessage *newMessage = new NewAwayMessage(this, "newAwayMessage");
  newMessage->setCaption("Kaim - New Away Message");
  newMessage->i_aimSettings = i_aimSettings;
  connect(newMessage, SIGNAL(makeAwayNow(QString)), this, SLOT(makeAwayNow(QString)));
  newMessage->show();
}

/***************************************************************************
 * Import buddy list from a gaim buddy list.
 ***************************************************************************/
void BuddyListWindow::slotImportFromGaim(){
  QString buddyFile = getenv("HOME");
  buddyFile += "/.gaim";
  QString s( QFileDialog::getOpenFileName(buddyFile,QString("*.blist"),this, "Gaim Buddy List (*.blist)") );
  if ( s.isEmpty() == true)
  return;
  importConfigAlias(s);
}

/***************************************************************************
 * Import buddy list from a kaim buddy list.
 ***************************************************************************/
void BuddyListWindow::slotImportFromKaim(){
  QString buddyFile = getenv("HOME");
  buddyFile += "/.kaim";
  QString s( QFileDialog::getOpenFileName(buddyFile,QString("*.buddyList"),this, "Kaim Buddy List (*.buddyList)") );
  if ( s.isEmpty() == true)
    return;
  importConfig(s);
}

/***************************************************************************
 * Import buddy list from a tik buddy list.
 ***************************************************************************/
void BuddyListWindow::slotImportFromTik(){
  QString buddyFile = getenv("HOME");
  buddyFile += "/.tic";
  QString s( QFileDialog::getOpenFileName(buddyFile,QString("*.config"),this, "Tik Buddy List (*.config)") );
  if ( s.isEmpty() == true)
    return;
  importConfig(s);
}

/***************************************************************************
 * Import buddy list from a buddy list.
 ***************************************************************************/
void BuddyListWindow::importConfig(QString fileName){
  // Got a file, open and parse.
  QFile file(fileName);
  QString config;
  if (file.open(IO_ReadWrite)){
    config += "CONFIG:";
    QTextStream stream(&file);
    while(!stream.eof()){
      config += stream.readLine();
      config += "\n";
    }
    file.close();
  }
  else
    QMessageBox::information(this, "Error","Unable to open file.", QMessageBox::Ok);
  // Got config from file, parse out lists 
  TBuddyList buddyList;
  TBuddyList permitList;
  TBuddyList denyList;
  int permitStatus;
  tocParseConfig(config, &buddyList, &permitList, &denyList, &permitStatus);
  
  // Add lists
  TBuddyList connectionBuddyList = connection->buddyList(); 
  TBuddyList connectionPermitList = connection->permitList(); 
  TBuddyList connectionDenyList = connection->denyList(); 
  
  connection->setBuddyList((connectionBuddyList + buddyList));
  connection->setPermitList((connectionPermitList + permitList));
  connection->setDenyList((connectionDenyList + denyList));
  emit ( saveSettings());
}

/***************************************************************************
 * Import buddy list from a buddy list.
 ***************************************************************************/
void BuddyListWindow::importConfigAlias(QString fileName){
  // Got a file, open and parse.
  QFile file(fileName);
  QString config;
  if (file.open(IO_ReadWrite)){
    config += "CONFIG:";
    QTextStream stream(&file);
    while(!stream.eof()){
      config += stream.readLine();
      config += "\n";
    }
    file.close();
  }
  else
    QMessageBox::information(this, "Error","Unable to open file.", QMessageBox::Ok);
  // Got config from file, parse out lists 
  TBuddyList buddyList;
  TBuddyList permitList;
  TBuddyList denyList;
  int permitStatus;
  tocParseConfigAlias(config, &buddyList, &permitList, &denyList, &permitStatus);
  
  // Add lists
  TBuddyList connectionBuddyList = connection->buddyList(); 
  TBuddyList connectionPermitList = connection->permitList(); 
  TBuddyList connectionDenyList = connection->denyList(); 
  
  connection->setBuddyList((connectionBuddyList + buddyList));
  connection->setPermitList((connectionPermitList + permitList));
  connection->setDenyList((connectionDenyList + denyList));
  emit ( saveSettings());
}


/***************************************************************************
 * The tree was clicked with the mouse, open right click menu if needed 
 ***************************************************************************/
void BuddyListWindow::clickedTree(QListViewItem *, const QPoint &point, int){ 
  openBuddyListMenu(point);
}

/***************************************************************************
 * Find out what number name is in the list and call away(int)
 ***************************************************************************/
void BuddyListWindow::makeAwayNow(QString name){
  int count = 1;
  int choice = -1;
	QMap <QString, QString>::Iterator it;
	for(it = i_aimSettings->mapAwayMessages->begin(); 
			it != i_aimSettings->mapAwayMessages->end(); ++it){
		if(it.key() == name){
			choice = count;
		}
		count++;
	}
  if (choice != -1){
    Away(choice);
	}
}

/***************************************************************************
 * Set the away message to text
 ***************************************************************************/
void BuddyListWindow::updateAway(QString text){
  //connection->setAwayMessage(text);
	awayMessage = text;
}

/***************************************************************************
 * Set Kaim to away mode.
 ***************************************************************************/
void BuddyListWindow::Away(int num){
	QMap <QString, QString>::Iterator it;
	int count = 1;
	for(it = i_aimSettings->mapAwayMessages->begin(); 
			it != i_aimSettings->mapAwayMessages->end(); ++it){
		if(count == num){
			awayWidget->setText(it.data());
			awayMessage = it.data();
			if( awayRightNow == false){
				QString message = it.data();	
				if (message.length() > 1000){
				  message = message.mid(0, 950);
				  message = message + "... Im me for the rest of this away message.";
				}
				connection->setAwayMessage(message);
			}
			break;
		} else {
			count++;
		}
	}
	if ((num != 0) && (num == count)){
    awayWidget->show();
		if (awayRightNow == false){
			awayRightNow = true;
	    emit(set_window(this->width(), this->height() + DEFAULT_AWAYWIDGET_HEIGHT) );
    }	
	}
	idleTimer->stop();	//we dont need the timer if away.
}

/***************************************************************************
 * Set away = false.  Kaim autoresponds.
 ***************************************************************************/
void BuddyListWindow::Back(){
	awayRightNow = false;
	awayWidget->hide();
  connection->setAwayMessage("");
	emit(set_window(this->width(), this->height() - DEFAULT_AWAYWIDGET_HEIGHT) );
	this->resizeEvent(0);

	resetIdleTimer();
}

/***************************************************************************
 * Opens the right menu if a treeList is selected.
 ***************************************************************************/
void BuddyListWindow::openBuddyListMenu(QPoint pos){
  _TreeMenu->clear();
  if(treeList->currentItem() != NULL)	{
    QString buddyName = treeList->currentItem()->text(0);
    // Check for folder
    QListViewItem *selected = treeList->currentItem();
    if (CheckIfFolder(selected) == true){
      _TreeMenu->insertItem(Icon("add"), "&Add Buddy", this, SLOT(slotAddBuddy()));
      _TreeMenu->insertSeparator();
			_TreeMenu->insertItem("Add &Folder", this, SLOT(slotAddFolder()));
			_TreeMenu->insertItem("&Remove Folder...", this, SLOT(slotRemoveFolder()));
			_TreeMenu->insertItem(Icon("edit"), "&Edit Folder", this, SLOT(slotEditBuddy()));
    }
    else {
      _TreeMenu->insertItem(Icon("chat_small"), "&New Instant Message", this, SLOT(DoubleClicked()));
      _TreeMenu->insertItem(Icon("info"), "&Get Buddy Info", this, SLOT(slotGetBuddyInfo()));
      _TreeMenu->insertItem(Icon("remove"), "&Remove Buddy...", this, SLOT(slotRemoveBuddy()));
      _TreeMenu->insertItem(Icon("log"), "&Open Logs Folder", this, SLOT(slotOpenLogs()));
      _TreeMenu->insertItem(Icon("pounce"),"&Pounce...", this, SLOT(slotAddPounceDialogWithBuddy()));
			_TreeMenu->insertItem(Icon("edit"), "&Edit Buddy", this, SLOT(slotEditBuddy()));
    }
  }	
  _TreeMenu->insertSeparator();
  _TreeMenu->insertItem(Icon("view"), "&View", _ViewMenu);
  _TreeMenu->exec(pos);
}


/***************************************************************************
 * Open buddies logs in netscape
 ***************************************************************************/
void BuddyListWindow::slotOpenLogs(){
  QString buddy = "";
  QListViewItem *cItem = treeList->currentItem();
  if ( cItem != NULL)
    buddy = cItem->text(0);
  
  QDir newDir("/dev/null");
  // Make sure the dir is there
  newDir.mkdir(QString(getenv("HOME")) + "/." + QString(APP_LOWERCASE_NAME), true);
  newDir.mkdir(QString(getenv("HOME")) + "/." + QString(APP_LOWERCASE_NAME) + "/logs/", true);
  newDir.mkdir(QString(getenv("HOME")) + "/." + QString(APP_LOWERCASE_NAME) + "/logs/" + buddy, true);
  
  browser->openURL(QString(getenv("HOME")) + "/." + APP_LOWERCASE_NAME + "/logs/" + buddy);
}

/***************************************************************************
 * About - Displays the about box
 ***************************************************************************/
void BuddyListWindow::MenuSlotAbout(){
  QMessageBox *about= new QMessageBox(0, "About Kaim");
  about->setCaption("About Kaim...");
  about->setText(
  (QString)"Kaim " + i_aimSettings->Version + " (" + i_aimSettings->ReleaseDate +")\n"
  "(c) 2000 Benjamin Meyer\n"
  "ben@meyerhome.net\n"
  "\n"
  "   Kaim is a AOL instant messenger for QT2.2.\n"
  "If you have any suggestions/comments/bugs \n"
  "please contact the Kaim team.\n"
  "\n"
  "For more information you can visit the Kaim\n"
  "webpage at http://kaim.sourceforge.net/.\n"
	"\n"
	"This program is under the GPL Licence\n"
  "For more information view the licence.");
  about->setIconPixmap(Icon("kaim"));
  //about->setIcon(QMessageBox::Information);
  about->setIcon(Icon("kaim_small"));
  about->show();
}

/***************************************************************************
 * Using the current broswer (selected in options) opens the kaim web site.
 ***************************************************************************/
void BuddyListWindow::MenuSlotOpenWebSite(){
  system(i_aimSettings->main_browser + QString( " ") + "http://kaim.sourceforge.net/ &");
}

/***************************************************************************
 * Brings up the Menu when the right mouse is clicked.
 ***************************************************************************/
void BuddyListWindow::mouseReleaseEvent(QMouseEvent* event){
  if (event->button() == RightButton){
    _AwayMenu->hide();
  }
}

/***************************************************************************
 * Open a pounce dialog for a new pounce to be added.
 ***************************************************************************/
void BuddyListWindow::slotAddPounceDialog(){
  NewPounceImp *pounceDialog = new NewPounceImp(this, "AddPounceDialog", "");
  connect(pounceDialog, SIGNAL(addPounce(pounce)), this, SLOT(slotAddPounce(pounce)));
  TBuddyList list = connection->buddyList();
  int numberOfBuddies = list.getCount();
	for (int i = 0; i < numberOfBuddies; i++){
		pounceDialog->buddyName->insertItem(list.getName(i), -1);
	}
  pounceDialog->show();
}

/***************************************************************************
 * Bring up the pounce dialog with the current selected buddy name 
 ***************************************************************************/
void BuddyListWindow::slotAddPounceDialogWithBuddy(){
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  const QString name = treeList->currentItem()->text(0);
  if (CheckIfFolder(treeList->currentItem()) == false){
    NewPounceImp *pounceDialog = new NewPounceImp(this, "AddPounceDialog", name);
    connect(pounceDialog, SIGNAL(addPounce(pounce)), this, SLOT(slotAddPounce(pounce)));
 
		TBuddyList list = connection->buddyList();
    int numberOfBuddies = list.getCount();
		for (int i = 0; i < numberOfBuddies; i++){
		  if (list.getName(i) != name)
				pounceDialog->buddyName->insertItem(list.getName(i), -1);
		}
	
		pounceDialog->show();
	}
}

/***************************************************************************
 * Bring up the pounce dialog with the current pounce info. 
 ***************************************************************************/
void BuddyListWindow::editPounce(int menuId){
  QString buddyName = _PounceDelete->text(menuId-101);
  if (!buddyName.isNull()){ //user didnt press Add Pounce, or Remove Pounce
    buddyName = tocNormalize(buddyName);
    pounce *pounceStruct = pounceMessages[ buddyName ];
    if(pounceStruct){
      NewPounceImp *pounceDialog = new NewPounceImp(this, "AddPounceDialog", "");
      connect(pounceDialog, SIGNAL(addPounce(pounce)), this, SLOT(slotAddPounce(pounce)));
      pounceDialog->setPounce(pounceStruct);
      pounceDialog->show();
    }
    else
     QMessageBox::critical(this, "Error",QString("Can't find pounce for: ") + buddyName, "Ok");
  }
}

/***************************************************************************
 * Add a pounce to the internal list of pounces.
 ***************************************************************************/
void BuddyListWindow::slotAddPounce(pounce newPounce){
  QString BuddyName = newPounce.buddyName;
  BuddyName = tocNormalize(BuddyName);
	pounce *newPounceStruct = pounceMessages[ BuddyName ];
  if(!newPounceStruct){
    newPounceStruct = new pounce;
    pounceMessages.insert(BuddyName, newPounceStruct);
    
    newPounceStruct->buddyName = BuddyName;
    newPounceStruct->signOn = newPounce.signOn;
    newPounceStruct->returnAway = newPounce.returnAway;
    newPounceStruct->returnIdle = newPounce.returnIdle;
    newPounceStruct->openChat = newPounce.openChat;
    newPounceStruct->sendMessage = newPounce.sendMessage;
    newPounceStruct->message = newPounce.message;
    newPounceStruct->execCommand = newPounce.execCommand;
    newPounceStruct->command = newPounce.command;
    newPounceStruct->doNotRemove = newPounce.doNotRemove;
    int id = _PounceDelete->insertItem(newPounce.buddyName);
		newPounceStruct->menuId = id + 101;
		if (newPounce.execCommand && newPounce.openChat){
		  _PounceMenu->insertItem(BuddyName + ": Chat & Exec Command",newPounceStruct->menuId);
		}
		if (newPounce.execCommand){
		  QString command =  newPounce.command;
      if (command.length() > 20){
        command = command.mid(0,16);
        command += "...";
      }
      if (command.length() > 0)
        _PounceMenu->insertItem(BuddyName + ": " + command,newPounceStruct->menuId);
			else
				_PounceMenu->insertItem(BuddyName + ": Exec Command",newPounceStruct->menuId);
		}
		if (newPounce.openChat){
			QString message =  newPounce.message;
      if (message.length() > 20){
        message = message.mid(0,16);
        message += "...";
      }
		  if (message.length() > 0)
        _PounceMenu->insertItem(BuddyName + ": " + message,newPounceStruct->menuId);
      else
        _PounceMenu->insertItem(BuddyName + ": Open Chat Window",newPounceStruct->menuId);	
    }
	}
  else{
    newPounceStruct->signOn = newPounce.signOn;
    newPounceStruct->returnAway = newPounce.returnAway;
    newPounceStruct->returnIdle = newPounce.returnIdle;
    newPounceStruct->openChat = newPounce.openChat;
    newPounceStruct->sendMessage = newPounce.sendMessage;
    newPounceStruct->message = newPounce.message;
		newPounceStruct->execCommand = newPounce.execCommand;
    newPounceStruct->command = newPounce.command;
    newPounceStruct->doNotRemove = newPounce.doNotRemove;
  }
}

/***************************************************************************
 * If the Pounce delete menu is empty, this inserts a disabled item "(Empty)"
 ***************************************************************************/
void BuddyListWindow::pounceDeleteMenuAboutToShow(){
	int id = 999;
	QMenuItem *item = _PounceDelete->findItem( id );
	
	if(_PounceDelete->count() < 1  && !item){
		_PounceDelete->insertItem("(Empty)", id);
		_PounceDelete->setItemEnabled(id, false);
		_PounceMenu->insertItem("(Empty)", id);
		_PounceMenu->setItemEnabled(id, false);
	} 
	else {
		if (item)	{
		  if (_PounceDelete->count() != 1){
				_PounceDelete->removeItem(id);
		    _PounceMenu->removeItem(id);
			}
	  }
	}
}

/***************************************************************************
 * Check to see if the buddy buddyName has a pounce for signOn and if so
 * pounce that buddy.
 ***************************************************************************/
void BuddyListWindow::checkPounceSignOn(QString buddyName){
  buddyName = tocNormalize(buddyName);
  pounce *pounceStruct = pounceMessages[ buddyName ];
  if(pounceStruct){
    if (pounceStruct->signOn == true){
      activatePounce(buddyName);
    }
  }
}

/***************************************************************************
 * Check to see if the buddy buddyName has a pounce for returnAway and if so
 * pounce that buddy.
 ***************************************************************************/
void BuddyListWindow::checkPounceReturnAway(QString buddyName){
	buddyName = tocNormalize(buddyName);
	pounce *pounceStruct = pounceMessages[ buddyName ];
  if(pounceStruct){
    if (pounceStruct->returnAway == true){
      activatePounce(buddyName); 
    }
  }
}

/***************************************************************************
 * Check to see if the buddy buddyName has a pounce for returnIdle and if so
 * pounce that buddy.
 ***************************************************************************/
void BuddyListWindow::checkPounceReturnIdle(QString buddyName){
  buddyName = tocNormalize(buddyName);
  pounce *pounceStruct = pounceMessages[ buddyName ];
  if(pounceStruct){
    if (pounceStruct->returnIdle == true){
      activatePounce(buddyName);
    }  
  }
}

/***************************************************************************
 * When this function is called it will attempt to do whatever the pounce
 * for buddy buddyName wants.
 ***************************************************************************/
void BuddyListWindow::activatePounce(QString buddyName){
	pounce *pounceStruct = pounceMessages[ buddyName ];
  if(pounceStruct){
    if (pounceStruct->openChat == true){
      ChatWithBuddy(buddyName);
    }
    if (pounceStruct->openChat == false && pounceStruct->sendMessage == true){
      connection->sendIM(pounceStruct->message,buddyName, false, false);
    }
    if (pounceStruct->openChat == true && pounceStruct->sendMessage == true){
      ChatWindow *targetWindow = messageWindows[(i_aimSettings->login_name) + buddyName ];
      if(targetWindow){
        sendingPounce = true;
				targetWindow->pounceMessage(pounceStruct->message);
				sendingPounce = false;
      }
    }
		if (pounceStruct->execCommand == true){
		  system(QString(pounceStruct->command) + " &");
		}
    // Find the menu id number and then remove the menu item.
    if (pounceStruct->doNotRemove == false){
		  if (pounceStruct->menuId != -1){
			  slotRemovePounce((pounceStruct->menuId)-101);
			}
		}
  }
}

/***************************************************************************
 *  If you remove a pounce from the delete list it is removed here.
 ***************************************************************************/
void BuddyListWindow::slotRemovePounce(int pounceMenuNumber){
  QString buddyName = _PounceDelete->text((pounceMenuNumber));
	pounce *pounceStruct = pounceMessages[ buddyName ];
  _PounceDelete->removeItem(pounceMenuNumber);
  _PounceMenu->removeItem(pounceMenuNumber+101);
	if (pounceStruct != NULL){
    pounceMessages.remove(buddyName);
    delete pounceStruct;
  }
}

/***************************************************************************
 * Resizes the object in the window, when the window is resized.
 ***************************************************************************/
void BuddyListWindow::resizeEvent ( QResizeEvent* ) {
  int mid = this->width()/2;
  			
	if(awayRightNow == false){
    if (i_aimSettings->main_noIcons == true){
       treeList->setGeometry (3, 25, this->width()-6, this->height()-28);	
    }
    else{
      treeList->setGeometry (5, 25, this->width()-10, this->height()-75);
      chatButton->setGeometry (mid-65, this->height()-45, 40, 40);
      awayButton->setGeometry (mid-20, this->height()-45, 40, 40);
      infoButton->setGeometry (mid+25, this->height()-45, 40, 40);
    }
  }
  else { //awayRightNow
    if (i_aimSettings->main_noIcons == true){
      treeList->setGeometry (3, 25, this->width()-6, this->height()-28-DEFAULT_AWAYWIDGET_HEIGHT);	
      awayWidget->setGeometry(3, this->height()-DEFAULT_AWAYWIDGET_HEIGHT, treeList->width(), DEFAULT_AWAYWIDGET_HEIGHT);
    }
    else{
     treeList->setGeometry (5, 25, this->width()-10, (this->height()-75-DEFAULT_AWAYWIDGET_HEIGHT));
     chatButton->setGeometry (mid-65, this->height()-45-DEFAULT_AWAYWIDGET_HEIGHT, 40, 40);
     awayButton->setGeometry (mid-20, this->height()-45-DEFAULT_AWAYWIDGET_HEIGHT, 40, 40);
     infoButton->setGeometry (mid+25, this->height()-45-DEFAULT_AWAYWIDGET_HEIGHT, 40, 40);
     awayWidget->setGeometry (5, this->height()-DEFAULT_AWAYWIDGET_HEIGHT, treeList->width(), DEFAULT_AWAYWIDGET_HEIGHT);
    }
  }
}


/***************************************************************************
 * Resizes the object in the window.
 ***************************************************************************/
void BuddyListWindow::autoResize(){
  if (autoResizing == true)
			return;
	
	autoResizing = true;
	// If it is off then we are still in this loop and exit.
	if (treeList->vScrollBarMode() == QScrollView::AlwaysOff ||
      treeList->vScrollBarMode() == QScrollView::AlwaysOff ){
	  return;
	}	
		
	if (treeList->vScrollBarMode() == QScrollView::Auto &&
      treeList->vScrollBarMode() == QScrollView::Auto ){
		treeList->setVScrollBarMode(QScrollView::AlwaysOff);
 	  treeList->setHScrollBarMode(QScrollView::AlwaysOff);
    qApp->processEvents();
	}
	else
		return;
	
	bool greaterThenMaxHeight = false;
	
	if (i_aimSettings->main_resizeHeight == true){
    int newHeight;
		if (treeList->contentsWidth() != treeList->visibleWidth()){     
		  //newHeight += treeList->horizontalScrollBar()->height();
		}
		newHeight = this->height()+(treeList->contentsHeight() - treeList->visibleHeight());
    int diff =  newHeight-this->height();
		if (diff > -10){
				//printf("diff: %i\n", newHeight-this->height());
				if (newHeight >	i_aimSettings->main_resizeHeightMin && newHeight < i_aimSettings->main_resizeHeightMax)	
        emit(set_window(this->width(), newHeight+6));
				else
						greaterThenMaxHeight = true;
    }
  }
  
  if (i_aimSettings->main_resizeWidth == true){
    int width = 10;
    //if (treeList->contentsHeight() != treeList->visibleHeight()){     
		if (greaterThenMaxHeight == true){
		  width += 15; //treeList->verticalScrollBar()->width();
		}
		//}
    int numberOfHeaders = (treeList->header())->count();
    for (int i = numberOfHeaders; i > 0; i--)
      width += treeList->columnWidth(i-1);
	  // Add frame boarder
		width += treeList->	lineWidth()*2;
    if ( width != treeList->width() )
		  if (width >	i_aimSettings->main_resizeWidthMin && width < i_aimSettings->main_resizeWidthMax)
        emit(set_window(width, this->height()));
  }
	
  if (treeList->vScrollBarMode() != QScrollView::Auto &&
      treeList->vScrollBarMode() != QScrollView::Auto ){
    treeList->setVScrollBarMode(QScrollView::Auto);
 	  treeList->setHScrollBarMode(QScrollView::Auto);
	}
	autoResizing = false;
}

/**
	* BuddyListWindowItem.  Derived from QListViewItem so we can get some more
	* functionality out of it.  Dimi Shahbaz
	*/

BuddyListWindowItem::BuddyListWindowItem(QListView *parent, QString label1)
	: QListViewItem (parent, label1) {

}

BuddyListWindowItem::BuddyListWindowItem(QListViewItem *parent, QString label1)
	: QListViewItem (parent, label1) {

}

BuddyListWindowItem::~BuddyListWindowItem(){

}

void BuddyListWindowItem::setDecoration(int d){
	mDecoration = d;
}

int BuddyListWindowItem::decoration(){
	return mDecoration;
}

void BuddyListWindowItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align){
	if(column != 0) //we only want to deal with the 1st column (index = 0)
		return QListViewItem::paintCell(p, cg, column, width, align);

	if(mDecoration == Normal) //no need to bother with the default view
		return QListViewItem::paintCell(p, cg, column, width, align);

	if(!p)
		return;
    
	const QPixmap *icon = pixmap(column);
	int iconWidth = icon ? icon->width() : 0;
	int iconHeight = icon ? icon->height() : height();
	int stringWidthPx;

	QFont f = QFont::defaultFont();
	if(mDecoration == Italics)
		f.setItalic(TRUE);
	else if(mDecoration == Bold)
		f.setBold(TRUE);
	else { //(mDecoration == Normal)
		qDebug("mDecoration == Normal.  We shouldn't have gotten here!\n");
		return;
	}
	
	QFontMetrics fm(f);
	stringWidthPx = fm.width(text(column));

	QString name = text(column);
	QString tmpStr = name;
	QString ellipsis = "...";
	int i = 1;
	while ( fm.width( tmpStr ) + iconWidth > width){ 
		tmpStr = name;
		tmpStr.truncate( tmpStr.length() - i);
		if(tmpStr.length() == 1){ //always leave the last character there, like Qt does
			tmpStr += "...";
			break;
		}
		tmpStr += "...";
		i++;
	}
	name = tmpStr;

	int r = (icon ? iconWidth : 1) + listView()->itemMargin();
	if(!this->isSelected()){ //draw dark text on light background
		p->fillRect( 0, 0, width, height(), cg.brush( QColorGroup::Base ));

		if(!name.isEmpty()){
			if(mDecoration == Italics){
				int r, g, b;
				cg.text().rgb(&r, &g, &b);
				p->setPen( QColor( r+128 < 255 ? r+128 : 255,
														g+128 < 255 ? r+128 : 255,
														b+128 < 255 ? r+128 : 255));			//lighter than text() color
			}
			else {//mDecoration == Bold
				p->setPen( cg.text() );			//normal text color
			}

			p->setFont( f );
			p->drawText( r, iconHeight-2, name );
		}
	} else{ //this->isSelected() //draw light text on dark background
		p->fillRect( 0, 0, width, height(), cg.brush( QColorGroup::Highlight ));

		if(!name.isEmpty()){
			p->setPen( cg.highlightedText() );
			p->setFont( f );
			p->drawText( r, iconHeight-2, name );
		}
	}

	if(icon){
		p->drawPixmap( 1, (height()-iconHeight)/2, *icon);
		//r += icon->width() + listView()->itemMargin();
	}

}



// BuddyListWindow
#include "buddylistwindow.moc"

