/***************************************************************************
                              buddylistwindow.h
                             -------------------
    begin                : Sun Feb 13 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 * The main window that displays the buddies.  Edit/IM etc.
 ***************************************************************************/

#ifndef BUDDYLISTWINDOW_H
#define BUDDYLISTWINDOW_H

#include <qwidget.h>
#include <qheader.h>
#include <kaimviewlist.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qdict.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qlineedit.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qinputdialog.h>

#include "chatwindow.h"
#include "info.h"
#include "setup.h"
#include "awaymessage.h"
#include "imconfirm.h"
#include "kaimdialogs.h"
#include "newawaymessage.h"
#include "tbuddylist.h"
#include "taim.h"
#include "newpounceimp.h"
#include "browser.h"

/**
  * @author Benjamin Meyer
  */

class BuddyListWindowItem; 

class BuddyListWindow : public QWidget  {
  Q_OBJECT

  signals:
    void quit();
    void logOff();
    void showOptions();
    void set_window(int, int);
    void saveSettings();
    void checkForNewRelease();
		
    void updateIfIsBuddy(QString, bool);
    void updateChatSettings(chat_Settings *chatSettings);
    void updateChatColorSettings(chat_Color_Settings *chatColorSettings);
    void updateChatSoundSettings(chat_Sound_Settings *chatSoundSettings);
    void updateChatToolBarSettings(chat_ToolBar_Settings *chatToolBarSettings);

  public:
    BuddyListWindow(QWidget *parent=0, const char *name=0, settings *aimSettings=0, TAim *i_connection=0 );
    ~BuddyListWindow();
    void resizeEvent ( QResizeEvent *e );
		bool awayRightNow;
    QValueList<QString *> signedOff;
    QValueList<QString *> signedOn;
    QValueList<QString *> awayMessageSent;
    Browser *browser;
		
  private:
    // internal vars
    settings *i_aimSettings;
    TAim *connection;
    bool viewAll;
		bool autoResizing;
	  bool sendingPounce;	
		QTimer *idleTimer;
		QListViewItem *lastSelectedItem;
	 	
    // Global chat window setttings
    chat_Settings *chatSettings;
    chat_Color_Settings *chatColorSettings;
    chat_Sound_Settings *chatSoundSettings;
    chat_ToolBar_Settings *chatToolBarSettings;

    // ChatWindow handling
    ChatWindow *messageWindow(const QString &name);
    QDict<ChatWindow> messageWindows;
    QDict<pounce> pounceMessages;
    
    // Away status
    QString awayMessage;
		
    // internal functions
    void initWindow();
    void initMenuBar();
    void initAwayList();
    void initViewAllSetting();
    void loadTreeBuddy(TBuddy *buddyInfo);
    void importConfig(QString fileName);
    void importConfigAlias(QString fileName);
    void autoResize();
	 	
    QPopupMenu *_FileMenu;
    QPopupMenu *_ToolsMenu;
		QPopupMenu *_ViewMenu;
    QPopupMenu *_PounceMenu;
    QPopupMenu *_PounceDelete;
    QPopupMenu *_importMenu;
    QPopupMenu *_AwayMenu;
    QPopupMenu *_BuddyMenu;
		QPopupMenu *_FolderMenu;
    QPopupMenu *_HelpMenu;
    QMenuBar *_menuBar;

    QPopupMenu *_TreeMenu; 
		
		//OLD CRAP**********************
    void loadSettings();
      
  protected:
    void mouseReleaseEvent(QMouseEvent* event);
    void closeEvent(QCloseEvent *);
		QPixmap *createBuddyPixmapWithIcon(QPixmap, int, QColor, QColor, QString);
		
    KaimListView *treeList;
    QPushButton *chatButton;
    QPushButton *infoButton;
    QPushButton *awayButton;
    
		AwayMessage *awayWidget;

  public slots:
    void clickedTree(QListViewItem *item, const QPoint &point, int);
    void sendIM(QString target, QString message);
    void sendAutoIM(QString target, QString message);
    void buddyChanged(int number);
    void settingsChanged();
    void chatToolbarUpdated();
    void initOptions();
    
		// ***** OLD CRAP
    //void Back();
    void loadList();
    void receivedIM(QString name, QString message, bool isAuto);
    void initChatSettings();

  private slots:
    void Back();
		void postBuddySignoff();
    void postBuddySignon(); 
    void postAwayMessageSent();
		void moveBuddy(QListViewItem* buddy, QListViewItem* folder);
				
    void Away(int);
    void makeAwayNow(QString name);
    void updateAway(QString text);
		void resetIdleTimer();
		void setAutoAway();

    void MenuSlotAbout();
    void MenuSlotOpenWebSite();
		void ButtonChatClicked();
    void ButtonShowAwayList();
    void ChatWithBuddy(QString buddyName);
    void RemoveChatWindow(QString name);

    void DoubleClicked();
    void TreeSelected(QListViewItem* selected);
    bool CheckIfFolder(QListViewItem* selected);
    void slotEditBuddy();
    void changeBuddyName();
    void buddySignedOn(QString buddy); 
    
    void viewLastOn();
    void viewIdle();
    
    void slotRemoveBuddy(QString name);
    void slotWarnBuddy();
    void slotWarnBuddy(QString name);
    void slotSendWarning(QString name, bool isAnonymous);
    void slotBlockBuddy();
    void slotBlockBuddy(QString name);
    void slotGetBuddyInfo();
    void slotGetBuddyInfo(QString name);
    void slotBuddyInfoRecieved(QString);

    void slotAddAwayMessage();
    void slotAddBuddy();
    void slotAddBuddy(QString buddy);
    void slotRemoveBuddy();
    void slotAddFolder();
    void slotRemoveFolder();
    void slotOpenLogs();
    
    void slotImportFromGaim();
    void slotImportFromKaim();
    void slotImportFromTik();

    void folderExpanded(QListViewItem*item);
    void folderCollapsed(QListViewItem*item);

    void slotAddPounceDialog();
    void slotAddPounceDialogWithBuddy();
    void slotAddPounce(pounce newPounce);
    void checkPounceSignOn(QString buddyName);
    void checkPounceReturnAway(QString buddyName);
    void checkPounceReturnIdle(QString buddyName);
    void editPounce(int menuId);
    void activatePounce(QString buddyName);
		void pounceDeleteMenuAboutToShow();
    void slotRemovePounce(int pounceMenuNumber);

    // ***** OLD CRAP
    void ChangeViewAllSetting();
    void openBuddyListMenu(QPoint pos);
};

class BuddyListWindowItem : public QListViewItem {
	public:
		BuddyListWindowItem(QListView *, QString);
		BuddyListWindowItem(QListViewItem *, QString);
		~BuddyListWindowItem();
		enum Decoration { Normal, Bold, Italics } ;

		void setDecoration(int);
		int decoration();

		void paintCell(QPainter *, const QColorGroup &, int , int , int );
	protected:
		int mDecoration;

};


#endif

// buddylistwindow.h
