/******************************************************************************
                                chatwindow.cpp
                             -------------------
    begin                : Mon May 8 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : ben@meyerhome.net
 ******************************************************************************/

/******************************************************************************
 *                                                                            *
 *   This program is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or        *
 *   (at your option) any later version.                                      *
 *                                                                            *
 ******************************************************************************/

#include "chatwindow.h"
#include "kaimapplication.h"
#include "linkdialog.h"
#include "GPGInterface.h"

/******************************************************************************
 * Constructor
 ******************************************************************************/
ChatWindow::ChatWindow(QString userName, QString buddysName, bool buddy, QWidget *parent): QMainWindow(parent,buddysName){
  myName = userName;
  buddyName = buddysName;
  isBuddy = buddy;
  
  sentAwayMessageToBuddy = false; 
  shiftIsPressed = false;
  fontSize = 3;
  flashNumber = false;
  cleared = false;
  temp_BuddyBgColor.setRgb(255,255,255);
  temp_BuddyTextColor.setRgb(0,0,0);
  lastMessage = "";
  currentMessageOut = "";
  previewBox = NULL;
  
  this->setCaption(buddyName + QString(" - Kaim"));

  chatSettings = new chat_Settings();
  chatSettings->chat_raise_window = false;		
  chatSettings->chat_flash_on_new_message = false;
  chatSettings->chat_log = false;
  chatSettings->chat_log_inHtml = false;
  chatSettings->chat_show_time = false;
  chatSettings->chat_log_time = true;
  chatSettings->chat_return_send = true;				
  chatSettings->chat_sreturn_send = true;			
  chatSettings->chat_send_blank = true;
  chatSettings->chat_ignore_buddyhtml = false;
  chatSettings->chat_line_limit = false;
  chatSettings->chat_line_limit_number = 50;
  chatSettings->chat_log_singleFile = false;
  chatSettings->chat_crypt = false;
  chatSettings->chat_smallest_font_size = 3;
  
  toolBarSettings = new chat_ToolBar_Settings();
  toolBarSettings->view_text = true;
  toolBarSettings->view_chat = true;
  toolBarSettings->view_prev = true;

  toolBarSettings->loc_text = 0;
  toolBarSettings->loc_chat = 0;
  toolBarSettings->loc_prev = 2;

  toolBarSettings->text_bold = true;
  toolBarSettings->text_italic = true;
  toolBarSettings->text_underline = true;

  toolBarSettings->text_fontdown = true;
  toolBarSettings->text_fontnormal = true;
  toolBarSettings->text_fontbig = true;

  toolBarSettings->text_bgcolor = true;
  toolBarSettings->text_pencolor = true;
  toolBarSettings->text_link = true;

  toolBarSettings->chat_clear = true;
  toolBarSettings->chat_save = true;
  toolBarSettings->chat_print = true;
  toolBarSettings->chat_time = true;

  toolBarSettings->chat_log = true;
  toolBarSettings->chat_sound = true;

  toolBarSettings->chat_remove = true;
  toolBarSettings->chat_warn = true;
  toolBarSettings->chat_block = true;
  toolBarSettings->chat_info = true;

  toolBarSettings->prev_prev = true;
  toolBarSettings->prev_send = true;

  colorSettings = new chat_Color_Settings();
  colorSettings->chat_yourText_color.setRgb(0,0,0);
  colorSettings->chat_yourName_color.setRgb(0,60,128);
  colorSettings->chat_buddyText_color.setRgb(0,0,0);
  colorSettings->chat_buddyName_color.setRgb(128,0,0);
  colorSettings->chat_yourbg_color.setRgb(255,255,255);
  colorSettings->chat_buddybg_color.setRgb(255,255,255);
  
  soundSettings = new chat_Sound_Settings();
  soundSettings->chat_sound = false;
  soundSettings->chat_sound_beepInitial = true;
  soundSettings->chat_sound_beepReceive = false;
  soundSettings->chat_sound_beepSend = false;
  soundSettings->chat_sound_playFile = false;
  soundSettings->chat_sound_fileSend = DATADIR "/sounds/Send.au";
  soundSettings->chat_sound_fileReceive = DATADIR "/sounds/Receive.au";
  soundSettings->chat_sound_fileInitial = DATADIR "/sounds/Receive.au";
  
  fileMenu=new QPopupMenu(this, "FileMenu");
  editMenu=new QPopupMenu(this, "EditMenu");
  viewMenu=new QPopupMenu(this, "ViewMenu");
  buddyMenu = new QPopupMenu(this, "BuddyMenu");
  insertMenu = new QPopupMenu(this, "InsertMenu");
  helpMenu = new QPopupMenu(this, "HelpMenu");
  
	// MENUBAR CONFIGURATION
  // set menuBar() the current menuBar
  menuBar()->clear();
  menuBar()->insertItem("&File", fileMenu);
  menuBar()->insertItem("&Edit", editMenu);
  menuBar()->insertItem("&View", viewMenu);
  menuBar()->insertItem("&Insert", insertMenu);
  menuBar()->insertItem("&Buddy", buddyMenu);
  menuBar()->insertItem("&Help", helpMenu);
  
  initMenuBar();
  // Create Tool Bars
  fileToolbar = new QToolBar(this, "file operations");
  textToolbar = new QToolBar(this, "text operations");
  previewToolbar = new QToolBar(this, "preview operations");
  previewToolbar->setMinimumSize(150,20);
  //fileToolbar->setHorizontalStretchable (true);
  //textToolbar->setHorizontalStretchable (true);
  previewToolbar->setHorizontalStretchable (true);
  // Fill Tool bars
  
  initToolBar();
  initChatWindow();
  this->setIcon(Icon("kaim_small"));
  this->resize(330,300);
  // Paint preview for current setup
  paintPreviewWidget();
  inputWindow->setFocus();
}

/******************************************************************************
 * Deconstructor
 ******************************************************************************/
ChatWindow::~ChatWindow(){
  if (chatSettings->chat_log == true){
    // End log file and close it.
    if (chatSettings->chat_log_inHtml == true)
      logStream << "</BODY>\n</HTML>";
    logFile.close();
  }
  if (chatSettings)
    delete (chatSettings);
  if (soundSettings)
    delete (soundSettings);
  if (colorSettings)
    delete (colorSettings);
  if(toolBarSettings)
    delete (toolBarSettings);
}

/******************************************************************************
 * Load the new settings
 ******************************************************************************/
void ChatWindow::setChatSettings(chat_Settings *newsettings){
  if (chatSettings == newsettings)
    return;  
  chatSettings->chat_raise_window = newsettings->chat_raise_window;		
  chatSettings->chat_flash_on_new_message = newsettings->chat_flash_on_new_message;
  chatSettings->chat_log_singleFile = newsettings->chat_log_singleFile;
	chatSettings->chat_log_inHtml = newsettings->chat_log_inHtml;
  chatSettings->chat_log_time = newsettings->chat_log_time;
  
	//chatSettings->chat_log = newsettings->chat_log;
  if (newsettings->chat_log == true && fileMenu->isItemChecked(ID_FILE_LOG) == false)
    slotChatLog(); 
  
  chatSettings->chat_show_time = newsettings->chat_show_time;
  chatSettings->chat_return_send = newsettings->chat_return_send;				
  chatSettings->chat_sreturn_send = newsettings->chat_sreturn_send;			
  chatSettings->chat_send_blank = newsettings->chat_send_blank;
  chatSettings->chat_ignore_buddyhtml = newsettings->chat_ignore_buddyhtml;
  chatSettings->chat_line_limit = newsettings->chat_line_limit;
  chatSettings->chat_line_limit_number = newsettings->chat_line_limit_number;
  
  chatSettings->chat_crypt = newsettings->chat_crypt;
  chatSettings->chat_smallest_font_size = newsettings->chat_smallest_font_size;

}

/******************************************************************************
 * Load the toolbar with the new settings
 ******************************************************************************/
void ChatWindow::setChatToolBarSettings(chat_ToolBar_Settings *newsettings){
  if (newsettings == toolBarSettings)
    return;  
    
  toolBarSettings->view_text = newsettings->view_text;
  toolBarSettings->view_chat = newsettings->view_chat;
  toolBarSettings->view_prev = newsettings->view_prev;
  // View Text
  if (newsettings->view_text == true) {
    textToolbar->show();
    viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, true);
  }
  else{
    textToolbar->hide();
    viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, false);
  }
  // View Chat
  if (newsettings->view_chat == true) {
    fileToolbar->show();
    viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, true);
  }
  else{
    fileToolbar->hide();
    viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, false);
  }
  // View Preview
  if (newsettings->view_prev == true) {
    previewToolbar->show();
    viewMenu->setItemChecked(ID_VIEW_PREVIEWTOOLBAR, true);
  }
  else{
    previewToolbar->hide();
    viewMenu->setItemChecked(ID_VIEW_PREVIEWTOOLBAR, false);
  }

  toolBarSettings->loc_text = newsettings->loc_text;
  toolBarSettings->loc_chat = newsettings->loc_chat;
  toolBarSettings->loc_prev = newsettings->loc_prev;
  
  toolBarSettings->text_bold = newsettings->text_bold;
  toolBarSettings->text_italic = newsettings->text_italic;
  toolBarSettings->text_underline = newsettings->text_underline;

  toolBarSettings->text_fontdown = newsettings->text_fontdown;
  toolBarSettings->text_fontnormal = newsettings->text_fontnormal;
  toolBarSettings->text_fontbig = newsettings->text_fontbig;

  toolBarSettings->text_bgcolor = newsettings->text_bgcolor;
  toolBarSettings->text_pencolor = newsettings->text_pencolor;
  toolBarSettings->text_link = newsettings->text_link;

  toolBarSettings->chat_clear = newsettings->chat_clear;
  toolBarSettings->chat_save = newsettings->chat_save;
  toolBarSettings->chat_print = newsettings->chat_print;
  toolBarSettings->chat_time = newsettings->chat_time;

  toolBarSettings->chat_log = newsettings->chat_log;
  toolBarSettings->chat_sound = newsettings->chat_sound;

  toolBarSettings->chat_remove = newsettings->chat_remove;
  toolBarSettings->chat_warn = newsettings->chat_warn;
  toolBarSettings->chat_block = newsettings->chat_block;
  toolBarSettings->chat_info = newsettings->chat_info;

  toolBarSettings->prev_prev = newsettings->prev_prev;
  toolBarSettings->prev_send = newsettings->prev_send;

  initToolBar();
  // scroll to the current text
  chatWindow->ensureVisible(0, chatWindow->contentsHeight());
}

/******************************************************************************
 * Load the new settings
 ******************************************************************************/
void ChatWindow::setChatColorSettings(chat_Color_Settings *newsettings){
  if (colorSettings == newsettings)
    return;  
  colorSettings->chat_yourText_color = newsettings->chat_yourText_color;
  colorSettings->chat_yourName_color = newsettings->chat_yourName_color;
  colorSettings->chat_buddyText_color = newsettings->chat_buddyText_color;
  colorSettings->chat_buddyName_color = newsettings->chat_buddyName_color;
  colorSettings->chat_yourbg_color = newsettings->chat_yourbg_color;
  colorSettings->chat_buddybg_color = newsettings->chat_buddybg_color;
}

/******************************************************************************
 * Load the new settings
 ******************************************************************************/
void ChatWindow::setChatSoundSettings(chat_Sound_Settings *newsettings){
  if (newsettings == soundSettings)
    return;  
  //Check if the button needs to be updated.
  if (soundSettings->chat_sound != newsettings->chat_sound)
    slotChatSound();
  soundSettings->chat_sound = newsettings->chat_sound;
  soundSettings->chat_sound_beepInitial = newsettings->chat_sound_beepInitial;
  soundSettings->chat_sound_beepReceive = newsettings->chat_sound_beepReceive;
  soundSettings->chat_sound_beepSend = newsettings->chat_sound_beepSend;
  soundSettings->chat_sound_playFile = newsettings->chat_sound_playFile;
  soundSettings->chat_sound_fileSend = newsettings->chat_sound_fileSend;
  soundSettings->chat_sound_fileReceive = newsettings->chat_sound_fileReceive;
  soundSettings->chat_sound_fileInitial = newsettings->chat_sound_fileInitial;
}

/******************************************************************************
 * Sets if the buddy is on the buddy list of not. 
 ******************************************************************************/
void ChatWindow::setIfIsBuddy(QString name, bool isABuddy){
  if (name != buddyName)
    return;  
  isBuddy = isABuddy;  
  initToolBar(); 
  initMenuBar();
}

/******************************************************************************
 * MenuBar setup
 ******************************************************************************/
void ChatWindow::initMenuBar(){
  // menuBar entry fileMenu
  fileMenu->clear();
  fileMenu->insertItem(Icon("log"), "&Log", this, SLOT(slotChatLog()), CTRL+Key_L, ID_FILE_LOG);
  fileMenu->insertItem(Icon("speaker"), "&Sound", this, SLOT(slotChatSound()), 0, ID_FILE_SOUND);
  fileMenu->insertItem(Icon("clear"), "&Clear", this, SLOT(slotChatClear()), CTRL+Key_D, ID_FILE_CLEAR);
  fileMenu->insertItem(Icon("save"), "&Save as...", this, SLOT(slotChatSaveAs()), 0,ID_FILE_SAVE_AS);
  fileMenu->insertSeparator();
  fileMenu->insertItem(Icon("printer"), "&Print...", this, SLOT(slotChatPrint()), CTRL+Key_P, ID_FILE_PRINT);
  fileMenu->insertSeparator();
  fileMenu->insertItem(Icon("close"), "Close", this, SLOT(slotChatClose()), CTRL+Key_Q, ID_FILE_CLOSE);

  if (chatSettings->chat_log == true)
    fileMenu->setItemChecked(ID_FILE_LOG, true);
  else
    fileMenu->setItemChecked(ID_FILE_LOG, false);

  if (soundSettings->chat_sound == true)
    fileMenu->setItemChecked(ID_FILE_SOUND, true);
  else
    fileMenu->setItemChecked(ID_FILE_SOUND, false);

  // menuBar entry editMenu
  editMenu->clear();
  editMenu->insertItem(Icon("undo"), "U&ndo", this, SLOT(slotEditUndo()), CTRL+Key_N, ID_EDIT_UNDO);
  editMenu->insertItem(Icon("redo"), "R&edo", this, SLOT(slotEditRedo()), CTRL+Key_E, ID_EDIT_REDO);
  editMenu->insertSeparator();
  editMenu->insertItem(Icon("cut"), "Cu&t", this, SLOT(slotEditCut()), CTRL+Key_X, ID_EDIT_CUT);
  editMenu->insertItem(Icon("copy"), "&Copy", this, SLOT(slotEditCopy()), CTRL+Key_C, ID_EDIT_COPY);
  editMenu->insertItem(Icon("paste"), "&Paste", this, SLOT(slotEditPaste()), CTRL+Key_V, ID_EDIT_PASTE);
  editMenu->insertSeparator();
  editMenu->insertItem("&Select All", this, SLOT(slotEditSelectAll()), CTRL+Key_A, ID_EDIT_SELECT_ALL);

  editMenu->setItemEnabled ( ID_EDIT_UNDO, false );
  editMenu->setItemEnabled ( ID_EDIT_REDO, false );

  // menuBar entry viewMenu
  viewMenu->clear();
  viewMenu->setCheckable(true);
  viewMenu->insertItem(Icon("time"), "&Time Stamp", this, SLOT(slotViewTimeStamp()), Key_F12, ID_VIEW_TIME);
  viewMenu->insertSeparator();
  viewMenu->insertItem("&Chat Toolbar", this, SLOT(slotViewChatToolBar()), 0, ID_VIEW_CHATTOOLBAR);
  viewMenu->insertItem("Text Tool&bar", this, SLOT(slotViewTextToolBar()), 0, ID_VIEW_TEXTTOOLBAR);
  viewMenu->insertItem("&Preview Toolbar", this, SLOT(slotViewPreviewToolBar()), 0, ID_VIEW_PREVIEWTOOLBAR);

  viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, true);
  viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, true);
  viewMenu->setItemChecked(ID_VIEW_PREVIEWTOOLBAR, true);

  if (toolBarSettings->chat_time == true)
    viewMenu->setItemChecked(ID_VIEW_TIME, true);
  else
    viewMenu->setItemChecked(ID_VIEW_TIME, false);
  
	insertMenu->clear();
  insertMenu->insertItem(Icon("bold"), "&Bold", this, SLOT(slotInsertBold()), CTRL+Key_B, ID_INSERT_BOLD);
  insertMenu->insertItem(Icon("italic"), "&Italic", this, SLOT(slotInsertItalic()), CTRL+Key_I, ID_INSERT_ITALIC);
  insertMenu->insertItem(Icon("underline"), "&Underline", this, SLOT(slotInsertUnderline()), CTRL+Key_U, ID_INSERT_UNDERLINE);
  insertMenu->insertSeparator();
  insertMenu->insertItem(Icon("small"), "S&hrink Font", this, SLOT(slotInsertShrinkFont()), 0, ID_INSERT_FONTSMALL);
  insertMenu->insertItem(Icon("normal"), "&Normal Font", this, SLOT(slotInsertNormalFont()), 0, ID_INSERT_FONTNORM);
  insertMenu->insertItem(Icon("big"), "In&crease Font", this, SLOT(slotInsertIncreaseFont()), 0, ID_INSERT_FONTBIG);
  insertMenu->insertSeparator();
  insertMenu->insertItem(Icon("palette"), "B&ackground Color...", this, SLOT(slotInsertBackGroundColor()), 0, ID_INSERT_BACCOLOR);
  insertMenu->insertItem(Icon("pen"), "&Pen Color...", this, SLOT(slotInsertPenColor()), 0, ID_INSERT_PENCOLOR);
  insertMenu->insertItem(Icon("link"), "&Link", this, SLOT(slotInsertLink()), CTRL+Key_K, ID_INSERT_LINK);
  insertMenu->insertSeparator();
  insertMenu->insertItem("L&ast Message", this, SLOT(slotInsertLastMessage()),Key_Insert, ID_INSERT_LASTMESSAGE);
  
  buddyMenu->clear();
  buddyMenu->insertItem(Icon("send"), "&Send Message", this, SLOT(slotBuddySendMessage()), CTRL+Key_S, ID_BUDDY_SEND);
  buddyMenu->insertItem(Icon("info"), "&Buddy Info", this, SLOT(slotBuddyInfo()), 0, ID_BUDDY_INFO);
  buddyMenu->insertItem(Icon("warn"), "&Warn Buddy...", this, SLOT(slotBuddyWarn()), 0, ID_BUDDY_WARN);
  if (isBuddy)
    buddyMenu->insertItem(Icon("remove"), "&Remove Buddy...", this, SLOT(slotBuddyAddRemove()), 0, ID_BUDDY_REMOVE);
  else
    buddyMenu->insertItem(Icon("add"), "&Add Buddy", this, SLOT(slotBuddyAddRemove()), 0, ID_BUDDY_REMOVE);
  buddyMenu->insertItem(Icon("block"), "B&lock Buddy...", this, SLOT(slotBuddyBlock()), 0, ID_BUDDY_BLOCK);

  // menuBar entry helpMenu
  helpMenu->clear();
  helpMenu->insertItem(Icon("gohome"), "Open Web Site", this, SIGNAL(openHomePage()));
	helpMenu->insertSeparator();
	helpMenu->insertItem(Icon("kaim_small"), "&About...", this, SIGNAL(showAbout()), 0, ID_HELP_ABOUT);
}

/******************************************************************************
 * ToolBar setup  Clears the toolbars and fills them with the widgets
 * specified in toolBarSettings then oriants the toolbars as chosen.
 ******************************************************************************/
void ChatWindow::initToolBar(){
  // Preview Tool Bar
  previewToolbar->clear();
  previewToolbar->hide();
  previewBox = NULL;
  if (toolBarSettings->prev_prev == true){
    previewBox = new QLabel(previewToolbar, "PreviewBox");
    previewBox->setMinimumSize(140,20);
    paintPreviewWidget();
    previewToolbar->addSeparator();
  }
  if (toolBarSettings->prev_send == true)
    sendMessageButton = new QToolButton(Icon("send"), "Send Message", 0, this,
                                          SLOT(slotBuddySendMessage()), previewToolbar);
  if (toolBarSettings->view_prev == true){
    if (toolBarSettings->loc_prev == 0){
      QMainWindow::moveToolBar(previewToolbar,QMainWindow::Top);
      previewToolbar->show();
    }
    if (toolBarSettings->loc_prev == 1){
      QMainWindow::moveToolBar(previewToolbar,QMainWindow::Right);
      previewToolbar->show();
    }
    if (toolBarSettings->loc_prev == 2){
      QMainWindow::moveToolBar(previewToolbar,QMainWindow::Bottom);
      previewToolbar->show();
    }
    if (toolBarSettings->loc_prev == 3){
      QMainWindow::moveToolBar(previewToolbar,QMainWindow::Left);
      previewToolbar->show();
    }
  }

  // File Tool Bar
  //fileToolbar->setVerticalStretchable(true);
  //fileToolbar->setOrientation (Vertical);
  fileToolbar->hide();
  fileToolbar->clear();
  if (toolBarSettings->chat_clear == true)
    chatClearButton = new QToolButton(Icon("clear"), "Clear Chat", 0, this,
                                          SLOT(slotChatClear()), fileToolbar);
  if (toolBarSettings->chat_save == true)
    chatSaveButton = new QToolButton(Icon("save"), "Save Chat As", 0, this,
                                          SLOT(slotChatSaveAs()), fileToolbar);
  if (toolBarSettings->chat_print == true)
    chatPrintButton = new QToolButton(Icon("printer"), "Print Chat", 0, this,
                                          SLOT(slotChatPrint()), fileToolbar);
  if ((toolBarSettings->chat_clear == true) ||
     (toolBarSettings->chat_save == true) ||
     (toolBarSettings->chat_print == true))
    fileToolbar->addSeparator();
  if (toolBarSettings->chat_time == true)
    chatTimeButton = new QToolButton(Icon("time"), "Show Time", 0, this,
                                          SLOT(slotViewTimeStamp()), fileToolbar);
  if (chatSettings->chat_show_time == true && toolBarSettings->chat_time == true)
    chatTimeButton->setDown(true);
  else
    chatTimeButton->setDown(false);
  
  if (toolBarSettings->chat_log == true)
    chatLogButton = new QToolButton(Icon("log"), "Log", 0, this,
                                          SLOT(slotChatLog()), fileToolbar);
  if (chatSettings->chat_log == true && toolBarSettings->chat_log == true)
    chatLogButton->setDown( true );
  else
    chatLogButton->setDown( false );  
  if (toolBarSettings->chat_sound == true)
    chatSoundMuteButton = new QToolButton(Icon("speaker"), "Sound", 0, this, SLOT(slotChatSound()), fileToolbar);

  if (soundSettings->chat_sound == true && toolBarSettings->chat_sound == true){
    chatSoundMuteButton->setDown ( true );
    chatSoundMuteButton->setTextLabel ("Sound Enabled", true);
  }
  else{
    if (toolBarSettings->chat_sound == true){
      chatSoundMuteButton->setDown ( false );    
      chatSoundMuteButton->setTextLabel ("Sound Muted", true);
    }
  } 
  if ((toolBarSettings->chat_time == true) ||
     (toolBarSettings->chat_log == true) ||
     (toolBarSettings->chat_sound == true)){
    if (toolBarSettings->chat_remove == true ||
	toolBarSettings->chat_warn == true ||
	toolBarSettings->chat_block == true ||
	toolBarSettings->chat_info == true)        
    fileToolbar->addSeparator();
  }
  if (toolBarSettings->chat_remove == true){
    if(isBuddy == true)
      userRemoveButton = new QToolButton(Icon("remove"), "Remove Buddy", 0, this, SLOT(slotBuddyAddRemove()), fileToolbar);
    else
      userRemoveButton = new QToolButton(Icon("add"), "Add Buddy", 0, this, SLOT(slotBuddyAddRemove()), fileToolbar);
  }
  if (toolBarSettings->chat_warn == true)
    userWarnButton = new QToolButton(Icon("warn"), "Warn Buddy", 0, this,
                                          SLOT(slotBuddyWarn()), fileToolbar);
  if (toolBarSettings->chat_block == true)
    userBlockButton = new QToolButton(Icon("block"), "Block Buddy", 0, this,
                                          SLOT(slotBuddyBlock()), fileToolbar);
  if (toolBarSettings->chat_info == true)
    userInfoButton = new QToolButton(Icon("info"), "Buddy Info", 0, this,
                                          SLOT(slotBuddyInfo()), fileToolbar);
  if (toolBarSettings->view_chat == true){
    if (toolBarSettings->loc_chat == 0){
      QMainWindow::moveToolBar(fileToolbar,QMainWindow::Top);
      fileToolbar->show();
    }
    if (toolBarSettings->loc_chat == 1){
      QMainWindow::moveToolBar(fileToolbar,QMainWindow::Right);
      fileToolbar->show();
    }
    if (toolBarSettings->loc_chat == 2){
      QMainWindow::moveToolBar(fileToolbar,QMainWindow::Bottom);
      fileToolbar->show();
    }
    if (toolBarSettings->loc_chat == 3){
      QMainWindow::moveToolBar(fileToolbar,QMainWindow::Left);
      fileToolbar->show();
    }
  }

  textToolbar->hide();
  // TextToolBar
  textToolbar->clear();
  if (toolBarSettings->text_bold == true)
    textBoldButton = new QToolButton(Icon("bold"), "Bold", 0, this,
                                          SLOT(slotInsertBold()), textToolbar);
  if (toolBarSettings->text_italic == true)
    textItalicButton = new QToolButton(Icon("italic"), "Italic", 0, this,
                                          SLOT(slotInsertItalic()), textToolbar);
  if (toolBarSettings->text_underline == true)
    textUnderLineButton = new QToolButton(Icon("underline"), "Underline", 0, this,
                                          SLOT(slotInsertUnderline()), textToolbar);

  if ((toolBarSettings->text_underline == true) ||
     (toolBarSettings->text_italic == true) ||
     (toolBarSettings->text_underline == true))
    textToolbar->addSeparator();
  if (toolBarSettings->text_fontbig == true)
    textUpFontButton = new QToolButton(Icon("small"), "Decrease Font", 0, this,
                                          SLOT(slotInsertShrinkFont()), textToolbar);
  if (toolBarSettings->text_fontnormal == true)
    textNormFontButton = new QToolButton(Icon("normal"), "Normal Font", 0, this,
                                          SLOT(slotInsertNormalFont()), textToolbar);
  if (toolBarSettings->text_fontdown == true)
    textDownFontButton = new QToolButton(Icon("big"), "Increase Font", 0, this,
                                          SLOT(slotInsertIncreaseFont()), textToolbar);
  if ((toolBarSettings->text_bgcolor == true) ||
     (toolBarSettings->text_pencolor == true) ||
     (toolBarSettings->text_link == true))
    textToolbar->addSeparator();
  if (toolBarSettings->text_bgcolor == true)
    colorBackButton = new QToolButton(Icon("palette"), "Background Color", 0, this,
                                          SLOT(slotInsertBackGroundColor()), textToolbar);
  if (toolBarSettings->text_pencolor == true)
    colorPenButton = new QToolButton(Icon("pen"), "Pen Color", 0, this,
                                          SLOT(slotInsertPenColor()), textToolbar);
  if (toolBarSettings->text_link == true)
    linkButton = new QToolButton(Icon("link"), "Insert Link", 0, this,
                                          SLOT(slotInsertLink()), textToolbar);
  if (toolBarSettings->view_text == true){
    if (toolBarSettings->loc_text == 0){
      QMainWindow::moveToolBar(textToolbar,QMainWindow::Top);
      textToolbar->show();
    }
    if (toolBarSettings->loc_text == 1){
      QMainWindow::moveToolBar(textToolbar,QMainWindow::Right);
      textToolbar->show();
    }
    if (toolBarSettings->loc_text == 2){
      QMainWindow::moveToolBar(textToolbar,QMainWindow::Bottom);
      textToolbar->show();
    }  
    if (toolBarSettings->loc_text == 3){
      QMainWindow::moveToolBar(textToolbar,QMainWindow::Left);
      textToolbar->show();
    }
  }
}

/******************************************************************************
 * Sets up the input window and the chat window
 ******************************************************************************/
void ChatWindow::initChatWindow(){
  spliter = new QSplitter(this, "Splitter" );
  spliter->setOrientation(QSplitter::Vertical);

  chatWindow = new KaimTextBrowser(spliter, "ChatWindow");
  chatWindow->setHScrollBarMode(QScrollView::AlwaysOff);
  connect(chatWindow, SIGNAL(gotoURL(QString)), this, SLOT(urlClicked(QString)));

  inputWindow = new QMultiLineEdit( spliter , "InputWindow");
  inputWindow->setWordWrap(QMultiLineEdit::WidgetWidth);
  connect(inputWindow, SIGNAL(undoAvailable(bool)), this, SLOT(slotEditTurnOnOffUndo(bool)));
  connect(inputWindow, SIGNAL(redoAvailable(bool)), this, SLOT(slotEditTurnOnOffRedo(bool)));
  connect(inputWindow, SIGNAL(textChanged()), this, SLOT(inputTextChanged()));
  postNumber = 0;
  flashWindow = false;
  spliter->setResizeMode(inputWindow, QSplitter::KeepSize);
  spliter->setResizeMode(chatWindow, QSplitter::Stretch);
  connect(inputWindow, SIGNAL(returnPressed()), this, SLOT(enterKeyPressed()));
  QValueList<int> list;
  list = spliter->sizes();
  list[1] = 40;
  spliter->setSizes(list); 
  
  this->setCentralWidget( spliter );
}

/******************************************************************************
 * Open this url in a web browser.  Use the global browser interface.
 ******************************************************************************/
void ChatWindow::urlClicked(QString url){
  browser->openURL(url);
}

/******************************************************************************
 * paint the Preview window
 ******************************************************************************/
void ChatWindow::paintPreviewWidget(){
  QPixmap temp;
  temp.resize(170, 20);
  QString bgRGB = "000000";
  bgRGB.sprintf("%02x%02x%02x", colorSettings->chat_yourbg_color.red(),
    colorSettings->chat_yourbg_color.green(),
    colorSettings->chat_yourbg_color.blue());
  if (bgRGB == "000000")
    temp.fill( black );
  else
    temp.fill(colorSettings->chat_yourbg_color);
  QPainter paint;
  paint.begin( &temp );
  QFont aFont = paint.font();
  aFont.setPixelSize(fontSize*4);
  paint.setFont(aFont);
  paint.setPen( colorSettings->chat_yourText_color );
  paint.drawText( 5, 0, 207, 20, AlignLeft|AlignVCenter ,"Preview Text" );
  // Stop painting
  paint.end();
  previewBox->setPixmap(temp);
}

/******************************************************************************
 * The buddy signed off and so somthing should be written.
 ******************************************************************************/
void ChatWindow::buddySignedOff(){
  QString CurrentTime = QTime::currentTime().toString().data();
  QString message = QString("<B>") + buddyName + " Signed off at: " + CurrentTime + "</B>\n";
  if (postNumber == 0)
    chatWindow->setText(message);
  else
    chatWindow->append(message);
  postNumber++;
  
  conversationBuffer += message;
  conversationTimeBuffer += message;
  
  chatWindow->ensureVisible(0, chatWindow->contentsHeight());
  // This should not be a hard coded value like this!!!
  chatWindow->repaintContents(0,0, 10000, 20000, true);
  
  // Add to the logs.
  if (chatSettings->chat_log == true){
    if (chatSettings->chat_log_inHtml == true){
      logStream << QString("<B>") + buddyName + " Signed off at: " + CurrentTime + "</B><br/>\n";
;
    }
    else{
      logStream << buddyName + " Signed off at: " + CurrentTime + "\n";
    }
  }
}

/******************************************************************************
 * The buddy signed on and so somthing should be written.
 ******************************************************************************/
void ChatWindow::buddySignedOn(){
  QString CurrentTime = QTime::currentTime().toString().data();
  QString message = QString("<B>") + buddyName + " Signed on at: " + CurrentTime + "</B>\n";
  if (postNumber == 0)
    chatWindow->setText(message);
  else
    chatWindow->append(message);
  postNumber++;

  conversationBuffer += message;
  conversationTimeBuffer += message;
  
  chatWindow->ensureVisible(0, chatWindow->contentsHeight());
  // This should not be a hard coded value like this!!!
  chatWindow->repaintContents(0,0, 10000, 20000, true);
  // Add to the logs.
  if (chatSettings->chat_log == true){
    if (chatSettings->chat_log_inHtml == true){
      logStream << QString("<B>") + buddyName + " Signed off at: " + CurrentTime + "</B><br/>\n";
;
    }
    else{
      logStream << buddyName + " Signed off at: " + CurrentTime + "\n";
    }
  }
}

/******************************************************************************
 * From another part of the system a message is being sent from you.
 ******************************************************************************/
void ChatWindow::pounceMessage(QString message){
  inputWindow->setText(message);
  slotBuddySendMessage();
}

/******************************************************************************
 * When you send an Away message to someone put it on the chat window also.
 ******************************************************************************/
void ChatWindow::awayMessage(QString message){
  message = QString("<B>") + "Auto Reply:</B> " + message +"\n";
  if (postNumber == 0)
    chatWindow->setText(message);
  else
    chatWindow->append(message);
  postNumber++;
  
  conversationBuffer += message;
  conversationTimeBuffer += message;
  
  chatWindow->ensureVisible(0, chatWindow->contentsHeight());
  // This should not be a hard coded value like this!!!
  chatWindow->repaintContents(0,0, 10000, 20000, true);
  // Add to the logs.
  if (chatSettings->chat_log == true){
    if (chatSettings->chat_log_inHtml == true){
      logStream << QString("<B>") + "Auto Reply:</B> " + message + "<br/>\n";
    }
    else{
      logStream << buddyName + "Auto Reply: " + message + "\n";
    }
  }
}

/******************************************************************************
 * Sets the log variable
 ******************************************************************************/
void ChatWindow::slotChatLog(){
  if (chatSettings->chat_log == false){
    chatSettings->chat_log = true;
    fileMenu->setItemChecked(ID_FILE_LOG, true);
    if (toolBarSettings->chat_log == true)
      chatLogButton->setDown ( true );
    openLogFile();
  }
  else{
    chatSettings->chat_log = false;
    fileMenu->setItemChecked(ID_FILE_LOG, false);
    if (toolBarSettings->chat_log == true)
      chatLogButton->setDown ( false );
    // Delete log file
    logFile.close();
    if (logFile.remove() == false)
      QMessageBox::critical(0, APP_NAME, QString("Can not delete logfile "), "OK");
  }
}

/******************************************************************************
 * Clear the chat window
 ******************************************************************************/
void ChatWindow::slotChatClear(){
  cleared = true;
  chatWindow->setText( "", chatWindow->context());
  postNumber = 0;
  // Clear conversation
  if (chatSettings->chat_log_inHtml == true){
    conversationBuffer = "<HTML>\n<HEAD>\n<TITLE>Conversations with ";
    conversationBuffer += buddyName;
    conversationBuffer += "</TITLE>\n</HEAD>\n<BODY BGCOLOR=\"#FFFFFF\">\n";
    // Clear other conversation buffer
    conversationTimeBuffer = "<HTML>\n<HEAD>\n<TITLE>Conversations with ";
    conversationTimeBuffer += buddyName;
    conversationTimeBuffer += "</TITLE>\n</HEAD>\n<BODY BGCOLOR=\"#FFFFFF\">\n";
  }
  else{
    conversationBuffer = "Conversations with " + buddyName + "\n";
    // Clear other conversation buffer
    conversationTimeBuffer = "Conversations with " + buddyName + "\n";
  }
}

/******************************************************************************
 * Save the chat in a file.
 ******************************************************************************/
void ChatWindow::slotChatSaveAs(){
  QString fn = QFileDialog::getSaveFileName(0, 0, this);
  if (!fn.isEmpty()){
    // open file for appending, to add the new message
    QFile SaveFile(fn);
    if (SaveFile.open( IO_WriteOnly) == false)
      QMessageBox::critical(0, APP_NAME, QString("Unable to open ") + fn + " for appending.", "OK");
    else{
      // All text
      QTextStream fileStream(&SaveFile);		
      if (cleared == false){
        if (chatSettings->chat_log_inHtml == true){
          fileStream << "<HTML>\n<HEAD>\n<TITLE>Conversations with ";
          fileStream << buddyName;
          fileStream << "</TITLE>\n</HEAD>\n<BODY BGCOLOR=\"#FFFFFF\">\n";
        }
        else{
          fileStream << "Conversations with " << buddyName << "\n";
        }
      }
      if (( chatSettings->chat_log_time == true) || ( chatSettings->chat_log == true))
        if (chatSettings->chat_log_inHtml == false)
 	  fileStream << (parseAndProcessALL(conversationTimeBuffer));
	else    
	  fileStream << conversationTimeBuffer;
      else
        fileStream << conversationBuffer;
      
      if (chatSettings->chat_log_inHtml == true){
        fileStream << "</BODY>\n</HTML>";
      }
      SaveFile.close();
      QMessageBox::information(0, APP_NAME, QString("Chat so far is saved in\n") + fn , "OK");
    }
  }
  else{
    // Cancel was hit, do nothing.
  }
}

/******************************************************************************
 * Print out the chat
 ******************************************************************************/
void ChatWindow::slotChatPrint(){
  QPrinter printer;
  printer.setFullPage(true);
  if ( printer.setup() ) {
    QPainter p( &printer );
    QPaintDeviceMetrics metrics(p.device());
    int dpix = metrics.logicalDpiX();
    int dpiy = metrics.logicalDpiY();
    const int margin = 72; // pt
    QRect body(margin*dpix/72, margin*dpiy/72, metrics.width()-margin*dpix/72*2,
      metrics.height()-margin*dpiy/72*2 );
    QFont font("times", 10);
    QSimpleRichText richText( conversationBuffer, font, chatWindow->context(), chatWindow->styleSheet(),
      chatWindow->mimeSourceFactory(), body.height() );
    richText.setWidth( &p, body.width() );
    QRect view( body );
    int page = 1;
    do {
      // Draw the first page
      richText.draw( &p,body.left(), body.top(), view, colorGroup() );
      view.moveBy( 0, body.height() );
      p.translate( 0 , -body.height() );
      // Draw page number
      p.setFont( font );
      p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
        view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page) );
      // Check if it is time to stop printing
      if ( view.top()  >= richText.height() ) break;
      // Guess it isn't so start a new page
      printer.newPage();
      // Set the new page number
      page++;
    } while (true);
  }
}

/******************************************************************************
 * Turn Sound on and off
 ******************************************************************************/
void ChatWindow::slotChatSound(){
  if (soundSettings->chat_sound == false){
    soundSettings->chat_sound = true;
    fileMenu->setItemChecked(ID_FILE_SOUND, true);
    if (toolBarSettings->chat_sound == true){
      if (chatSoundMuteButton){
        chatSoundMuteButton->setDown ( true );
        chatSoundMuteButton->setTextLabel ("Sound Enabled", true);
      }
    }
    if (soundSettings->chat_sound_beepReceive == false &&
        soundSettings->chat_sound_beepSend == false){
      soundSettings->chat_sound_beepReceive = true;
      soundSettings->chat_sound_beepSend = true;
    }
  }
  else{
    soundSettings->chat_sound = false;
    fileMenu->setItemChecked(ID_FILE_SOUND, false);
    if (toolBarSettings->chat_sound == true){
      if (chatSoundMuteButton){
	chatSoundMuteButton->setDown ( false );
        chatSoundMuteButton->setTextLabel ("Sound Muted", true); 
      }
    }
  }
}

/******************************************************************************
 * Close out of the Chat
 ******************************************************************************/
void ChatWindow::closeEvent(QCloseEvent *) {
  // Close window
  emit quiting(buddyName);
}

/******************************************************************************
 * Close out of the Chat
 ******************************************************************************/
void ChatWindow::slotChatClose(){
  QApplication::sendEvent(this, new QCloseEvent());
}

/******************************************************************************
 * Cut highlighted
 ******************************************************************************/
void ChatWindow::slotEditCut(){
  if (inputWindow->hasFocus())
    inputWindow->cut();
}

/******************************************************************************
 * Copy highlighted to clipboard
 ******************************************************************************/
void ChatWindow::slotEditCopy(){
  if (inputWindow->hasFocus())
    inputWindow->copy();
  else
    chatWindow->copy();  
}

/******************************************************************************
 * Paste the clipboard to edit box
 ******************************************************************************/
void ChatWindow::slotEditPaste(){
  if (inputWindow->hasFocus())
    inputWindow->paste();
}

/******************************************************************************
 * Select All
 ******************************************************************************/
void ChatWindow::slotEditSelectAll(){
  if (inputWindow->hasFocus())
    inputWindow->selectAll();
  else
    chatWindow->selectAll(); 
}

/******************************************************************************
 *  Enable/disable undo
 ******************************************************************************/
void ChatWindow::slotEditTurnOnOffUndo(bool change){
  if (change == false)
    editMenu->setItemEnabled ( ID_EDIT_UNDO, false );
  else
    editMenu->setItemEnabled ( ID_EDIT_UNDO, true );
}

/******************************************************************************
 * Enable/disable redo
 ******************************************************************************/
void ChatWindow::slotEditTurnOnOffRedo(bool change){
  if (change == false)
    editMenu->setItemEnabled ( ID_EDIT_REDO, false );
  else
    editMenu->setItemEnabled ( ID_EDIT_REDO, true );
}

/******************************************************************************
 * Edit Undo
 ******************************************************************************/
void ChatWindow::slotEditUndo(){
  if (inputWindow->hasFocus())
    inputWindow->undo();
}

/******************************************************************************
 * Edit Redo
 ******************************************************************************/
void ChatWindow::slotEditRedo(){
  if (inputWindow->hasFocus())
    inputWindow->redo();
}


/******************************************************************************
 * Toggle the Chat Toolbar
 ******************************************************************************/
void ChatWindow::slotViewChatToolBar(){
  // turn Toolbar on or off
  if (fileToolbar->isVisible())
  {
    fileToolbar->hide();
    viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, false);
  }
  else
  {
    fileToolbar->show();
    viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, true);
  };
}

/******************************************************************************
 * Toggle the Text Toolbar
 ******************************************************************************/
void ChatWindow::slotViewTextToolBar(){
  // turn Toolbar on or off
  if (textToolbar->isVisible())
  {
    textToolbar->hide();
    viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, false);
  }
  else
  {
    textToolbar->show();
    viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, true);
  };
}

/******************************************************************************
 * Toggle the Preview Toolbar
 ******************************************************************************/
void ChatWindow::slotViewPreviewToolBar(){
  // turn Toolbar on or off
  if (previewToolbar->isVisible())
  {
    previewToolbar->hide();
    viewMenu->setItemChecked(ID_VIEW_PREVIEWTOOLBAR, false);
  }
  else
  {
    previewToolbar->show();
    viewMenu->setItemChecked(ID_VIEW_PREVIEWTOOLBAR, true);
  };
}

/******************************************************************************
 * Toggle the TimeStamp Variable
 ******************************************************************************/
void ChatWindow::slotViewTimeStamp(){
  if (chatSettings->chat_show_time == false){
    chatSettings->chat_show_time = true;
    viewMenu->setItemChecked(ID_VIEW_TIME, true);
    if ( toolBarSettings->chat_time == true)
      chatTimeButton->setDown ( true );
    chatSettings->chat_log_time = true;
    chatWindow->setText(conversationTimeBuffer);
    if (chatSettings->chat_log == true){
      // Turn off
      slotChatLog();
      // And back on, but with time (in case)
      slotChatLog();
    }
  }
  else{
    chatSettings->chat_show_time = false;
    viewMenu->setItemChecked(ID_VIEW_TIME, false);
    if ( toolBarSettings->chat_time == true)
      chatTimeButton->setDown ( false );
    chatSettings->chat_log_time = true;
    chatWindow->setText(conversationBuffer);
  }
  // scroll to the new text
  chatWindow->ensureVisible(0, chatWindow->contentsHeight());
}

/******************************************************************************
 * Inserts a set of tags into the input window.
 ******************************************************************************/
void ChatWindow::slotInsertTags(QString open, QString close){
  // Cut any selected text to clipboard
  inputWindow->cut();
  // Copy text from the clipboard (paste)
  QString oldtext = QApplication::clipboard()->text();
  // Add tags
  oldtext = open  + oldtext + close;
  // paste new text into input window
  inputWindow->insert(oldtext);
  int a,b;
  inputWindow->getCursorPosition(&a,&b); 
  inputWindow->setCursorPosition(a,b-close.length(),false);
  currentMessageOut = inputWindow->text();
}


/******************************************************************************
 * Insert bold tags in the input window
 ******************************************************************************/
void ChatWindow::slotInsertBold(){
  slotInsertTags( "<b>" , "</b>" );
}

/******************************************************************************
 * Insert italic tags in the input window
 ******************************************************************************/
void ChatWindow::slotInsertItalic(){
  slotInsertTags( "<i>" , "</i>" );
}

/******************************************************************************
 * Insert underline tags in the input window
 ******************************************************************************/
void ChatWindow::slotInsertUnderline(){
  slotInsertTags( "<u>" , "</u>" );
}

/******************************************************************************
 * Insert small tags in the input window
 ******************************************************************************/
void ChatWindow::slotInsertShrinkFont(){
  if (fontSize > 0)
    fontSize--;
  paintPreviewWidget();
}

/******************************************************************************
 * Set the Font back to normal
 ******************************************************************************/
void ChatWindow::slotInsertNormalFont(){
  fontSize = 3;
  paintPreviewWidget();
}

/******************************************************************************
 * insert font tags
 ******************************************************************************/
void ChatWindow::slotInsertIncreaseFont(){
  if (fontSize < 7)
    fontSize++;
  paintPreviewWidget();
}

/******************************************************************************
 *  Get the background color you want.
 ******************************************************************************/
void ChatWindow::slotInsertBackGroundColor(){
  colorSettings->chat_yourbg_color = QColorDialog::getColor ( colorSettings->chat_yourbg_color, this, "bgColorSelector" );
  paintPreviewWidget();
}

/******************************************************************************
 * Brings up the color dialog and get the pen color.
 ******************************************************************************/
void ChatWindow::slotInsertPenColor(){
  colorSettings->chat_yourText_color = QColorDialog::getColor ( colorSettings->chat_yourText_color, this,"PenColorSelector" );
  paintPreviewWidget();
}

/******************************************************************************
 * Inserts the link tags into the input window
 ******************************************************************************/
void ChatWindow::slotInsertLink(){
  //KaimLinkDialog *linkDlg = new	KaimLinkDialog(this, "link dialog", TRUE);
  KaimLinkDialog linkDlg(this, "link dialog", true);

  linkDlg.setURLText( QApplication::clipboard()->text() );
  linkDlg.setDescText( QApplication::clipboard()->text() );
	
  QString tag = linkDlg.go();
 
  if(!tag.isEmpty()){
    inputWindow->cut();
    inputWindow->insert(tag);
    currentMessageOut = inputWindow->text();
  }
}

/***************************************************************************
 * Puts the last message sent into the input box.
 ***************************************************************************/
void ChatWindow::slotInsertLastMessage(){
  if (inputWindow->text() == QString(""))
    inputWindow->setText(lastMessage);
  else  
    inputWindow->append(lastMessage);
}

#include <iostream.h>
#include <string.h>
/******************************************************************************
 * Send the message in the input window
 ******************************************************************************/
void ChatWindow::slotBuddySendMessage(){
  //currentMessageOut = "";
    
  // Example Output HTML
  //<HTML><BODY BGCOLOR="#FFFF00"><FONT COLOR="#FF00FF">Room for one more?</FONT></BODY></HTML>
  //inputWindow->setCursorPosition(0, 0, true); 	
  bool send = true;
  // A blank QMultilineedit has 2 chars in it at start \n\n??
  //printf("Input length: %i\n", inputWindow->length());
  if ((inputWindow->length() == 0) && (chatSettings->chat_send_blank == false)){
    send = false;
    QString message = inputWindow->text().data();
    if(message == "\n")
      inputWindow->setText( "" );
  }  
  if (send == true){
    QString nameRGB;
    nameRGB= colorSettings->chat_yourName_color.name();
    QString myRGB;
    myRGB= colorSettings->chat_yourText_color.name();
    QString bgRGB;
    bgRGB = colorSettings->chat_yourbg_color.name();

    // Send the input window data
    QString message = inputWindow->text().data();
    inputWindow->setText( "" );
    // Save this message in case people want to use it.
    QString lastLetter = message.mid(message.length()-1,1);
    if (lastLetter == "\n"){
      message = message.mid(0, message.length()-1);
      lastMessage = message;
    }
    lastLetter = message.mid(message.length()-1,1);
    if (lastLetter == "\n"){
      message = message.mid(0, message.length()-1);
      lastMessage = message;
    }
    //printf("Sending This Message: %s\n", message.data());
    updateHTML(message, myName, nameRGB);
    // Set the input window blank
    //emit signal
   /*	
    QString key = "C8530C2F";
		istrstream istr("Whats Up!");
    strstream ostr;
		printf("1\n");
				
		GPGInterface *gpg = new GPGInterface("C8530C2F","lemonwatch",false);
    printf("2\n");
		gpg->encrypt("ben",&istr,&ostr,false);
    printf("3\n");
			
		char Tbuffer[4096];
     printf("4\n");
			bool have_message = false;
     QString Tmessage;
printf("5\n");
			
     while (ostr.getline(Tbuffer, 4096))
     {
	  if (have_message && strncmp("-----", Tbuffer, 5) == 0)
	       // Strip the trailing \n since it isn't needed (none of them are but I'm not sure if
	       // other versions of pgp are as indifferent to \n in the armor as gpg
	       Tmessage.left(Tmessage.find("\n"));
	  else if (have_message){
	       Tmessage += Tbuffer;
	       Tmessage += "\n";
	  }
	  
			// A blank line separates the header from the message
	  else if (*Tbuffer == '\0')
	       have_message = true;
     }
     char TempV[12] = " > data.gpg";
		 printf("7\n");
			Tmessage += TempV;
		 QString bob = Tmessage;
		 printf("8\n");
			system(bob.data());
		
		*/
    // If not normal font or color add them.
    QString fontSizeString;
    fontSizeString.setNum(fontSize);
    // Add font color size
    if (fontSize != 3)
      message = QString("<FONT SIZE=\"") + fontSizeString + "\">" + message + "</FONT>";
    if (myRGB != "#000000")
      message = QString("<FONT COLOR=\"") + myRGB + "\">" + message + "</FONT>";

    // add background color add it
    if (bgRGB != "#ffffff")
      message = QString("<BODY BGCOLOR=\"") + bgRGB + "\">" + message + "</BODY>";

    message = QString("<HTML>") + message + "</HTML>";
    emit( messageOut( message, buddyName ) );
		emit( antiIdle() );
    // emit beep in wanted.
    if (( soundSettings->chat_sound == true ) &&
      ( soundSettings->chat_sound_beepSend == true) &&
      ( soundSettings->chat_sound_playFile == false)){
      printf("\a");
      fflush(stdout);
    }
    else{
      if (( soundSettings->chat_sound == true ) &&
      ( soundSettings->chat_sound_beepSend == true) &&
      ( soundSettings->chat_sound_playFile == true)){
        system(QString("play ") + soundSettings->chat_sound_fileSend +" &");
      }

    }   
  }
}

/******************************************************************************
 * Get information about the current buddy
 ******************************************************************************/
void ChatWindow::slotBuddyInfo(){
  emit( getInfo( buddyName ) );
	emit( antiIdle() );
}

/******************************************************************************
 * Send a warning message against the buddy
 * Pass to main window for conferm/action
 ******************************************************************************/
void ChatWindow::slotBuddyWarn(){
  emit( warningOut(buddyName) );
	//emit( antiIdle() ); // XXX: need this here?
}

/******************************************************************************
 * Remove the buddy from the current list
 * Pass to main window for conferm/action
 ******************************************************************************/
void ChatWindow::slotBuddyAddRemove(){
  if (isBuddy == true){
    emit( removeBuddy(buddyName) );
  }
  else{
    emit( addBuddy(buddyName) ); 
  }
}

/******************************************************************************
 * Block this buddy from being on my list. 
 * Pass to main window for conferm/action
 ******************************************************************************/
void ChatWindow::slotBuddyBlock(){
  emit( blockBuddy(buddyName) );
	//emit( antiIdle() );	// XXX: need this here?
}

/***************************************************************************
 * Occurs when the message comes in
 ***************************************************************************/
void ChatWindow::messageIn(QString message){
  if(message.length() == 0 || message == "" )
    message = "  ";  
  //printf("Message In: %s\n", message.data());
  QString buddyRGB;
  buddyRGB = colorSettings->chat_buddyName_color.name();

  if (chatSettings->chat_raise_window == true){
    this->show();
    this->raise();
  }
  if (chatSettings->chat_ignore_buddyhtml == true)
    updateHTML(parseAndProcessALL(message), buddyName, buddyRGB);
  else
    updateHTML(message, buddyName, buddyRGB);
  if (chatSettings->chat_flash_on_new_message == true){
    if (flashWindow == false){
      flashWindow = true;
      flashIcon();
    }
    // Set off timer that will change icon until got focus
    // new function needed.
  }

  if (( soundSettings->chat_sound == true ) &&
      ( soundSettings->chat_sound_beepReceive == true) &&
      ( soundSettings->chat_sound_playFile == false)){
    printf("\a");
    fflush(stdout);
  }
  if (( soundSettings->chat_sound == true ) &&
      ( soundSettings->chat_sound_beepReceive == true) &&
      ( soundSettings->chat_sound_playFile == true)){
    system(QString("play ") + soundSettings->chat_sound_fileReceive +" &");
  }
}

/***************************************************************************
 * Helper Function
 ***************************************************************************/
inline bool tagIsOpen(const QString &html, const QString &tag){
  return html.contains("<" + tag, false) > html.contains("</" + tag + ">", false);
}

/***************************************************************************
 * Occurs when the message comes in
 ***************************************************************************/
void ChatWindow::updateHTML(QString message, QString sender, QString HTMLColor){
  message = stripTags(message);
  if (message.length() == 1 || message.length() == 0 || message == "")
    message += " ";  
  //printf("message:%s\n", message.data());  
  // Get Timestap info
  QString CurrentTime = QTime::currentTime().toString().data();
	
  QString textRGB = "#000000";
  if (sender == myName)
    textRGB = colorSettings->chat_yourText_color.name();
  else
    textRGB = colorSettings->chat_buddyText_color.name();

  QString fontSizeString;
  fontSizeString.setNum(fontSize);
  // Do only if it is special
  if ((fontSizeString != "3") || (textRGB != "#000000")){
   if (sender == myName)
     message = QString("<font color=\"") + textRGB + "\" size=\""+ fontSizeString +"\">" + message + "</font>";
  else
    message = QString("<font color=\"") + textRGB + "\">" + message + "</font>";
  }
	
  /***** Get the name and : ****/
  QString TimeMessage = QString("<B><font color=\"") +
            HTMLColor + QString("\">") + CurrentTime + " " + sender + ": </font></B>"  + message;
	
  QString Message = QString("<B><font color=\"") +
            HTMLColor + QString("\">") + " " + sender + ": </font></B>"  + message;
	
  QString bgRGB = "#ffffff";
  if (sender == myName)
    bgRGB = colorSettings->chat_yourbg_color.name();
  else 
    bgRGB = colorSettings->chat_buddybg_color.name();
  
  // Add to the logs.
  if (chatSettings->chat_log == true){
    if (chatSettings->chat_log_inHtml == true){
      if (chatSettings->chat_log_time == true)
        logStream << TimeMessage << "<br>\n";
      else
        logStream << Message << "<br>\n";
    }
    else{
      // Write correct buffer
      if ( chatSettings->chat_log_time == true)
        logStream << parseAndProcessALL(TimeMessage) << "\n";
      else
        logStream << parseAndProcessALL(Message) << "\n";
    }
    logFile.flush();
  }
  
  Message.replace( QRegExp("\n"), "<br/>" );
  TimeMessage.replace( QRegExp("\n"), "<br/>" );
  
  // now add the message to the buffer
  conversationBuffer += Message;
  conversationTimeBuffer += TimeMessage;
  if (chatSettings->chat_log_inHtml == true){
    conversationBuffer += "<br/>\n";
    conversationTimeBuffer += "<br/>\n";
  }
  //printf("%s\n",bgRGB.data());	
  // Always add a background so there will be a return.
  //if (bgRGB != "#ffffff"){
    Message = QString("<table bgcolor=" + bgRGB + " border=0 cellpadding=0 cellspacing=0><tr><td>") + Message + "</td></tr></table><br/>";
    TimeMessage = QString("<table bgcolor=" + bgRGB + " border=0 cellpadding=0 cellspacing=0><tr><td>") + TimeMessage + "</td></tr></table><br/>";
  //}
	
  if(chatSettings->chat_show_time == true){
    if (postNumber == 0)
      chatWindow->setText(TimeMessage);
    else
      chatWindow->append(TimeMessage);
  }
  else{
    // NO TIME
    if (postNumber == 0)
      chatWindow->setText(Message);
    else
      chatWindow->append(Message);
  }
  
  // Incriment the post numbers
  postNumber++;

  if (postNumber == 1){
    if (( soundSettings->chat_sound == true ) &&
      (soundSettings->chat_sound_beepInitial == true) &&
      soundSettings->chat_sound_playFile == false){
      printf("\a");
      fflush(stdout);
    }
    else{
      if ( (soundSettings->chat_sound == true ) &&
      (soundSettings->chat_sound_beepInitial == true) &&
      (soundSettings->chat_sound_playFile == true))
        system(QString("play ") + soundSettings->chat_sound_fileInitial +" &");
    } 
  }

  // scroll to the new text
  chatWindow->ensureVisible(0, chatWindow->contentsHeight());
  // This should not be a hard coded value like this!!!
  chatWindow->repaintContents(0,0, 10000, 20000, true);

  // if conversationLength is getting too long, cut the top out
  if (chatSettings->chat_line_limit == true)
    if(postNumber > chatSettings->chat_line_limit_number){
      int spos = conversationBuffer.find("<B>");
      conversationBuffer.remove(spos, 4);
      spos = conversationBuffer.find("<B>");
      conversationBuffer.remove(0, spos);
      postNumber--;
      chatWindow->setText( conversationBuffer, chatWindow->context());
    }

  // reset buddy colors
  colorSettings->chat_buddyText_color = temp_BuddyTextColor;
  colorSettings->chat_buddybg_color = temp_BuddyBgColor;
  
}

/***************************************************************************
 * Parses the message
 ***************************************************************************/
#define SKIP_TO_GT {while(newstr[++i] != '>' && i < newstr.length());}

#define KILL_LT { newstr.replace(i, 1, "&lt;"); i += 3; }

#define SKIP_TO_GT_CHECK_QUOTE {int quotecount = 0;\
					while(newstr[++i] != '>' && i < newstr.length()){\
						if(newstr[i] == '"') quotecount++;}\
					if(quotecount % 2 == 1){\
						newstr.replace(i, 1, "\">");\
						i++;}}

QString ChatWindow::parseAndProcessALL(QString &newstr){
  QString returnVal = "";
  unsigned int beg = 0;
  int openTag = 1;
  while (openTag == 1)
  for(unsigned i = 0; i <= newstr.length(); i++){
    if (i == 0)
      openTag = 0;	
    if(newstr[i] == '<'){
      beg = i;
      SKIP_TO_GT
      unsigned int dif = i-beg;
      newstr.remove(beg,dif+1);
      openTag = 1;
      returnVal = "";
      i = newstr.length();
    }
    else returnVal += newstr[i];
  }
  return returnVal;
}

/***************************************************************************
 * Parses the message
 ***************************************************************************/
void ChatWindow::parseAndProcess(QString &newstr){
  //make this more modular to allow for easier additions in the future.
  newstr = parseForHTML(newstr);
  //TODO: IF selected in options dialog
  //newstr = parseForSpelling(newstr);
  //newstr = parseForAutoReplace(newstr);
}	

QString ChatWindow::parseForHTML(QString &newstr){
  bool bodyUsed = false;
  bool divUsed = false;
  bool pUsed = false;
	
  for(unsigned i = 0; i < newstr.length(); i++)
  {
    if(newstr[i] == '>')
    {
      newstr.replace(i, 1, "&gt;");
      i += 3;
    }

    else if(newstr[i] == '<')
    {
      if(newstr.mid(i, 3).upper() == "<A ") SKIP_TO_GT_CHECK_QUOTE
      else if(newstr.mid(i, 3).upper() == "<B>") i += 2;
      else if(newstr.mid(i, 5).upper() == "<BIG>") i += 4;
      else if(newstr.mid(i, 4).upper() == "<BR>") i += 3;
      else if(newstr.mid(i, 6).upper() == "<CODE>") i += 5;
      else if(newstr.mid(i, 4).upper() == "<EM>") i += 3;
      else if(newstr.mid(i, 5).upper() == "<FONT") SKIP_TO_GT_CHECK_QUOTE
      else if(newstr.mid(i, 3).upper() == "<I>") i += 2;
      else if(newstr.mid(i, 7).upper() == "<SMALL>") i += 6;
      else if(newstr.mid(i, 4).upper() == "<TT>") i += 3;
      else if(newstr.mid(i, 3).upper() == "<U>") i += 2;
      else if(!divUsed && newstr.mid(i, 4).upper() == "<DIV")
    {
      divUsed = true;
      SKIP_TO_GT_CHECK_QUOTE
    }
    else if(!pUsed && newstr.mid(i, 2).upper() == "<P")
    {
      pUsed = true;
      SKIP_TO_GT_CHECK_QUOTE
    }
    else if(!bodyUsed && newstr.mid(i, 5).upper() == "<BODY")
    {
      bodyUsed = true;
      SKIP_TO_GT_CHECK_QUOTE
    }
    else if(newstr.mid(i, 7).upper() == "</HTML>") KILL_LT
    else if(newstr.mid(i, 7).upper() == "</DIV>") KILL_LT
    else if(newstr.mid(i, 7).upper() == "</BODY>") KILL_LT
    else if(newstr.mid(i, 2).upper() == "</") SKIP_TO_GT
    else KILL_LT;
    }
  }
  return QString::null;
}

/*******************************************************************
 *  Allow user to run message through spell checker before send if desired
 ********************************************************************/
QString ChatWindow::parseForSpelling(QString &newstr){
  //Run message through a spell checker automatically if desired in options
  return newstr;
}

/********************************************************************
 * Automatically replace abbreviations, acronyms, etc. before sending
 ********************************************************************/
QString ChatWindow::parseForAutoReplace(QString &newstr){
  return newstr;
}

/***************************************************************************
 * Strip the tags from a qstring
 ***************************************************************************/
QString ChatWindow::stripTags(const QString &oldstr){
  //printf("StripTags First product: %s\n", oldstr.data());
      
  QString newstr = oldstr;
  int bodyPos;

  // 1) cut first <html> , last </html> tags
  bodyPos = newstr.find( QRegExp("<html>", false) );
  if(bodyPos != -1){
    newstr.replace(bodyPos, 6, "");
    bodyPos = newstr.findRev( QRegExp("</html>", false) );
    if(bodyPos != -1)
      newstr.replace(bodyPos, 7, "");
  }
  // set Font sizes
  bodyPos = newstr.find( QRegExp("<font size=", false) );
  while (bodyPos != -1){
    bool end = false;
    signed int value = bodyPos+13;
    // Find the end
    while (end == false){
      if (newstr.length() <= value){
        end = true;	      
        value = -1;
      }
      if (newstr.at(value) == '"')
	end = true;
      value++;     
    }
    printf("here%s\n",newstr.data() );
    if (value == 0)
      bodyPos = -1;	
    if (value-bodyPos > 14){
      QString number = QString(newstr.at(bodyPos+12));
      if ( number == "-"){
	newstr.replace(bodyPos+12, value-bodyPos-13, QString("%1").arg( chatSettings->chat_smallest_font_size));
      }
      else{
	newstr.replace(bodyPos+12, value-bodyPos-13, QString("%1").arg(7));
      }
    }
    printf("here%s\n",newstr.data() );
    if (value-bodyPos == 14){
      QString number = QString(newstr.at(bodyPos+12));
      printf("size:%s\n",number.data());
      if ( chatSettings->chat_smallest_font_size > atoi(number))
        newstr.replace(bodyPos+12, 1, QString("%1").arg( chatSettings->chat_smallest_font_size));
    }
    printf("bod%i\n",bodyPos);
    signed int num = bodyPos;
    if (bodyPos != -1)
      bodyPos = newstr.find("<font size=", (bodyPos+1), false);
    if (num == bodyPos)
      bodyPos = -1;	
  }

  
  // 2) cut <p> tags
  newstr.replace( QRegExp("<p>", false), "" );
  // 2b) replace </p> with <br>
  newstr.replace( QRegExp("</p>", false), "<br>" );
  // 3) replace <body> with <div>, inserting a STYLE= value to apply text and
  // background colors
  bodyPos = newstr.find("<BODY BGCOLOR=");
  if(bodyPos != -1){
    temp_BuddyBgColor = colorSettings->chat_buddybg_color;
    QString BGCOLOR = newstr.mid(bodyPos+16,8);
    colorSettings->chat_buddybg_color.setRgb(
      ((BGCOLOR.mid(0,2)).toLong(0,16)),
      ((BGCOLOR.mid(2,2)).toLong(0,16)),
      ((BGCOLOR.mid(4,2)).toLong(0,16))
      );
    //printf("%s\n",BGCOLOR.data());	
    // Get rid of the BODY BGCOLOR TAG :)
    bool end = false;
    signed int value = bodyPos;
    // Find the end
    while (end == false){
      if (newstr.length() <= value){
        end = true;	      
        value = -1;
      }
      if (newstr.at(value) == '>')
	end = true;
      value++;     
    }
    // Remove it.
    //printf("\nstring old: %s\n", newstr.data());
    if (value != 0){
      newstr.replace(bodyPos, value-bodyPos, "");
    }
    //newstr.replace( QRegExp("</BODY>", false), "" );
  }
  //printf("After removing body: %s\n", newstr.data());
  
  bodyPos = newstr.find("<FONT COLOR=");
  if(bodyPos != -1){
    temp_BuddyTextColor = colorSettings->chat_buddyText_color;
    QString COLOR = newstr.mid(bodyPos+14,8);
    colorSettings->chat_buddyText_color.setRgb(
      ((COLOR.mid(0,2)).toLong(0,16)),
      ((COLOR.mid(2,2)).toLong(0,16)),
      ((COLOR.mid(4,2)).toLong(0,16))
      );
    // Get rid of the FONT TAG :)
    bool end = false;
    signed int value = bodyPos;
    while (end == false){
      if (newstr.length() <= value){
        end = true;	      
        value = -1;
      }
      if (newstr.at(value) == '>')
	end = true;
      value++;     
    }
    //printf("\nstring old: %s\n", newstr.data());
    if (value != -1){
      newstr.replace(bodyPos, value-bodyPos, "");
    }
    
    // Find the end tag
    end = false;
    value = bodyPos;
    while (end == false){
      if (newstr.length() <= value){
        end = true;	      
        value = -1;
      }
      if (value != -1 && newstr.at(value) == '<'){
        if (value+6 < newstr.length())
          if ((newstr.at(value+2) == 'f' || newstr.at(value+2) == 'F')  && newstr.at(value+6) == '>')
            end = true;
      }	
      value++;     
    }
    //printf("string new: %s\n", newstr.data());
    // Remove it
    if (value != 0) 
      newstr.replace(value-1, 7, "");
    //printf("string final: %s\n", newstr.data());
    //printf("string new: %s\n", newstr.data());
  }
  // 3b) remove </body>
  newstr.replace( QRegExp("</body>", false), "" );
  // 4) make sure the allowed tags are balanced, inserting a </tag> at the end of
  // the message to balance if needed.
  #define CLOSE_TAG(a) if(tagIsOpen(newstr, a)) newstr += "</" a ">";
  CLOSE_TAG("a");
  CLOSE_TAG("b");
  CLOSE_TAG("big");
  CLOSE_TAG("code");
  CLOSE_TAG("em");
  CLOSE_TAG("font");
  CLOSE_TAG("i");
  CLOSE_TAG("small");
  CLOSE_TAG("tt");
  CLOSE_TAG("u");
  #undef CLOSE_TAG
  // 4b) check allowed tags, make sure quotes are balanced within them
  // 5) replace any "<" or ">" that's not part of an allowed tag with &lt; or &gt;
  //printf("StripTags Final product: %s\n", newstr.data());
  return (newstr);
}

/***************************************************************************
 * Opens a log file
 ***************************************************************************/
void ChatWindow::openLogFile(void){
  chatSettings->chat_log = true;
  
  QString fileNamePath;
    
  if (chatSettings->chat_log_singleFile == true){
    fileNamePath = QString(getenv("HOME")) + "/." + QString(APP_LOWERCASE_NAME) + "/logs/"
       	+ buddyName + "/" + buddyName + ".log";
    logFile.setName(fileNamePath);
  }
  else{
    // Each chat gets it's own file.
    QString endingTag;
    if (chatSettings->chat_log_inHtml == true)
      endingTag = ".html";
    else
      endingTag = ".txt";
    QString date = ((QDate::currentDate ()).toString());
    date.replace( QRegExp(" "), "" );

    QString logFolder = QString(getenv("HOME")) + "/." + QString(APP_LOWERCASE_NAME) + "/logs/" + buddyName + "/";
    QString fileNamePath = logFolder + buddyName + "-" + date + endingTag;
	
  
    logFile.setName(fileNamePath);
    int logNumber = 2;
    QString logNumberString;
    while (logFile.exists() == true){
      logNumberString.setNum(logNumber);
      fileNamePath = logFolder + buddyName + "-" + date + "-" + logNumberString + endingTag;
      logFile.setName(fileNamePath);
      logNumber++;
    }
  }
  
  if (logFile.open( IO_WriteOnly) == false){
    QDir newDir("/dev/null");
    //Make Dir
    newDir.mkdir(QString(getenv("HOME")) + "/." + QString(APP_LOWERCASE_NAME), true);
    newDir.mkdir(QString(getenv("HOME")) + "/." + QString(APP_LOWERCASE_NAME) + "/logs/", true);
    newDir.mkdir(QString(getenv("HOME")) + "/." + QString(APP_LOWERCASE_NAME) + "/logs/" + buddyName, true);
    if (logFile.open( IO_WriteOnly) == false){
      QMessageBox::critical(0, APP_NAME, QString("Unable to open ") + fileNamePath + " for appending.", "OK");
      chatSettings->chat_log = false;
      fileMenu->setItemChecked(ID_FILE_LOG, false );
      chatLogButton->setDown ( false );
    }
    else{
      logStream.setDevice(&logFile);		
      if (chatSettings->chat_log_inHtml == true){
        logStream << "<HTML>\n<HEAD>\n<TITLE>Conversations with ";
        logStream << buddyName;
        logStream << "</TITLE>\n</HEAD>\n<BODY BGCOLOR=\"#FFFFFF\">\n";
      }
      else{
        logStream << "Conversations with " << buddyName << "\n";
      }
    }
  }
  else{
    logStream.setDevice(&logFile);	 
    if (chatSettings->chat_log_inHtml == true){
      logStream << "<HTML>\n<HEAD>\n<TITLE>Conversations with ";
      logStream << buddyName;
      logStream << "</TITLE>\n</HEAD>\n<BODY BGCOLOR=\"#FFFFFF\">\n";
    }
    else{
      logStream << "Conversations with " << buddyName << "\n";
    }
  }
  if (chatSettings->chat_log_inHtml == true){
    if (( chatSettings->chat_log_time == true) && ( chatSettings->chat_log == true))
      logStream << conversationTimeBuffer;
    else
      logStream << conversationBuffer;
  }
  else {
  if (( chatSettings->chat_log_time == true) && ( chatSettings->chat_log == true))
      logStream << parseAndProcessALL(conversationTimeBuffer);
    else
      logStream << parseAndProcessALL(conversationBuffer);
  }


}

/***************************************************************************
 * Resizes the window
 * Occurs when the window size is changed.
 ***************************************************************************/
void ChatWindow::resizeEvent ( QResizeEvent * ){
  // scroll to the current text
  chatWindow->ensureVisible(0, chatWindow->contentsHeight()+100);

  // Until this function is needed....
	
	/*
	int toolBarHeight = 0;
	int toolBarWidth = 0;
  // Get the Toolbar's Height
  toolBarHeight = fileToolbar->height();
	// If the tool bar is mimimized
	if (toolBarHeight <= 1)
	  toolBarHeight = 10;
	// If the toolbar is up on left or right
	if ((fileToolbar->orientation()) == Vertical){
    toolBarHeight = 0;
	  toolBarWidth = fileToolbar->width();
  }
	
  //Toolbar on top
  if (fileToolbar->y() == this->height()-toolBarHeight)
  	spliter->setGeometry(toolBarWidth, 30,
	   this->width()-toolBarWidth, this->height()-toolBarHeight-30);
  // Toolbar on bottom
  if (fileToolbar->y() == this->height()-toolBarHeight-spliter->y())
  	spliter->setGeometry(toolBarWidth, 30+toolBarHeight,
	   this->width()-toolBarWidth, this->height()-toolBarHeight-30);
  */
}

/***************************************************************************
 * Sends the message wehn shift-enter is hit.
 * The return key itself will add a line so it is removed
 ***************************************************************************/
void ChatWindow::enterKeyPressed(){
  if((shiftIsPressed == true) && (chatSettings->chat_return_send == true)){
    inputWindow->append("\n");
  }
  else  
  if((shiftIsPressed == true && (chatSettings->chat_sreturn_send == true))
   ||(chatSettings->chat_return_send == true)){
    // Repace the text with what was just there before the return. :)
    inputWindow->setText(currentMessageOut);
    lastMessage = currentMessageOut;
    slotBuddySendMessage();
    currentMessageOut = "";
  }
}

/***************************************************************************
 * Saves what is in the input window.
 * Occurs when the text in the input window changes.
 ***************************************************************************/
void ChatWindow::inputTextChanged(){
  if ((inputWindow->text()).length() != 0)
    currentMessageOut = inputWindow->text();
}

/***************************************************************************
 * Sets shiftIsPressed = true
 * Occurs when the shift key is pressed.
 ***************************************************************************/
void ChatWindow::keyPressEvent(QKeyEvent *e){
  if(e->key() == Key_Shift){
    shiftIsPressed = true;
  }
}

/***************************************************************************
 * Sets shiftIsPressed = false
 * Occurs when the shift key is released .
 ***************************************************************************/
void ChatWindow::keyReleaseEvent(QKeyEvent *e){
  if(e->key() == Key_Shift){
    shiftIsPressed = false;
  }
}

/***************************************************************************
 * Turns an icon on and off untill screen has focus.
 ***************************************************************************/
void ChatWindow::flashIcon(){
  if (this->isActiveWindow() == true){
    flashWindow = false;
    this->setIcon(Icon("kaim_small"));
  }  
  if (flashWindow == true){
    QTimer::singleShot( 500, this, SLOT(flashIcon()));
    if (flashNumber == false){
      this->setIcon(Icon("kaim_small"));
      flashNumber = true;
    }  
    else{
      flashNumber = false;	
      this->setIcon(Icon("kaim_small_blink"));
    }
    //swap icons
  }
}

// chatwindow.cpp
#include "chatwindow.moc"
