/***************************************************************************
                                 chatwindow.h
                             -------------------
    begin                : Mon May 8 2000
    copyright            : (C) 2001 by Benjamin Meyer
    email                : ben@meyerhome.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 * This class provides a Chat window in that will parse incoming text
 * display, send, log, emit sound, and several other aim spisific features
 *
 * @Author Benjamin Meyer <benjamin@csh.rit.edu>
 * For other authors see authors page
 ***************************************************************************/

#ifndef CHATWINDOW_H
#define CHATWINDOW_H

#include <qmainwindow.h>
#include <qpopupmenu.h>
#include <qprinter.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qaccel.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qfiledialog.h>
#include <qmsgbox.h>
#include <qtoolbutton.h>
#include <qsplitter.h>
#include <kaimtextbrowser.h>
#include <qmultilineedit.h>
#include <qdatetime.h>
#include <qfont.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qcolordialog.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qpaintdevicemetrics.h>
#include <qsimplerichtext.h>
#include <qfile.h>
#include <qtextstream.h>

#include <stdlib.h>
#include <stdio.h>

#include "resource.h"
#include "browser.h"

struct chat_Settings {		
  bool chat_raise_window;         // Done
  bool chat_flash_on_new_message; // Done
  bool chat_log;                  // Done
  bool chat_log_inHtml;           // Done
  bool chat_show_time;            // Done
  bool chat_log_time;             // Done
  bool chat_return_send;	  // Done
  bool chat_sreturn_send;         // Done
  bool chat_send_blank;           // Done
  bool chat_ignore_buddyhtml;     // Done
  bool chat_line_limit;           // Done
  int chat_line_limit_number;     // Done
  bool chat_log_singleFile;
  bool chat_crypt;
  int chat_smallest_font_size;
};

struct chat_Color_Settings{
  QColor chat_yourText_color;     // Done
  QColor chat_yourName_color;     // Done
  QColor chat_buddyText_color;    // Done
  QColor chat_buddyName_color;    // Done
  QColor chat_yourbg_color;       // Done
  QColor chat_buddybg_color;      // Done
};

struct chat_Sound_Settings{
  bool chat_sound;                // Done
  bool chat_sound_beepInitial;     // Done
  bool chat_sound_beepReceive;    // Done
  bool chat_sound_beepSend;       // Done
  bool chat_sound_playFile;       // Done
  QString chat_sound_fileSend;
  QString chat_sound_fileReceive;
  QString chat_sound_fileInitial; 
};

struct chat_ToolBar_Settings {		
  bool view_text;                 // Done
  bool view_chat;                 // Done
  bool view_prev;                 // Done

  // 0 top
  // 1 right
  // 2 bottom
  // 3 left                       // Done
  int loc_text;                   // Done
  int loc_chat;                   // Done
  int loc_prev;                   // Done

  bool text_bold;                 // Done
  bool text_italic;               // Done
  bool text_underline;            // Done

  bool text_fontdown;             // Done
  bool text_fontnormal;           // Done
  bool text_fontbig;              // Done

  bool text_bgcolor;              // Done
  bool text_pencolor;             // Done
  bool text_link;                 // Done

  bool chat_clear;                // Done
  bool chat_save;                 // Done
  bool chat_print;                // Done
  bool chat_time;                 // Done

  bool chat_log;                  // Done
  bool chat_sound;                // Done

  bool chat_remove;               // Done
  bool chat_warn;                 // Done
  bool chat_block;                // Done
  bool chat_info;                 // Done

  bool prev_prev;                 // Done
  bool prev_send;                 // Done
};

/**
  *@author Benjamin Meyer
  */

class ChatWindow : public QMainWindow  {

  Q_OBJECT

  signals:
    void messageOut(QString buddy, QString Message);
		void antiIdle();
    void addBuddy(QString name);
    void removeBuddy(QString name);
    void blockBuddy(QString name);
    void warningOut(QString userName);
    void getInfo(QString Name);
    void quiting(QString name);
    void showAbout();
    void openHomePage();
		
  public:
    ChatWindow(QString userName, QString buddysName, bool buddy, QWidget *parent=0);
    ~ChatWindow();
    void buddySignedOff();
    void buddySignedOn();
    void pounceMessage(QString message);
    void awayMessage(QString message);
    bool sentAwayMessageToBuddy;
    Browser *browser;

  private:
    // Internal Settings
    QString myName;
    QString buddyName;
    QString currentMessageOut;
    bool shiftIsPressed;
    bool cleared;
    bool isBuddy;
    int fontSize;
    chat_ToolBar_Settings *toolBarSettings;
    chat_Color_Settings *colorSettings;
    chat_Settings *chatSettings;
    chat_Sound_Settings *soundSettings;
    QColor temp_BuddyBgColor;
    QColor temp_BuddyTextColor;
    QString lastMessage;
	
    // The Log Data's
    QFile logFile;
    QTextStream logStream;
		
    // The two buffers, one with time, one without.
    QString conversationBuffer;
    QString conversationTimeBuffer;
		
    // Normal function calls in the app
    void updateHTML(QString message, QString sender, QString HTMLColor);
    QString parseAndProcessALL(QString &newstr);
    void parseAndProcess(QString &newstr);
    QString parseForSpecialCharacters(QString &newstr);
    QString parseForHTML(QString &newstr);
    QString parseForSpelling(QString &newstr);
    QString parseForAutoReplace(QString &newstr);
	
    QString stripTags(const QString &oldstr);
    void openLogFile(void);
 
    // The init function for the widget
    void initToolBar();
    void initMenuBar();
    void initChatWindow();

    // The widgets in the tool bar
    QLabel *previewBox;
    QToolButton *sendMessageButton;
 
    QToolButton *chatClearButton;
    QToolButton *chatSaveButton;
    QToolButton *chatPrintButton;
    QToolButton *chatTimeButton;
    QToolButton *chatLogButton;
    QToolButton *chatSoundMuteButton;
 
    QToolButton *userRemoveButton;
    QToolButton *userWarnButton;
    QToolButton *userBlockButton;
    QToolButton *userInfoButton;
 
    QToolButton *textBoldButton;
    QToolButton *textItalicButton;
    QToolButton *textUnderLineButton;
    QToolButton *textStrikeButton;
 
    QToolButton *textUpFontButton;
    QToolButton *textNormFontButton;
    QToolButton *textDownFontButton;
    QToolButton *colorBackButton;
    QToolButton *colorPenButton;
    QToolButton *linkButton;
	
    // The Menu Bars
    QPopupMenu *fileMenu;
    QPopupMenu *editMenu;
    QPopupMenu *viewMenu;
    QPopupMenu *insertMenu;
    QPopupMenu *buddyMenu;
    QPopupMenu *helpMenu;

   	QToolBar *fileToolbar;
    QToolBar *textToolbar;
    QToolBar *previewToolbar;

    // The Chat window
    KaimTextBrowser *chatWindow;
    // used to get around the 0/1 issue on chatWindow input
    int postNumber;
    // Flash vars
    bool flashWindow;
    bool flashNumber;
    QMultiLineEdit *inputWindow;
    QSplitter *spliter;
 	
  public slots:
    void messageIn(QString message);
    
    void setIfIsBuddy(QString name, bool isABuddy);  
    void setChatToolBarSettings(chat_ToolBar_Settings *newsettings);
    void setChatColorSettings(chat_Color_Settings *newsettings);
    void setChatSettings(chat_Settings *newsettings);
    void setChatSoundSettings(chat_Sound_Settings *newsettings);
	
    /*void setAway(bool, const QString &);
      void setMyName(const QString &);
     */
  private slots:
    void flashIcon();
    void enterKeyPressed();
    void paintPreviewWidget();
    void inputTextChanged();
    void urlClicked(QString url);
    
    void slotChatLog();
    void slotChatClear();
    void slotChatSaveAs();
    void slotChatPrint();
    void slotChatClose();
    void slotChatSound();

    void slotEditTurnOnOffUndo(bool);
    void slotEditTurnOnOffRedo(bool);
    void slotEditUndo();
    void slotEditRedo();
    void slotEditCut();
    void slotEditCopy();
    void slotEditPaste();
    void slotEditSelectAll();

    void slotViewChatToolBar();
    void slotViewTextToolBar();
    void slotViewPreviewToolBar();
    void slotViewTimeStamp();

    void slotInsertTags(QString open, QString close);
    void slotInsertBold();
    void slotInsertItalic();
    void slotInsertUnderline();
    void slotInsertShrinkFont();
    void slotInsertNormalFont();
    void slotInsertIncreaseFont();
    void slotInsertBackGroundColor();
    void slotInsertPenColor();
    void slotInsertLink();
    void slotInsertLastMessage();
	
    void slotBuddySendMessage();
    void slotBuddyInfo();
    void slotBuddyWarn();
    void slotBuddyAddRemove();
    void slotBuddyBlock();

  protected:
    void keyReleaseEvent(QKeyEvent *e);
    void keyPressEvent(QKeyEvent *e);
		void resizeEvent(QResizeEvent *e );
    void closeEvent(QCloseEvent *);

};

#endif

// chatwindow.h
