/******************************************************************************
                                  kaim.cpp
                             -------------------
    begin                : Sat May 20 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ******************************************************************************/

/******************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by the
 *   Free Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.
 *
 ******************************************************************************/

/******************************************************************************
 * Main QObject for application.  All other windows are kept as children of
 * this class.  They are created and distroyed when needed.  Data pointer are
 * passed to the window that are needed.  These include the connection and
 * the settings.
 ******************************************************************************/

#include "kaim.h"
#include <qapplication.h>
#include "qmessagebox.h"
#include "kaimapplication.h"
#include "aim.h"
#include <stdlib.h>
#include <qtextstream.h>
#include <qfile.h>

/******************************************************************************
 * Constructor
 ******************************************************************************/
Kaim::Kaim(QWidget* parent, const char *name):QMainWindow(parent, name) {
  autoUpdateCheck = true;
	loggingOnError = false;
  quiting = false;
	manualLogOff = false;
	checkForUpdateReportErrors = false;
	browser = new Browser();
	//QMessageBox::information(this, "Message","1", QMessageBox::Ok);
  this->setIcon(Icon("kaim_small"));
  connect(this, SIGNAL(settingsChanged()), this, SLOT(SaveAimSettings()));
  
  // Get settings from file and set them to the current aimSettings
  getSettings = new setup();
  getSettings->read();
  aimSettings = getSettings->i_currentSettings;
  if (aimSettings->main_browser != "netscape"){
	 	browser->setBrowser(2);
	  browser->otherBrowser = aimSettings->main_browser;
	}
	// init the connection
  connection = new TAim(this, "Connection");
  connect(connection, SIGNAL(statusChanged(int)), this, SLOT(watchConnection(int)));
  connect(connection, SIGNAL( displayError(QString)), this, SLOT( displayError(QString)));

  buddyWindow = new BuddyListWindow(this, "buddyDialog", aimSettings, connection);
  connect(buddyWindow, SIGNAL(quit()), this, SLOT(Quit()));
  connect(buddyWindow, SIGNAL(logOff()), this, SLOT(Logoff()));
  connect(buddyWindow, SIGNAL(showOptions()), this, SLOT(BringUpSetup()));
  connect(buddyWindow, SIGNAL(saveSettings()), this, SLOT(SaveAimSettings()));
  connect(buddyWindow, SIGNAL(set_window(int,int)), this, SLOT(changeSize(int,int))); 
  connect(buddyWindow, SIGNAL(checkForNewRelease()), this, SLOT(checkForUpdatesWError()));
	connect(this, SIGNAL(settingsChanged()), buddyWindow, SLOT(settingsChanged()));
  // Option Box
  optionsDialog = new ConfigWindow(0/*this*/, "OptionsDialog", aimSettings,connection);
  optionsDialog->setIcon(Icon("kaim_small"));
  optionsDialog->setCaption("Kaim - Options");
  connect(optionsDialog, SIGNAL(dataUpdated()), this, SIGNAL(settingsChanged()));
  connect(optionsDialog, SIGNAL(chatToolbarUpdated()), buddyWindow, SLOT(chatToolbarUpdated())); 
  buddyWindow->hide();
  signOnDialog = NULL;
  
  loginDialog = new Login(this, "LoginDialog");
  connect(loginDialog, SIGNAL(setupClicked()), this, SLOT(BringUpSetup()));
  connect(loginDialog, SIGNAL(signOnClicked()), this, SLOT(TryToConnect()));
  connect(loginDialog, SIGNAL(destroyed()), this, SLOT(Quit()));
  connect(this, SIGNAL(settingsChanged()), loginDialog, SLOT(loadSettings()));
  loginDialog->hide();

  // Set state to Disconected();
  Disconected();
}

/******************************************************************************
 * Deconstructor
 ******************************************************************************/
Kaim::~Kaim(){
  delete getSettings;
}

/******************************************************************************
 * Called from buddylistwindow to resize the entire widget.
 ******************************************************************************/
void Kaim::changeSize(int x, int y){
  this->resize(x,y); 
}

/******************************************************************************
 * Network code had an error, display
 ******************************************************************************/
void Kaim::displayError(QString error){
  loggingOnError = true;
  if (manualLogOff == true){
		if (error != "Disconnected.")
			QMessageBox::critical(0, "Kaim Error",error, "Ok");
    manualLogOff = false;
	}
  else
	  QMessageBox::critical(0, "Kaim Error",error, "Ok");
}

/******************************************************************************
 * Kaim has been disconnected, bring up login screen
 ******************************************************************************/
void Kaim::Disconected(){
  if (connection->getStatus() != TAIM_OFFLINE){
    return;
  }    
  if ( signOnDialog != NULL){
    // Get rid of the signon window
    delete (signOnDialog);
    signOnDialog = NULL;
  }
  buddyWindow->hide();
  buddyWindow->awayRightNow = false;
  (buddyWindow->signedOff).clear();
  (buddyWindow->signedOn).clear();
	
	this->setMinimumSize(240,220);
	this->resize(240,220);
  
  //this->setMaximumSize(240,255);
    qApp->processEvents();
  // Bring up the Login Window
  this->setIcon(Icon(QString("kaim_small")));
  this->setCaption("Kaim");
  
  // Give it the settings and tell it to load them
  loginDialog->i_aimSettings = aimSettings;
  loginDialog->show();
	this->setCentralWidget(loginDialog);
	loginDialog->loadSettings();
  if (loggingOnError == false)
    if (aimSettings->login_enable_auto == true)
      loginDialog->SignOnClicked();  
}

/******************************************************************************
 * The Login Screen wants to try to log in.
 ******************************************************************************/
void Kaim::TryToConnect(){
  if (connection->getStatus() != TAIM_OFFLINE){
    return;
  } 
	loggingOnError = false;
  // Get rid of the loginDialog
  loginDialog->hide();
  this->setMinimumSize(240,220);
  this->resize(240,220);
	// Bring up the SignOn Window
  getSettings->readBuddyList();
  TBuddyList buddy;
  TBuddyList permit;
  TBuddyList deny;
  int permitStatus;
  tocParseConfig(getSettings->i_currentSettings->config, &buddy, &permit, &deny, &permitStatus);

  connection->setBuddyList(buddy);
  connection->setPermitList(permit);
  connection->setDenyList(deny);
  connection->setPermissions(permitStatus);
  if (signOnDialog == NULL){
    signOnDialog = new signOn(this, "SignOnDialog");
    // signOnWindow will kill itself if cancel is hit.
    connect(signOnDialog, SIGNAL(cancelHit()), this, SLOT(Disconected()));
    connect(signOnDialog, SIGNAL(connected()), this, SLOT(Connected()));
  }
  signOnDialog->i_aimSettings = aimSettings;
  signOnDialog->connection = connection;
  signOnDialog->show();
	this->setCentralWidget(signOnDialog);
  this->setIcon(Icon("kaim_small"));
  this->setCaption("Kaim - Login");
  // Tell it to start trying to connect
  signOnDialog->Connect();
}

/******************************************************************************
 * Kaim is connected to a toc server, Bring up buddylist
 ******************************************************************************/
void Kaim::Connected(){
  // If error on login
  // QMessageBox::critical(0, "Kaim Network Error","CONNECTED", "Ok");
  this->setMinimumSize(100,50);
  //this->setMaximumSize(1000,5000);
  if (connection->getStatus() != TAIM_OFFLINE){
   
    // Get rid of the signon window
    delete (signOnDialog);
    signOnDialog = NULL;
    if (aimSettings->main_noIcons == true)
    this->resize(170,310);
    qApp->processEvents();
    this->setCaption("Kaim");
    buddyWindow->loadList();
    buddyWindow->initOptions();
    buddyWindow->show();
    this->setCentralWidget(buddyWindow);

    SaveAimSettings();
  }
	
	//This is only set false through a command line option.
	if (autoUpdateCheck == true)
	  checkForUpdates();
}

/******************************************************************************
 * Someone wants the setup window, bring it up.  Saves when exiting.
 * The options window internally handles if changes are made.
 ******************************************************************************/
void Kaim::BringUpSetup(){
  optionsDialog->updateData();
  optionsDialog->show();
}

/******************************************************************************
 * Catches offline signals and acts
 ******************************************************************************/
void Kaim::watchConnection(int n){
  if ( n == TAIM_OFFLINE){
    if (quiting == false)
			Disconected();
	}
}

/******************************************************************************
 * Main window requests to log off.  Go offline and bring up login window.
 ******************************************************************************/
void Kaim::Logoff(){
  manualLogOff = true;
	connection->setStatus(TAIM_OFFLINE);
  Disconected();
	SaveAimSettings();
}

/******************************************************************************
 * Save the current settings to file.
 * Occurs when settings are changed (i.e. login and setup)
 ******************************************************************************/
void Kaim::SaveAimSettings(){
	QString conf;
  getSettings->save();
  if (connection->getStatus() != TAIM_OFFLINE){
    setProfile();
    conf = tocWriteConfig( &(connection->buddyList()),
      &(connection->permitList()),
      &(connection->denyList()),
      connection->permissions() );
    getSettings->i_currentSettings->config = conf;	
    getSettings->saveBuddyList();
  }
}

/***************************************************************************
 * Sets the current profile on the server to what is in the settings. 
 ***************************************************************************/
void Kaim::setProfile(){
  connection->setUserInfo(getSettings->i_currentSettings->profile->text());
}

/***************************************************************************
 * Check with the website to see if there are any updates.  State no
 * if there is none. 
 ***************************************************************************/
void Kaim::checkForUpdatesWError(){
  checkForUpdateReportErrors = true;
  checkForUpdates();
}

/***************************************************************************
 * Check with the website to see if there are any updates. 
 ***************************************************************************/
void Kaim::checkForUpdates(){
  system("wget -q -O /tmp/kaiminfo http://kaim.sourceforge.net/update/current_version ");
  QString settingsFile = "/tmp/kaiminfo";
  QFile file(settingsFile);
  QString newVersion = getSettings->i_currentSettings->Version;
  if (file.open(IO_ReadWrite)){
    newVersion = "";
		QTextStream stream(&file);
    while(!stream.eof()){
      newVersion += stream.readLine();
    }
    file.close();
    file.remove();
	}
	else{
    QMessageBox::information(0, "Kaim - Error",("Could not retrieve latest version\n"
								"information.  The website might be down."), QMessageBox::Ok);
	}
	bool isNewVersion = false;
	isNewVersion = newVersion != (getSettings->i_currentSettings->Version);
	if (isNewVersion)
	if ((checkForUpdateReportErrors == true) ||
	 		newVersion != getSettings->i_currentSettings->IgnoreThisUpdate
					){
		system("wget -q -O /tmp/kaiminfo http://kaim.sourceforge.net/update/latest_changes ");
    QString settingsFile = "/tmp/kaiminfo";
    QFile file(settingsFile);
    QString newChanges = "";
    if (file.open(IO_ReadWrite)){
		  QTextStream stream(&file);
      while(!stream.eof()){
        newChanges += stream.readLine();
				newChanges += "\n";
      }
      file.close();
      file.remove();
	  }
	  else{
      QMessageBox::information(0, "Kaim - Error",("Could not retrieve latest version changes\n"
								"The website might be down."), QMessageBox::Ok);
    } 
		newChanges = QString("A newer version of Kaim has been released.\n\n")
				+ newChanges;
		int choice = -1;
		choice = QMessageBox::information(0, QString("New Kaim Release: ") + newVersion,newChanges, "OK","Open Web Site","Don't show this version again.");

		if (choice == 1){
			system(getSettings->i_currentSettings->main_browser + QString( " ") + "http://kaim.sourceforge.net/ &");
		}
		if (choice == 2){
			getSettings->i_currentSettings->IgnoreThisUpdate = newVersion;
		}
	}
	else{
	  if (checkForUpdateReportErrors == true)
		  QMessageBox::information(0, "Kaim - Results","There are no new versions\n of Kaim at this time", "OK");
		checkForUpdateReportErrors = false;
	}
			
}

/***************************************************************************
 * Resizes the Buddy list window
 * Occurs when the window size is changed.
 ***************************************************************************/
void Kaim::resizeEvent ( QResizeEvent *e ){
  buddyWindow->setGeometry(0,0,this->width(),this->height());
	QWidget::resizeEvent(e);
  
	int moveDown = aimSettings->main_moveDown;
	if ( aimSettings->main_loginMoveLeft == true){
				this->setGeometry(this->geometry().x() - this->frameGeometry().x(), (this->geometry().y() - this->frameGeometry().y())+moveDown,this->width(),this->height());
	}
	if ( aimSettings->main_loginMoveRight == true){
		  int rootWidth = (QApplication::desktop())->width();
			this->setGeometry(rootWidth - this->frameGeometry().width(), (this->geometry().y() - this->frameGeometry().y())+moveDown,this->width(),this->height());

  }

}

/***************************************************************************
 * Quits the application
 * Logs off and saves first
 ***************************************************************************/
void Kaim::Quit(){
  if (connection->getStatus() != TAIM_OFFLINE){
    quiting = true;
		manualLogOff = true;
		connection->setStatus(TAIM_OFFLINE);
	} 
	SaveAimSettings();
  emit quit();  
}

/***************************************************************************
 * Close event occured, quit.
 ***************************************************************************/
void Kaim::closeEvent(class QCloseEvent *){
  Quit();
}
// kaim.cpp
#include "kaim.moc"

