/******************************************************************************
                                   kaim.h
                             -------------------
    begin                : Sat May 20 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ******************************************************************************/

/******************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by the
 *   Free Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.
 *
 ******************************************************************************/

/******************************************************************************
 * Main QObject for application.  All other windows are kept as children of
 * this class.  They are created and distroyed when needed.  Data pointer are
 * passed to the window that are needed.  These include the connection and
 * the settings.
 ******************************************************************************/

#ifndef KAIM_H
#define KAIM_H

#include <qmainwindow.h> 
#include "login.h"              // AIM Login Window
#include "signon.h"             // AIM SignOn Window
#include "buddylistwindow.h"    // AIM Buddy Lists Window
#include "taim.h"               // the toc protocal object
#include "kaimdialogs.h"	      // AIM Options Window
#include "setup.h"		          // AIM Settings
#include "tbuddylist.h"
#include "browser.h"

#include <qpixmap.h>

/**
  *@author Benjamin Meyer
  */

class Kaim : public QMainWindow  {
  Q_OBJECT

  signals:
    void quit();
    void settingsChanged();
	
  public:
    Kaim(QWidget* parent=0, const char *name=0);
    ~Kaim();
    bool autoUpdateCheck;
		
  private:
    Browser *browser;
		Login *loginDialog;
    signOn *signOnDialog;
    BuddyListWindow *buddyWindow;
    ConfigWindow *optionsDialog;
   
    setup *getSettings;
    settings *aimSettings;
    TAim *connection;
		bool checkForUpdateReportErrors;
    bool loggingOnError;
    bool quiting;
	  bool manualLogOff;
		
  private slots:
    void changeSize(int, int);  
    void displayError(QString error);
    void Quit();
    void Connected();
    void TryToConnect();
    void Disconected();
    void BringUpSetup();
    void SaveAimSettings();
    void Logoff();
    void watchConnection(int);
    void setProfile();
    void checkForUpdates();
	  void checkForUpdatesWError();	
		
  protected:
		void closeEvent(class QCloseEvent *);
    void resizeEvent( QResizeEvent *e );
};

#endif

// kaim.h

