// kaimiconloader.h
//
// Kaim
//
// Copyright 2000 by Neil Stevens <multivac @ fcmail.com>
//
// Licensed under the GNU Library General Public License

// KaimIconLoader interface inspired by KIconLoader in KDE

#ifndef KAIMICONLOADER_H
#define KAIMICONLOADER_H

#include <qpixmap.h>
#include <qdict.h>

class KaimIconLoader
{
	public:
		// First parameter is the default icon path.
		// See setIconPath for details on what that does.
		//
		// Second parameter is the size of the hash array to be used by the dictionary.
		KaimIconLoader(const QString & = "/usr/local/share/kaim/", int = 17);
		~KaimIconLoader();

		// Clear the cache of QPixmaps stored.
		// This ensures that the next load of any icon reads from the disk
		//
		// Warning: This cache has no space limits; if you want them, use QPixmapCache
		// KaimIconLoader is just designed to replace #included xpm files
		void clearCache(void);

		// Set the directory where the icons are stored.
		// This path is prepended to the icon names used.
		//
		// Setting a new icon path automatically clears the cache.
		void setIconPath(const QString &);

		// Load an icon with the specified name.
		//
		// Example: If the IconPath is /usr/local/share/kaim/
		// loadIcon("buddylist") will load /usr/local/share/kaim/buddylist.png
		const QPixmap &loadIcon(const QString &);
	private:
		QDict<QPixmap> cache;
		QString iconPath;
};

#endif
