/***************************************************************************
                             kaimtextbrowser.cpp  
                             -------------------
    begin                : Sun Nov 28 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

#include <kaimtextbrowser.h>
#include <qclipboard.h>
#include <qapplication.h>

/***************************************************************************
 * Constructor
 ***************************************************************************/
KaimTextBrowser::KaimTextBrowser(QWidget *parent, const char *name ) : QTextBrowser(parent,name){
  copyURL = false;
	URLRightClick = new QPopupMenu(this, "RightClick");
  URLRightClick->insertItem("&Open", this, SLOT(slotRightClick_Open()));
	URLRightClick->insertItem("&Copy URL", this, SLOT(slotRightClick_Copy()));
	URLMouseEvent = NULL;
	URLMouseEventPress = NULL;
  URLMouseEventRelease = NULL;
	testing = false;
}

/***************************************************************************
 * Deconstructor
 ***************************************************************************/
KaimTextBrowser::~KaimTextBrowser(){
  if (URLMouseEvent != NULL )
		delete URLMouseEvent;
	if (URLMouseEventRelease != NULL )
	  delete URLMouseEventRelease;
	if (URLMouseEventPress != NULL )
		delete URLMouseEventPress;
}

/***************************************************************************
 * When a url is clicked this is done instead of the normal.
 ***************************************************************************/
void KaimTextBrowser::setSource(const QString& name){
	if (testing == true){
		testing = false;
		URLRightClick->exec(tempPointGlobal);
	}
	else{
		if (copyURL == true){
	    QClipboard *cb = qApp->clipboard();
	    
			cb->setText(name);
	    copyURL = false;
	  }
	  else		
		  emit gotoURL(name);
  }
}

/***************************************************************************
 * When a mouse selects stuff emit a signal.
 ***************************************************************************/
void KaimTextBrowser::viewportMouseMoveEvent( QMouseEvent* e){
  if ( e->state() == 0 ) {
    if (this->hasSelectedText() == true)
      copy();
	}
  QTextBrowser::viewportMouseMoveEvent(e);
}

/***************************************************************************
 * Brings up the Menu when the right mouse is clicked.
 ***************************************************************************/
void KaimTextBrowser::viewportMousePressEvent(QMouseEvent* event){
	QTextBrowser::viewportMousePressEvent(event);
  // Right click was called
	if (event->button() == RightButton){
		// Save mouse position for posible menu exec call
		tempPointGlobal = event->globalPos();
		tempPointPos = event->pos();
		
		// General memory goodness
		if (URLMouseEvent != NULL )
		  delete URLMouseEvent;
		URLMouseEvent = new QMouseEvent(QEvent::MouseButtonPress,tempPointPos,tempPointGlobal,LeftButton,LeftButton);
	  
		// Call left click with testing on and if setSource gets called then we 
		// know it is a link and can show the menu.
		// Set up the two events
		if (URLMouseEventRelease != NULL )
		  delete URLMouseEventRelease;
    URLMouseEventRelease = new QMouseEvent(QEvent::MouseButtonRelease,tempPointPos,tempPointGlobal,LeftButton,LeftButton);
	  if (URLMouseEventPress != NULL )
		  delete URLMouseEventPress;
    URLMouseEventPress = new QMouseEvent(QEvent::MouseButtonPress,tempPointPos,tempPointGlobal,LeftButton,LeftButton);
    // Set testing to true and call both events.
		testing = true;
		this->viewportMousePressEvent(URLMouseEventPress);
		this->viewportMouseReleaseEvent(URLMouseEventRelease);
	}
}

/***************************************************************************
 * Copy the url highlighted
 ***************************************************************************/
void KaimTextBrowser::slotRightClick_Copy(){
  copyURL = true;
	if (URLMouseEventRelease != NULL )
	  delete URLMouseEventRelease;
  URLMouseEventRelease = new QMouseEvent(QEvent::MouseButtonRelease,tempPointPos,tempPointGlobal,LeftButton,LeftButton);
	this->viewportMousePressEvent(URLMouseEvent);
  this->viewportMouseReleaseEvent(URLMouseEventRelease);
}


/***************************************************************************
 * Simply do a leftClick
 ***************************************************************************/
void KaimTextBrowser::slotRightClick_Open(){
  if (URLMouseEventRelease != NULL )
	  delete URLMouseEventRelease;
  URLMouseEventRelease = new QMouseEvent(QEvent::MouseButtonRelease,tempPointPos,tempPointGlobal,LeftButton,LeftButton);
  this->viewportMousePressEvent(URLMouseEvent);
  this->viewportMouseReleaseEvent(URLMouseEventRelease);
}

// kaimtextbrowser.cpp

