/***************************************************************************
                          			kaimviewlist.cpp
                             -------------------
    begin                : Fri Jan 15 2001
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kaimviewlist.h"
#include <qapplication.h>

/***************************************************************************
 * Constructor
 ***************************************************************************/
KaimListView::KaimListView(QWidget *parent, const char *name ) : QListView(parent,name) {
	currentSelectedItem = NULL;
	connect(this, SIGNAL(pressed ( QListViewItem *)), this, SLOT(itemPressed(QListViewItem *)));
	internalCursor.setShape(0);
	this->setFocusPolicy(QWidget::ClickFocus);
	this->setCursor(internalCursor);

	buddyEdit= new EditBox(this->viewport(),"EditBox");
	buddyEdit->hide();
}

/***************************************************************************
 * Deconstructor
 ***************************************************************************/
KaimListView::~KaimListView(){
  //delete buddyEdit;
}

/***************************************************************************
 * Item has been pressed, lets watch it. 
 ***************************************************************************/
void KaimListView::itemPressed(QListViewItem *item){
  if (item != NULL){
		if (checkIfFolder(item) == false){
		  currentSelectedItem = item;
      internalCursor.setShape(13);
			qApp->setOverrideCursor(internalCursor);
		}
	}
}
		
/***************************************************************************
 * The mouse has been released. 
 ***************************************************************************/
void KaimListView::contentsMouseReleaseEvent(QMouseEvent* event){
	QListViewItem *newFolder = this->currentItem();
	// Make sure they are both real.
	if (currentSelectedItem != NULL  && newFolder != NULL){
		// Make sure that they are not the same.
		if (this->isSelected(currentSelectedItem) == false){
		  // Ok we have two valid items.
			// Is the new one a folder or buddy?	
			if (checkIfFolder(newFolder) != true){
				// get the buddies parent
				newFolder = newFolder->parent();
			}
			// Just in case the parent was null
			if (newFolder != NULL){
				// If the new folder and buddies current parent are the same don't do anything.	
			  if (newFolder != 	currentSelectedItem->parent())
          emit moveItem(currentSelectedItem, newFolder);
			}
	  }
	}
	currentSelectedItem = NULL;
  QListView::contentsMouseReleaseEvent(event);
	qApp->restoreOverrideCursor();
}

/***************************************************************************
 * Checks if the item that is selected is a folder or not.
 * @Return bool if "selected" is a folder.
 ***************************************************************************/
bool KaimListView::checkIfFolder(QListViewItem* selected){
  QListViewItem *parent = this->firstChild();
  bool isFolder = false;
  if(selected == 0) return false;
  if(parent == 0) return true;
  while( parent != 0 ){
    if (parent == selected)
      isFolder = true;
    parent = parent->nextSibling();
  }
  return isFolder;
}


/***************************************************************************
 * Overrides the default action of QEditLine  Hides this window.
 ***************************************************************************/
void KaimListView::focusOutEvent( QFocusEvent * ){
	currentSelectedItem = NULL;
	qApp->restoreOverrideCursor();
}

// KaimListView.cpp

