/***************************************************************************
                                  login.cpp
                             -------------------
    begin                : Sun Feb 13 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 * Login Box.  Ask for Name and Password.  Starts SignOn proccess.  Options
 * can also be brought up from here.
 ***************************************************************************/

#include "login.h"
#include "kaimapplication.h"

/***************************************************************************
 * Constructor
 ***************************************************************************/
Login::Login(QWidget *parent, const char *name) : QWidget(parent,name){
  // Load GUI
  initDialog();
  // Load main image from memory
  logoImage->setBackgroundPixmap(Icon("kaimlogo"));
  // Change Password fild to show stars and not the letters
  aolPass->setEchoMode(QLineEdit::Password);
  // Connect the GUI to the appropiate slots
  connect(autoLogin, SIGNAL(toggled(bool)), this, SLOT(toggleAuto(bool)));
  connect(save, SIGNAL(toggled(bool)), this, SLOT(toggleSave(bool)));
  connect(aolName, SIGNAL(activated(int) ), this, SLOT(moveToPass(int)));	
  connect(aolName, SIGNAL(textChanged( const QString &)), this, SLOT(newName(const QString&)));
  connect(aolPass, SIGNAL(returnPressed()), this, SLOT(SignOnClicked()));	
  connect(signOn, SIGNAL(clicked()), this, SLOT(SignOnClicked()));
  connect(setup, SIGNAL(clicked()), this, SLOT(SetupClicked()));
  // Start the sign on if auto login is enabled
  if (autoLogin->isChecked() == true);
    emit(signOnClicked());
}

/***************************************************************************
 * Destructor
 ***************************************************************************/
Login::~Login(){
}

/***************************************************************************
 * initDialog
 ***************************************************************************/
void  Login::initDialog(){
  Layout1 = new QVBoxLayout( this ); 
  Layout1->setSpacing( 6 );
  Layout1->setMargin( 11 );

  logoImage = new QLabel( this, "TextLabel1" );
  logoImage->setText( tr( "" ) );
	logoImage->setFixedSize(220,100);
  Layout1->addWidget( logoImage );

  Layout2 = new QHBoxLayout; 
  Layout2->setSpacing( 6 );
  Layout2->setMargin( 0 );

  aolnameL = new QLabel( this, "aolnameL" );
  aolnameL->setText( tr( "Name" ) );
  Layout2->addWidget( aolnameL );

  aolName = new QComboBox( true , this, "aolName" );
  Layout2->addWidget( aolName );
  Layout1->addLayout( Layout2 );

  Layout3 = new QHBoxLayout; 
  Layout3->setSpacing( 6 );
  Layout3->setMargin( 0 );

  aolpassL = new QLabel( this, "aolpassL" );
  aolpassL->setText( tr( "Password" ) );
  Layout3->addWidget( aolpassL );

  aolPass = new QLineEdit( this, "aolPass" );
  Layout3->addWidget( aolPass );
  Layout1->addLayout( Layout3 );

  Layout4 = new QHBoxLayout; 
  Layout4->setSpacing( 6 );
  Layout4->setMargin( 0 );

  save = new QCheckBox( this, "save" );
  save->setText( tr( "Save Password" ) );
  Layout4->addWidget( save );

  autoLogin = new QCheckBox( this, "autoLogin" );
  autoLogin->setText( tr( "Auto Login" ) );
  Layout4->addWidget( autoLogin );
  Layout1->addLayout( Layout4 );

  Layout5 = new QHBoxLayout; 
  Layout5->setSpacing( 6 );
  Layout5->setMargin( 0 );

  version = new QLabel( this, "verions" );
  version->setText( tr( "Ver 0.00" ) );
  Layout5->addWidget( version );

  setup = new QPushButton( this, "setup" );
  setup->setText( tr( "Options" ) );
  Layout5->addWidget( setup );

  signOn = new QPushButton( this, "signOn" );
  signOn->setText( tr( "Sign On" ) );
  Layout5->addWidget( signOn );
  Layout1->addLayout( Layout5 );
}

/***************************************************************************
 * Load settings (name and password and both checks)
 ***************************************************************************/
void Login::loadSettings(){
  // Set version number
  version->setText((i_aimSettings->Version).mid((i_aimSettings->Version).length()-4,4));
  QToolTip::add(version,i_aimSettings->ReleaseDate);
  // Load Name and Password
  aolPass->setText(i_aimSettings->login_password);
  aolName->insertItem(i_aimSettings->login_name);
  // Check buttons
  save->setChecked(i_aimSettings->login_enable_save);
  autoLogin->setChecked(i_aimSettings->login_enable_auto);
  
  QString homeDirr = QDir::homeDirPath() + "/.kaim/";
  aolName->clear(); 
  QDir home(homeDirr);
  home.setFilter( QDir::Files | QDir::NoSymLinks );
  if ( home.exists() ){
    QString fileName = "";
    QString buddy = "";
    for (int i = home.count(); i > 0; i--){
      fileName = home[i];
      buddy = fileName.mid(0, fileName.length()-10);
      fileName = fileName.mid(fileName.length()-10,10);
      if (fileName == ".buddyList"){
        aolName->insertItem(buddy);
      }
    }
  }
  for (int i = aolName->count(); i > 0; i--){
    if (aolName->text(i-1) == i_aimSettings->login_name)
      aolName->setCurrentItem(i-1);
  }
  if (aolName->count() == 0)
    aolName->insertItem("<new user>");
}

/***************************************************************************
 * If AutoSave is unchecked then it is deleted
 ***************************************************************************/
void Login::toggleSave(bool value){
  if (value == false){
    removePassword(aolName->currentText());
  }
} 

/***************************************************************************
 * If AutoLogin is checked the other one is automaticly checked and disabled
 ***************************************************************************/
void Login::toggleAuto(bool value){
  if (value == true){
    save->setEnabled(false);
    save->setChecked(true);
  }
  else{
    save->setEnabled(true);	
  }
} 

/***************************************************************************
 * Moves the Cursor to the Password field if return is hit
 ***************************************************************************/
void Login::moveToPass(int){
  aolPass->setFocus();
}

/***************************************************************************
 * When the name changes change the password too.
 ***************************************************************************/
void Login::newName(const QString &name){
  readPassword(name);
}

/******************************************************************************
 * Saves the password
 ******************************************************************************/
void Login::savePassword(QString name){
  if (name.length() > 0){
    QString passFile = getenv("HOME");
    passFile = passFile + "/.kaim/" + name + ".password";
    QFile::remove(passFile);
    QFile file(passFile);
    if (file.open(IO_ReadWrite))
    {
      QTextStream stream(&file);
      stream << aolPass->text();		
      file.close();
    }
  }  
}

/******************************************************************************
 * Reads the password 
 ******************************************************************************/
void Login::readPassword(QString name){
  QString passFile = getenv("HOME");
	passFile = passFile + "/.kaim/" + name + ".password";
  QFile file(passFile);
  QString password;
  if (file.exists()){
    if (file.open(IO_ReadOnly))
    {
      QTextStream stream(&file);
      while(!stream.eof()){
        password += stream.readLine();
      }
      file.close();
      aolPass->setText(password);
    }
    if (password == QString("") || !password)
      save->setChecked(false);
    else
      save->setChecked(true);
  }
  else{
    aolPass->setText("");
    if (autoLogin->isChecked() == false)
      save->setChecked(false);
  }
}

/******************************************************************************
 * Remove the password 
 ******************************************************************************/
void Login::removePassword(QString name){
  QString passFile = getenv("HOME");
  passFile = passFile + "/.kaim/" + name + ".password";
  QFile::remove(passFile);
}

/***************************************************************************
 * Attempts to Sign on
 * Occurs when the signOn button is clicked.
 ***************************************************************************/
void Login::SignOnClicked(){
  if ((aolName->currentText() == QString("")) || (aolPass->text() == QString("")) ){
     QMessageBox::information(this, "Message",("Please enter a name and password"), QMessageBox::Ok);
  }
  else{
    i_aimSettings->login_password = aolPass->text();
    i_aimSettings->login_name = aolName->currentText();
    i_aimSettings->login_enable_save = save->isChecked();
    i_aimSettings->login_enable_auto = autoLogin->isChecked();
    if (i_aimSettings->login_enable_save == true)
      savePassword(i_aimSettings->login_name);
    else
      removePassword(aolName->currentText());
    emit(signOnClicked());
  }
}

/***************************************************************************
 * Brings up the Setup Dialog box
 * Occurs when the Setup Button is hit.
 ***************************************************************************/
void Login::SetupClicked(){
  i_aimSettings->login_enable_save = save->isChecked();
  i_aimSettings->login_enable_auto = autoLogin->isChecked();
  emit (setupClicked());
} 

// login.cpp
#include "login.moc"
