/***************************************************************************
                               ChatOptionsImp.cpp
                             -------------------
    begin                : Tuesday Nov 14 2000
    copyright            : (C) 2000 Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/


#include "chatoptionsimp.h"
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qspinbox.h>

/******************************************************************************
 * Constructor
 ******************************************************************************/
ChatOptionsImp::ChatOptionsImp(QWidget *parent, const char *name, settings *aimSettings) : ChatOptions ( parent,name,false ){
  i_aimSettings = aimSettings;
  toolbarOptions = new ChatToolbarOptionsImp(this, "ChatToolBarOptions",aimSettings);
  connect(ConfigureToolBars, SIGNAL(clicked()), this, SLOT(openToolbarOptions()));
  connect(toolbarOptions, SIGNAL(saveSettings()), this, SIGNAL(chatToolbarUpdated()));
  toolbarOptions->hide();
  
  connect(chat_color_bname, SIGNAL(clicked()), this, SLOT(changeBuddyNameColor())); 
  connect(chat_color_btext, SIGNAL(clicked()), this, SLOT(changeBuddyTextColor())); 
  connect(chat_color_yname, SIGNAL(clicked()), this, SLOT(changeUserNameColor())); 
  connect(chat_color_ytext, SIGNAL(clicked()), this, SLOT(changeUserTextColor())); 
  connect(chat_color_ybcolor, SIGNAL(clicked()), this, SLOT(changeUserBGColor())); 
  connect(chat_color_bbcolor, SIGNAL(clicked()), this, SLOT(changeBuddyBGColor())); 
  connect(BuddyColorDefault, SIGNAL(clicked()), this, SLOT(setDefaultBuddyColors()));
  connect(MyColorDefault, SIGNAL(clicked()), this, SLOT(setDefaultUserColors()));
}

/******************************************************************************
 * Deconstructor
 ******************************************************************************/
ChatOptionsImp::~ChatOptionsImp(){
}

/******************************************************************************
 * Copy the settings to the dialogs
 ******************************************************************************/
void ChatOptionsImp::updateData(){

  chat_message_flash->setChecked(i_aimSettings->chatSettings->chat_flash_on_new_message);
  chat_message_raise->setChecked(i_aimSettings->chatSettings->chat_raise_window);
  chat_log_everything->setChecked(i_aimSettings->chatSettings->chat_log);
  chat_log_html->setChecked(i_aimSettings->chatSettings->chat_log_inHtml);
  chat_log_single_file->setChecked(!i_aimSettings->chatSettings->chat_log_singleFile);
  chat_show_time->setChecked(i_aimSettings->chatSettings->chat_show_time);
  minimumFontSize->setCurrentItem(i_aimSettings->chatSettings->chat_smallest_font_size);
  chat_log_time->setChecked(i_aimSettings->chatSettings->chat_log_time);
  chat_send_return->setChecked(i_aimSettings->chatSettings->chat_return_send);
  chat_send_shift->setChecked(i_aimSettings->chatSettings->chat_sreturn_send);
  chat_send_blank->setChecked(i_aimSettings->chatSettings->chat_send_blank);
  chat_remove_tages->setChecked(i_aimSettings->chatSettings->chat_ignore_buddyhtml);
  chat_conversation_limit->setChecked(i_aimSettings->chatSettings->chat_line_limit);
  if (i_aimSettings->chatSettings->chat_line_limit == true)
    chat_conversation_limit_number->setEnabled(true);
  else
    chat_conversation_limit_number->setEnabled(false);

  chat_conversation_limit_number->setValue(i_aimSettings->chatSettings->chat_line_limit_number);

  chat_log_everything->setChecked(i_aimSettings->chatSettings->chat_log);
 
  QPixmap color;
  color.resize(40,20);
  
  color.fill(i_aimSettings->chatColorSettings->chat_yourText_color);
  chat_color_ytext->setPixmap(color);  
  color.fill(i_aimSettings->chatColorSettings->chat_yourName_color);
  chat_color_yname->setPixmap(color);
  color.fill(i_aimSettings->chatColorSettings->chat_yourbg_color);
  chat_color_ybcolor->setPixmap(color);

  color.fill(i_aimSettings->chatColorSettings->chat_buddyText_color);
  chat_color_btext->setPixmap(color);  
  color.fill(i_aimSettings->chatColorSettings->chat_buddyName_color);
  chat_color_bname->setPixmap(color);
  color.fill(i_aimSettings->chatColorSettings->chat_buddybg_color);
  chat_color_bbcolor->setPixmap(color);
}

/******************************************************************************
 * Copy the settings to the dialogs
 ******************************************************************************/
void ChatOptionsImp::getData(){
  i_aimSettings->chatSettings->chat_raise_window = chat_message_raise->isChecked();
  i_aimSettings->chatSettings->chat_flash_on_new_message = chat_message_flash->isChecked();
  i_aimSettings->chatSettings->chat_log = chat_log_everything->isChecked();
  i_aimSettings->chatSettings->chat_log_inHtml = chat_log_html->isChecked();
  i_aimSettings->chatSettings->chat_log_singleFile = !chat_log_single_file->isChecked();
  i_aimSettings->chatSettings->chat_show_time = chat_show_time->isChecked();
  i_aimSettings->chatSettings->chat_smallest_font_size = minimumFontSize->currentItem();

  i_aimSettings->chatSettings->chat_log_time = chat_log_time->isChecked();
  i_aimSettings->chatSettings->chat_return_send = chat_send_return->isChecked();
  i_aimSettings->chatSettings->chat_sreturn_send = chat_send_shift->isChecked();
  i_aimSettings->chatSettings->chat_send_blank = chat_send_blank->isChecked();
  i_aimSettings->chatSettings->chat_ignore_buddyhtml = chat_remove_tages->isChecked();
  i_aimSettings->chatSettings->chat_line_limit = chat_conversation_limit->isChecked();
  i_aimSettings->chatSettings->chat_line_limit_number = chat_conversation_limit_number->value();

  i_aimSettings->chatSettings->chat_log = chat_log_everything->isChecked();

  emit (saveSettings());
}

/******************************************************************************
 * Show the toolbar options
 ******************************************************************************/
void ChatOptionsImp::openToolbarOptions(){
   toolbarOptions->show();
}

/******************************************************************************
 * Bring up the dialog to change the user text color.
 ******************************************************************************/
void ChatOptionsImp::changeUserTextColor(){
  QColor tempColor;
	tempColor = QColorDialog::getColor ( i_aimSettings->chatColorSettings->chat_yourText_color, this, "bgColorSelector" );
	if (tempColor.isValid() == true){
    i_aimSettings->chatColorSettings->chat_yourText_color = tempColor;
		QPixmap color;
    color.resize(40,20);
    color.fill(i_aimSettings->chatColorSettings->chat_yourText_color);
    chat_color_ytext->setPixmap(color);
	}	
}

/******************************************************************************
 * Bring up the dialog to change the user text color.
 ******************************************************************************/
void ChatOptionsImp::changeBuddyTextColor(){
  QColor tempColor;
	tempColor = QColorDialog::getColor ( i_aimSettings->chatColorSettings->chat_buddyText_color, this, "bgColorSelector" );
  if (tempColor.isValid() == true){
    i_aimSettings->chatColorSettings->chat_buddyText_color = tempColor;
	  QPixmap color;
    color.resize(40,20);
    color.fill(i_aimSettings->chatColorSettings->chat_buddyText_color);
    chat_color_btext->setPixmap(color);  
  }
}

/******************************************************************************
 * Bring up the dialog to change the user name color.
 ******************************************************************************/
void ChatOptionsImp::changeUserNameColor(){
  QColor tempColor;
	tempColor = QColorDialog::getColor ( i_aimSettings->chatColorSettings->chat_yourName_color, this, "bgColorSelector" );
  if (tempColor.isValid() == true){
		tempColor = i_aimSettings->chatColorSettings->chat_yourName_color;
		QPixmap color;
    color.resize(40,20);
    color.fill(i_aimSettings->chatColorSettings->chat_yourName_color);
    chat_color_yname->setPixmap(color);
  }
}

/******************************************************************************
 * Bring up the dialog to change the buddy name color.
 ******************************************************************************/
void ChatOptionsImp::changeBuddyNameColor(){
  QColor tempColor = QColorDialog::getColor ( i_aimSettings->chatColorSettings->chat_buddyName_color, this, "bgColorSelector" );
	if (tempColor.isValid() == true){
		i_aimSettings->chatColorSettings->chat_buddyName_color = tempColor;
    QPixmap color;
    color.resize(40,20);
    color.fill(i_aimSettings->chatColorSettings->chat_buddyName_color);
    chat_color_bname->setPixmap(color);
  }
}  

/******************************************************************************
 * Bring up the dialog to change the buddy background color.
 ******************************************************************************/
void ChatOptionsImp::changeBuddyBGColor(){
  QColor tempColor = QColorDialog::getColor ( i_aimSettings->chatColorSettings->chat_buddybg_color, this, "bgColorSelector" );
  if (tempColor.isValid() == true){
	  i_aimSettings->chatColorSettings->chat_buddybg_color = tempColor;
	  QPixmap color;
    color.resize(40,20);
    color.fill(i_aimSettings->chatColorSettings->chat_buddybg_color);
    chat_color_bbcolor->setPixmap(color);
  }
}

/******************************************************************************
 * Bring up the dialog to change the users background color.
 ******************************************************************************/
void ChatOptionsImp::changeUserBGColor(){
  QColor tempColor = QColorDialog::getColor ( i_aimSettings->chatColorSettings->chat_yourbg_color, this, "bgColorSelector" );
  if (tempColor.isValid() == true){
	 i_aimSettings->chatColorSettings->chat_yourbg_color = tempColor;
	  QPixmap color;
    color.resize(40,20);
    color.fill(i_aimSettings->chatColorSettings->chat_yourbg_color);
    chat_color_ybcolor->setPixmap(color);
  }
}

/******************************************************************************
 * set the default buddy colors
 ******************************************************************************/
void ChatOptionsImp::setDefaultBuddyColors(){
  i_aimSettings->chatColorSettings->chat_buddyText_color.setRgb(0,0,0);
  i_aimSettings->chatColorSettings->chat_buddyName_color.setRgb(128,0,0);
  i_aimSettings->chatColorSettings->chat_buddybg_color.setRgb(255,255,255);
  
  QPixmap color;
  color.resize(40,20);
  color.fill(i_aimSettings->chatColorSettings->chat_buddyText_color);
  chat_color_btext->setPixmap(color);  
  color.fill(i_aimSettings->chatColorSettings->chat_buddyName_color);
  chat_color_bname->setPixmap(color);
  color.fill(i_aimSettings->chatColorSettings->chat_buddybg_color);
  chat_color_bbcolor->setPixmap(color);
}

/******************************************************************************
 * set the default user colors
 ******************************************************************************/
void ChatOptionsImp::setDefaultUserColors(){
  i_aimSettings->chatColorSettings->chat_yourText_color.setRgb(0,0,0);
  i_aimSettings->chatColorSettings->chat_yourName_color.setRgb(0,60,128);
  i_aimSettings->chatColorSettings->chat_yourbg_color.setRgb(255,255,255);
  
  QPixmap color;
  color.resize(40,20);
  color.fill(i_aimSettings->chatColorSettings->chat_yourText_color);
  chat_color_ytext->setPixmap(color);  
  color.fill(i_aimSettings->chatColorSettings->chat_yourName_color);
  chat_color_yname->setPixmap(color);
  color.fill(i_aimSettings->chatColorSettings->chat_yourbg_color);
  chat_color_ybcolor->setPixmap(color);
}

//ChatOptionsImp.cpp
