/***************************************************************************
                           networkoptionsimp.cpp
                             -------------------
    begin                : Sunday Feb 11 2001
    copyright            : (C) 2001 Benjamin Meyer
    email                : ben@meyerhome.net
 ***************************************************************************/

#include <qlineedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qcheckbox.h>

#include "networkoptionsimp.h"

/******************************************************************************
 * Constructor
 ******************************************************************************/
NetworkOptionsImp::NetworkOptionsImp(QWidget *parent, const char *name, settings *aimSettings) : NetworkOptions( parent,name,false ){
  i_aimSettings = aimSettings;
	connect(login_name, SIGNAL(textChanged( const QString &)), this, SLOT(readPassword(const QString&)));
  connect(login_password, SIGNAL(textChanged(const QString &)), this, SLOT(passwordChanged(const QString &)));
  proxy_socks5->setChecked(true);
  proxy_none->setChecked(true);
}

/******************************************************************************
 * Deconstructor
 ******************************************************************************/
NetworkOptionsImp::~NetworkOptionsImp(){
}

/******************************************************************************
 * Copy the settings to the dialogs
 ******************************************************************************/
void NetworkOptionsImp::updateData(){
	proxy_socks5->setChecked(true);
	
	login_server_toc->setText(i_aimSettings->login_server_toc);
	login_server_login->setText(i_aimSettings->login_server_login);
	login_port_toc->setValue(atoi(i_aimSettings->login_port_toc));
	login_port_login->setValue(atoi(i_aimSettings->login_port_login));
	login_enable_save->setChecked(i_aimSettings->login_enable_save);
	login_enable_auto->setChecked(i_aimSettings->login_enable_auto);
	if (i_aimSettings->proxy_Choice == 0)
			proxy_none->setChecked(true);
	//if (i_aimSettings->proxy_Choice == 1)
	//		proxy_socks4->setChecked(true);
	if (i_aimSettings->proxy_Choice == 2)
			proxy_socks5->setChecked(true);
	proxy_server->setText(i_aimSettings->proxy_server);
	proxy_port->setText(i_aimSettings->proxy_port);
	
  QString homeDirr = QDir::homeDirPath() + "/.kaim/";
  login_name->clear(); 
  QDir home(homeDirr);
  home.setFilter( QDir::Files | QDir::NoSymLinks );
  if ( home.exists() ){
    QString fileName = "";
    QString buddy = "";
    for (int i = home.count(); i > 0; i--){
      fileName = home[i];
      buddy = fileName.mid(0, fileName.length()-10);
      fileName = fileName.mid(fileName.length()-10,10);
      if (fileName == ".buddyList"){
        login_name->insertItem(buddy);
      }
    }
  }
  for (int i = login_name->count(); i > 0; i--){
    if (login_name->text(i-1) == i_aimSettings->login_name)
      login_name->setCurrentItem(i-1);
  }
  if (login_name->count() == 0)
    login_name->insertItem("<new user>");
}

/******************************************************************************
 * Copy the settings to the dialogs
 ******************************************************************************/
void NetworkOptionsImp::getData(){
  i_aimSettings->login_server_toc = login_server_toc->text();
  i_aimSettings->login_server_login = login_server_login->text();
  i_aimSettings->login_port_toc = login_port_toc->text();
  i_aimSettings->login_port_login = login_port_login->text();
  //i_aimSettings->login_enable_save = login_enable_save->isChecked();
  i_aimSettings->login_enable_auto = login_enable_auto->isChecked();
  //i_aimSettings->login_name = login_name->text();
  //i_aimSettings->login_password = login_password->text();
  if (proxy_none->isChecked())
  i_aimSettings->proxy_Choice = 0;
  //if (proxy_socks4->isChecked())
  //  i_aimSettings->proxy_Choice = 1;
  if (proxy_socks5->isChecked())
    i_aimSettings->proxy_Choice = 2;
  i_aimSettings->proxy_server = proxy_server->text();
  i_aimSettings->proxy_port = proxy_port->text();
}

void NetworkOptionsImp::passwordChanged(const QString &){
  if (login_enable_save->isChecked() == true)
    savePassword(login_name->currentText());
}

/******************************************************************************
 * Saves the password
 ******************************************************************************/
void NetworkOptionsImp::savePassword(const QString &name){
  if (name.length() > 0){
    QString passFile = getenv("HOME");
    passFile = passFile + "/.kaim/" + name + ".password";
    QFile::remove(passFile);
    QFile file(passFile);
    if (file.open(IO_ReadWrite))
    {
      QTextStream stream(&file);
      stream << login_password->text();		
      file.close();
    }
  }  
}

/******************************************************************************
 * Reads the password 
 ******************************************************************************/
void NetworkOptionsImp::readPassword(const QString &name){
  QString passFile = getenv("HOME");
  passFile = passFile + "/.kaim/" + name + ".password";
  QFile file(passFile);
  QString password;
  if (file.exists()){
    if (file.open(IO_ReadOnly))
    {
      QTextStream stream(&file);
      while(!stream.eof()){
        password += stream.readLine();
      }
      file.close();
      login_password->setText(password);
    }
    if (password == QString("") || !password)
      login_enable_save->setChecked(false);
    else
      login_enable_save->setChecked(true);
  }
  else{
    login_password->setText("");
    if (login_enable_auto->isChecked() == false)
      login_enable_save->setChecked(false);
  }
}

/******************************************************************************
 * Remove the password 
 ******************************************************************************/
void NetworkOptionsImp::removePassword(QString name){
  QString passFile = getenv("HOME");
  passFile = passFile + "/.kaim/" + name + ".password";
  QFile::remove(passFile);
}

// NetworkOptionsImp.cpp
