/***************************************************************************
                          				setup.cpp
                             -------------------
    begin                : Thu Feb 17 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "setup.h"

/***************************************************************************
 * Constructor
 ***************************************************************************/
setup::setup(){
  i_currentSettings = new settings;
	
  i_currentSettings->chatSettings = new chat_Settings;
  i_currentSettings->chatColorSettings = new chat_Color_Settings;
  i_currentSettings->chatSoundSettings = new chat_Sound_Settings;
  i_currentSettings->chatToolBarSettings = new chat_ToolBar_Settings;

  i_currentSettings->profile = new QMultiLineEdit(0, "Profile");
  i_currentSettings->mapAwayMessages = new QMap<QString, QString>; 
  // Setting up defaults
  loadDefaults();
}

/***************************************************************************
 * Deconstructor
 ***************************************************************************/
setup::~setup(){
  delete i_currentSettings->chatSettings;
  delete i_currentSettings->chatColorSettings;
  delete i_currentSettings->chatSoundSettings;
  delete i_currentSettings->chatToolBarSettings;
  delete i_currentSettings->profile;
  delete i_currentSettings->mapAwayMessages;
  
  delete i_currentSettings;
}

/***************************************************************************
 * Resets the Current Settings to the base default
 ***************************************************************************/
void setup::loadDefaults(){
  i_currentSettings->config = "";
    
  // Sound
  i_currentSettings->SoundMode = 1;
  i_currentSettings->sound_enable_login = true;
  i_currentSettings->sound_enable_logout = false;
  i_currentSettings->sound_file_login = DATADIR "/sounds/BuddyArrive.au";
  i_currentSettings->sound_file_logout = DATADIR "/sounds/BuddyLeave.au";
 	
  // Network
  i_currentSettings->login_server_toc = "toc.oscar.aol.com";
  i_currentSettings->login_server_login = "login.oscar.aol.com";
  //i_currentSettings->login_port_toc = "9898";
  //i_currentSettings->login_port_login = "5190";
  i_currentSettings->login_port_toc = "23";
  i_currentSettings->login_port_login = "23";
  i_currentSettings->login_enable_save = true;
  i_currentSettings->login_enable_auto = false;
  i_currentSettings->login_name = "<new user>";
  i_currentSettings->login_password = "";
  i_currentSettings->proxy_Choice = 0;
  i_currentSettings->proxy_server = "";
  i_currentSettings->proxy_port = "";
 
  // Away
  i_currentSettings->away_disable_sound = true;
  i_currentSettings->away_enable_auto = false;
  i_currentSettings->away_enable_ai = false;
  i_currentSettings->away_enable_info = false;
  i_currentSettings->away_disable_sound = false;
  i_currentSettings->away_show_messageBox = true;
	i_currentSettings->away_auto_delay_minutes = 20;
	i_currentSettings->away_auto_default_message = 1;
	
  // Profile
  i_currentSettings->profile->setText("<P>Visit the Kaim website at:<A HREF=\"http://kaim.sourceforge.net/\">http://kaim.sourceforge.net/</A>.</P>");
	
  // Chat
  i_currentSettings->chatSettings->chat_raise_window = true;		
  i_currentSettings->chatSettings->chat_flash_on_new_message = false;
  i_currentSettings->chatSettings->chat_log = true;
  i_currentSettings->chatSettings->chat_log_inHtml = true;
  i_currentSettings->chatSettings->chat_show_time = true;
  i_currentSettings->chatSettings->chat_log_time = true;
  i_currentSettings->chatSettings->chat_return_send = true;				
  i_currentSettings->chatSettings->chat_sreturn_send = false;			
  i_currentSettings->chatSettings->chat_send_blank = false;
  i_currentSettings->chatSettings->chat_ignore_buddyhtml = false;
  i_currentSettings->chatSettings->chat_line_limit = false;
  i_currentSettings->chatSettings->chat_line_limit_number = 100;

  i_currentSettings->chatSettings->chat_log_singleFile = false;
  i_currentSettings->chatSettings->chat_crypt = false;
  i_currentSettings->chatSettings->chat_smallest_font_size = 3;

  i_currentSettings->chatToolBarSettings->view_text = true;
  i_currentSettings->chatToolBarSettings->view_chat = true;
  i_currentSettings->chatToolBarSettings->view_prev = true;
  i_currentSettings->chatToolBarSettings->loc_text = 0;
  i_currentSettings->chatToolBarSettings->loc_chat = 0;
  i_currentSettings->chatToolBarSettings->loc_prev = 2;
  i_currentSettings->chatToolBarSettings->text_bold = true;
  i_currentSettings->chatToolBarSettings->text_italic = true;
  i_currentSettings->chatToolBarSettings->text_underline = true;
  i_currentSettings->chatToolBarSettings->text_fontdown = true;
  i_currentSettings->chatToolBarSettings->text_fontnormal = true;
  i_currentSettings->chatToolBarSettings->text_fontbig = true;
  i_currentSettings->chatToolBarSettings->text_bgcolor = true;
  i_currentSettings->chatToolBarSettings->text_pencolor = true;
  i_currentSettings->chatToolBarSettings->text_link = true;
  i_currentSettings->chatToolBarSettings->chat_clear = true;
  i_currentSettings->chatToolBarSettings->chat_save = true;
  i_currentSettings->chatToolBarSettings->chat_print = true;
  i_currentSettings->chatToolBarSettings->chat_time = true;
  i_currentSettings->chatToolBarSettings->chat_log = true;
  i_currentSettings->chatToolBarSettings->chat_sound = true;
  i_currentSettings->chatToolBarSettings->chat_remove = true;
  i_currentSettings->chatToolBarSettings->chat_warn = true;
  i_currentSettings->chatToolBarSettings->chat_block = true;
  i_currentSettings->chatToolBarSettings->chat_info = true;
  i_currentSettings->chatToolBarSettings->prev_prev = true;
  i_currentSettings->chatToolBarSettings->prev_send = true;

  i_currentSettings->chatColorSettings->chat_yourText_color.setRgb(0,0,0);
  i_currentSettings->chatColorSettings->chat_yourName_color.setRgb(0,60,128);
  i_currentSettings->chatColorSettings->chat_buddyText_color.setRgb(0,0,0);
  i_currentSettings->chatColorSettings->chat_buddyName_color.setRgb(128,0,0);
  i_currentSettings->chatColorSettings->chat_yourbg_color.setRgb(255,255,255);
  i_currentSettings->chatColorSettings->chat_buddybg_color.setRgb(255,255,255);

  i_currentSettings->chatSoundSettings->chat_sound = false;
  i_currentSettings->chatSoundSettings->chat_sound_beepInitial = false;
  i_currentSettings->chatSoundSettings->chat_sound_beepReceive = true;
  i_currentSettings->chatSoundSettings->chat_sound_beepSend = false;
  i_currentSettings->chatSoundSettings->chat_sound_playFile = false;
  i_currentSettings->chatSoundSettings->chat_sound_fileSend = DATADIR "/sounds/Send.au";
  i_currentSettings->chatSoundSettings->chat_sound_fileReceive = DATADIR "/sounds/Receive.au";
  i_currentSettings->chatSoundSettings->chat_sound_fileInitial = DATADIR "/sounds/Receive.au";
  
  // General
  i_currentSettings->main_resizeHeight = true;
  i_currentSettings->main_resizeHeightMax = 480;
  i_currentSettings->main_resizeHeightMin = 100;
  i_currentSettings->main_resizeWidth = false;
  i_currentSettings->main_resizeWidthMax = 640;
  i_currentSettings->main_resizeWidthMin = 50;
  i_currentSettings->main_loginMoveRight = false;
  i_currentSettings->main_loginMoveLeft = false;
  i_currentSettings->main_loginMoveNo = true;
  i_currentSettings->main_showIdle = true;
  i_currentSettings->main_showLastOn = false;
  i_currentSettings->main_browser = "netscape";
  i_currentSettings->main_noIcons = false;
	i_currentSettings->main_moveDown = 0;
	i_currentSettings->main_idleLayout = 2;


  // Internal Settings (not to be saved)
  i_currentSettings->Version = "Version 0.62";
  i_currentSettings->ReleaseDate = "2/13/01";
	i_currentSettings->IgnoreThisUpdate = i_currentSettings->ReleaseDate;
}

/***************************************************************************
 * Save the settings to the file
 ***************************************************************************/
void setup::save(){
  // Gets Window Placement
  // int winX = this->x();
  // int winY = this->y();
  if (i_currentSettings->login_name == QString("<new user>"))
    return;
	  
  QString settingsFile = getenv("HOME");
  settingsFile = settingsFile + "/.kaim/settings";

  QFile::remove(settingsFile);
	
  QFile file(settingsFile);
  //Opens File for general settings
  if (file.open(IO_ReadWrite))
  {
    QTextStream stream(&file);		
    stream << "[Kaim Start]\n";
    stream << "\n";
    // Sound
    stream <<"[Sound]\n";
    stream <<"SoundMode=" << i_currentSettings->SoundMode << "\n";
    if (i_currentSettings->sound_enable_login == true)
      stream <<"sound_enable_login=1\n";
    else
      stream <<"sound_enable_login=0\n";
		if (i_currentSettings->sound_enable_logout == true)
			stream <<"sound_enable_logout=1\n";
		else
			stream <<"sound_enable_logout=0\n";
		stream <<"sound_file_login=" << i_currentSettings->sound_file_login << "\n";
		stream <<"sound_file_logout=" << i_currentSettings->sound_file_logout << "\n";
		stream <<"\n";
		
		// Network
		stream <<"[Network]\n";
		stream <<"login_server_toc=" << i_currentSettings->login_server_toc << "\n";
		stream <<"login_server_login="<< i_currentSettings->login_server_login << "\n";
		stream <<"login_port_toc=" <<  i_currentSettings->login_port_toc << "\n";
		stream <<"login_port_login=" << i_currentSettings->login_port_login << "\n";
		if (i_currentSettings->login_enable_auto == true)
			stream <<"login_enable_auto=1\n";
		else
			stream <<"login_enable_auto=0\n";
		stream <<"login_name="<< i_currentSettings->login_name << "\n";
		stream <<"proxy_Choice=" << i_currentSettings->proxy_Choice << "\n";
		stream <<"proxy_server=" << i_currentSettings->proxy_server << "\n";
		stream <<"proxy_port=" << i_currentSettings->proxy_port << "\n";
		stream <<"\n";
		
		// Permit Deny
		
		// Away
		stream<<"[Away]\n";
		if (i_currentSettings->away_enable_auto == true)
  		stream << "away_enable_auto=1\n";
  	else
  		stream << "away_enable_auto=0\n";
  	if (i_currentSettings->away_enable_ai == true)
  		stream << "away_enable_ai=1\n";
  	else
  		stream << "away_enable_ai=0\n";
  	if (i_currentSettings->away_enable_info == true)
  		stream << "away_enable_info=1\n";
  	else
  		stream << "away_enable_info=0\n";
  	if (i_currentSettings->away_disable_sound == true)
			stream << "away_enable_sound=1\n";
		else
  		stream << "away_enable_sound=0\n";
  	if (i_currentSettings->away_show_messageBox == true)
			stream << "away_show_messageBox=1\n";
		else
			stream << "away_show_messageBox=0\n";
		
		stream << QString("away_auto_delay_minutes=%1\n").arg(i_currentSettings->away_auto_delay_minutes);
	  stream << QString("away_auto_default_message=%1\n").arg(i_currentSettings->away_auto_default_message);
		stream <<"\n";
		
		// General
		stream <<"[General]\n";
		if (i_currentSettings->main_resizeHeight == true)
			stream <<"main_resizeHeight=1\n";
		else
			stream <<"main_resizeHeight=0\n";
		stream << QString("main_resizeHeightMax=%1\n").arg(i_currentSettings->main_resizeHeightMax);
		stream << QString("main_resizeHeightMin=%1\n").arg(i_currentSettings->main_resizeHeightMin);
		if (i_currentSettings->main_resizeWidth == true)
			stream <<"main_resizeWidth=1\n";
		else
			stream <<"main_resizeWidth=0\n";
		stream << QString("main_resizeWidthMax=%1\n").arg(i_currentSettings->main_resizeWidthMax);
		stream << QString("main_resizeWidthMin=%1\n").arg(i_currentSettings->main_resizeWidthMin);
		

		if (i_currentSettings->main_loginMoveRight == true)
	 		stream <<"main_loginMoveRight=1\n";
		else
			stream <<"main_loginMoveRight=0\n";
		if (i_currentSettings->main_loginMoveLeft == true)
	  	stream <<"main_loginMoveLeft=1\n";
		else
			stream <<"main_loginMoveLeft=0\n";
		if (i_currentSettings->main_loginMoveNo == true)
	  	stream <<"main_loginMoveNo=1\n";
		else
			stream <<"main_loginMoveNo=0\n";
		if (i_currentSettings->main_showIdle == true)
	  	stream <<"main_showIdle=1\n";
		else
			stream <<"main_showIdle=0\n";


		if (i_currentSettings->main_showLastOn == true)
			stream <<"main_showLastOn=1\n";
		else
			stream <<"main_showLastOn=0\n";
		stream << QString("main_browser=") +  i_currentSettings->main_browser + "\n";
		if (i_currentSettings->main_noIcons == true)
			stream <<"main_noIcons=1\n";
		else
			stream <<"main_noIcons=0\n";

		stream << QString("main_moveDown=%1\n").arg(i_currentSettings->main_moveDown);
		stream << QString("main_idleLayout=%1\n").arg(i_currentSettings->main_idleLayout);
		
	  stream <<"\n";
    
		// Chat
    stream <<"[Chat]\n";
    if (i_currentSettings->chatSettings->chat_raise_window == true) stream <<"chat_raise_window=1\n";
    else stream <<"chat_raise_window=0\n";
    if (i_currentSettings->chatSettings->chat_flash_on_new_message == true) stream <<"chat_flash_on_new_message=1\n";
    else stream <<"chat_flash_on_new_message=0\n";
    if (i_currentSettings->chatSettings->chat_log == true) stream <<"chat_log=1\n";
    else stream <<"chat_log=0\n";
    if (i_currentSettings->chatSettings->chat_log_inHtml == true) stream <<"chat_log_inHtml=1\n";
    else stream <<"chat_loginHtml=0\n";
    if (i_currentSettings->chatSettings->chat_show_time == true) stream <<"chat_show_time=1\n";
    else stream <<"chat_show_time=0\n";
    if (i_currentSettings->chatSettings->chat_log_time == true) stream <<"chat_log_time=1\n";
    else stream <<"chat_log_time=0\n";
    if (i_currentSettings->chatSettings->chat_return_send == true) stream <<"chat_return_send=1\n";
    else stream <<"chat_return_send=0\n";
    if (i_currentSettings->chatSettings->chat_sreturn_send == true) stream <<"chat_sreturn_send=1\n";
    else stream <<"chat_sreturn_send=0\n";
    if (i_currentSettings->chatSettings->chat_send_blank == true) stream <<"chat_send_blank=1\n";
    else stream <<"chat_send_blank=0\n";
    if (i_currentSettings->chatSettings->chat_ignore_buddyhtml == true) stream <<"chat_ignore_buddyhtml=1\n";
    else stream <<"chat_ignore_buddyhtml=0\n";
    if (i_currentSettings->chatSettings->chat_line_limit == true) stream <<"chat_line_limit=1\n";
    else stream <<"chat_line_limit=0\n";

   stream << "chat_line_limit_number=" << QString("%1").arg(i_currentSettings->chatSettings->chat_line_limit_number) << "\n";

   if (i_currentSettings->chatSettings->chat_log_singleFile == true)
     stream <<"chat_log_singleFile=1\n";
   else stream <<"chat_log_singleFile=0\n";

   if (i_currentSettings->chatSettings->chat_crypt == true)
     stream <<"chat_crypt=1\n";
   else stream <<"chat_crypt=0\n";

   stream << "chat_smallest_font_size=" << QString("%1").arg(i_currentSettings->chatSettings->chat_smallest_font_size) << "\n";

   if (i_currentSettings->chatToolBarSettings->view_text == true) stream <<"view_text=1\n";
    else stream <<"view_text=0\n";
    if (i_currentSettings->chatToolBarSettings->view_chat == true) stream <<"view_chat=1\n";
    else stream <<"view_chat=0\n";
    if (i_currentSettings->chatToolBarSettings->view_prev == true) stream <<"view_prev=1\n";
    else stream <<"view_prev=0\n";
    
    stream << "loc_text=" << (QString("%1")).arg(i_currentSettings->chatToolBarSettings->loc_text) << "\n";
    stream << "loc_chat=" << (QString("%1")).arg(i_currentSettings->chatToolBarSettings->loc_chat) << "\n";
    stream << "loc_prev=" << (QString("%1")).arg(i_currentSettings->chatToolBarSettings->loc_prev) << "\n";
    
    if (i_currentSettings->chatToolBarSettings->text_bold == true) stream <<"text_bold=1\n";
    else stream <<"text_bold=0\n";
    if (i_currentSettings->chatToolBarSettings->text_italic == true) stream <<"text_italic=1\n";
    else stream <<"text_italic=0\n";
    if (i_currentSettings->chatToolBarSettings->text_underline == true) stream <<"text_underline=1\n";
    else stream <<"text_underline=0\n";
    if (i_currentSettings->chatToolBarSettings->text_fontdown == true) stream <<"text_fontdown=1\n";
    else stream <<"text_fontdown=0\n";
    if (i_currentSettings->chatToolBarSettings->text_fontnormal == true) stream <<"text_fontnormal=1\n";
    else stream <<"text_fontnormal=0\n";
    if (i_currentSettings->chatToolBarSettings->text_fontbig == true) stream <<"text_fontbig=1\n";
    else stream <<"text_fontbig=0\n";
    if (i_currentSettings->chatToolBarSettings->text_bgcolor == true) stream <<"text_bgcolor=1\n";
    else stream <<"text_bgcolor=0\n";
    if (i_currentSettings->chatToolBarSettings->text_pencolor == true) stream <<"text_pencolor=1\n";
    else stream <<"text_pencolor=0\n";
    if (i_currentSettings->chatToolBarSettings->text_link == true) stream <<"text_link=1\n";
    else stream <<"text_link=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_clear == true) stream <<"chat_clear=1\n";
    else stream <<"chat_clear=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_save == true) stream <<"chat_save=1\n";
    else stream <<"chat_save=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_print == true) stream <<"chat_print=1\n";
    else stream <<"chat_print=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_time == true) stream <<"chat_time=1\n";
    else stream <<"chat_time=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_sound == true) stream <<"chat_sound=1\n";
    else stream <<"chat_sound=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_remove == true) stream <<"chat_remove=1\n";
    else stream <<"chat_remove=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_warn == true) stream <<"chat_warn=1\n";
    else stream <<"chat_warn=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_block == true) stream <<"chat_block=1\n";
    else stream <<"chat_block=0\n";
    if (i_currentSettings->chatToolBarSettings->chat_info == true) stream <<"chat_info=1\n";
    else stream <<"chat_info=0\n";
    if (i_currentSettings->chatToolBarSettings->prev_prev == true) stream <<"prev_prev=1\n";
    else stream <<"prev_prev=0\n";
    if (i_currentSettings->chatToolBarSettings->prev_send == true) stream <<"prev_send=1\n";
    else stream <<"prev_send=0\n";

  stream << "chat_yourText_color.R=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourText_color.red()) << "\n";
  stream << "chat_yourText_color.G=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourText_color.green()) << "\n";
  stream << "chat_yourText_color.B=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourText_color.blue()) << "\n";
  
  stream << "chat_yourName_color.R=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourName_color.red()) << "\n";
  stream << "chat_yourName_color.G=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourName_color.green()) << "\n";
  stream << "chat_yourName_color.B=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourName_color.blue()) << "\n";
  
  stream << "chat_buddyText_color.R=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddyText_color.red()) << "\n";
  stream << "chat_buddyText_color.G=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddyText_color.green()) << "\n";
  stream << "chat_buddyText_color.B=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddyText_color.blue()) << "\n";
  
  stream << "chat_buddyName_color.R=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddyName_color.red()) << "\n";
  stream << "chat_buddyName_color.G=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddyName_color.green()) << "\n";
  stream << "chat_buddyName_color.B=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddyName_color.blue()) << "\n";

  stream << "chat_buddyBg_color.R=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddybg_color.red()) << "\n";
  stream << "chat_buddyBg_color.G=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddybg_color.green()) << "\n";
  stream << "chat_buddyBg_color.B=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_buddybg_color.blue()) << "\n";

  stream << "chat_yourBg_color.R=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourbg_color.red()) << "\n";
  stream << "chat_yourBg_color.G=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourbg_color.green()) << "\n";
  stream << "chat_yourBg_color.B=" << (QString("%1")).arg(i_currentSettings->chatColorSettings->chat_yourbg_color.blue()) << "\n";

    if (i_currentSettings->chatSoundSettings->chat_sound == true) stream <<"chat_sound_enable=1\n";
    else stream <<"chat_sound_enable=0\n";
    if (i_currentSettings->chatSoundSettings->chat_sound_beepInitial == true) stream <<"chat_sound_beepInitial=1\n";
    else stream <<"chat_sound_beepInitial=0\n";
    if (i_currentSettings->chatSoundSettings->chat_sound_beepReceive == true) stream <<"chat_sound_beepReceive=1\n";
    else stream <<"chat_sound_beepReceive=0\n";
    if (i_currentSettings->chatSoundSettings->chat_sound_beepSend == true) stream <<"chat_sound_beepSend=1\n";
    else stream <<"chat_sound_beepSend=0\n";
    if (i_currentSettings->chatSoundSettings->chat_sound_playFile == true) stream <<"chat_sound_playFile=1\n";
    else stream <<"chat_sound_playFile=0\n";

    stream <<"chat_sound_fileSend=" << i_currentSettings->chatSoundSettings->chat_sound_fileSend << "\n";

    stream <<"chat_sound_fileReceive=" << i_currentSettings->chatSoundSettings->chat_sound_fileReceive << "\n"; 
    stream <<"chat_sound_fileInitial=" << i_currentSettings->chatSoundSettings->chat_sound_fileInitial << "\n"; 

    stream << "IgnoreThisUpdate=" << i_currentSettings->IgnoreThisUpdate << "\n";
				
		stream <<"\n";
    stream <<"[Kaim END]\n";
    file.close();
  }
  else{
    //Error in opening file to read data.
    QMessageBox::critical(0, "ERROR","Can not open settings file to save.", "Ok");
  }
	
  if (i_currentSettings->login_name != "<new user>"){
    // Save Profile
    QString awayListFile = getenv("HOME");
    awayListFile = awayListFile + "/.kaim/";
    awayListFile = awayListFile + i_currentSettings->login_name + ".profile";
	
    QFile::remove(awayListFile);
    QFile afile(awayListFile);
    //Opens File for general settings
    if (afile.open(IO_ReadWrite)){
      QTextStream stream(&afile);
      stream <<"[Kaim Profile]\n";
      stream<<(i_currentSettings->profile->text());		
      afile.close();
    }
    else{
      //Error in opening file to read data.
      QMessageBox::critical(0, "ERROR","Can not open profile file to save.", "Ok");
    }
    // Buddies are saved in the accuall AimBuddyList because the includes overlaped
    //if (i_currentSettings->login_name != QString("<new user>")){

    // Save Away List
    QString profileFile = getenv("HOME");
    profileFile = profileFile + "/.kaim/";
    profileFile = profileFile + i_currentSettings->login_name + ".awaylist";
	
    QFile::remove(profileFile);
    QFile pfile(profileFile);
    //Opens File for awaylist
    if (pfile.open(IO_ReadWrite)){
      QTextStream stream(&pfile);

      stream <<"[Kaim Away List]\n";

			QMap <QString,QString>::Iterator it;
			for(it = i_currentSettings->mapAwayMessages->begin(); 
							it != i_currentSettings->mapAwayMessages->end(); ++it){
	      stream <<"[Kaim Away Message]\n";
				stream << it.key().latin1() << "\n" << it.data().latin1() << "\n\n";
			}
      pfile.close();
    }
    else{
      //Error in opening file to read data.
      QMessageBox::critical(0, "ERROR","Can not open awaylist file to save.", "Ok");
    }
    //}
  }
}

/******************************************************************************
 * Saves the config
 ******************************************************************************/
void setup::saveBuddyList(){
  if (i_currentSettings->login_name == QString("<new user>"))
    return;
    
  QString settingsFile = getenv("HOME");
  settingsFile = settingsFile + "/.kaim/";
  settingsFile += i_currentSettings->login_name;
  settingsFile += ".buddyList";
  QFile::remove(settingsFile);
  QFile file(settingsFile);
  if (file.open(IO_ReadWrite))
  {
    QTextStream stream(&file);
    stream << i_currentSettings->config << "\n";		
    file.close();
  }
}

/******************************************************************************
 * Reads the config 
 ******************************************************************************/
void setup::readBuddyList(){
  QString settingsFile = getenv("HOME");
  settingsFile = settingsFile + "/.kaim/";
  settingsFile += i_currentSettings->login_name;
  settingsFile += ".buddyList";
  QFile file(settingsFile);
  QString config;
  if (file.open(IO_ReadWrite))
  {
    config += "CONFIG:";
    QTextStream stream(&file);
    while(!stream.eof()){
      config += stream.readLine();
      config += "\n";
    }
    i_currentSettings->config = config;		
    file.close();
  }
}

/***************************************************************************
 * Read the settings from the file
 ***************************************************************************/
void setup::read(){	
  //Get File Name
  QString settingsFile = getenv("HOME");
	
  QDir newDir(settingsFile + "/.kaim");
  //Make Dir
  newDir.mkdir(settingsFile + "/.kaim", true);
					
  settingsFile = settingsFile + "/.kaim/settings";
	
  QFile file(settingsFile);
  //Opens File
  if (file.open(IO_ReadWrite))
  {
    int r=0,g=0,b=0;
    QTextStream stream(&file);
    QString line="", setting="", settingValue = "";
    int value = 0;
    int equals = -1;
    while(!stream.eof()){
      setting = "";
      settingValue = "";
      value = 0;
      line = stream.readLine();
      equals = line.find("=");
      if (equals > -1)
      setting = line.mid(0, equals);
       
      settingValue = line.mid(equals+1,line.length()-equals-1);

			if (setting == QString("IgnoreThisUpdate"))
        i_currentSettings->IgnoreThisUpdate = settingValue;
					
			// Sound
			if (strcmp(setting, "SoundMode") == 0)
				value = 1;	
			if (strcmp(setting, "sound_enable_login") == 0)
				value = 2;	
			if (strcmp(setting, "sound_enable_logout") == 0)
				value = 3;	
			if (strcmp(setting, "sound_file_login") == 0)
				value = 7;	
			if (strcmp(setting, "sound_file_logout") == 0)
				value = 8;	
			
			// Network
			if (strcmp(setting, "login_server_toc") == 0)
				value = 12;	
			if (strcmp(setting, "login_server_login") == 0)
				value = 13;	
			if (strcmp(setting, "login_port_toc") == 0)
				value = 14;	
			if (strcmp(setting, "login_port_login") == 0)
				value = 15;	
			if (strcmp(setting, "login_enable_auto") == 0)
				value = 17;	
			if (strcmp(setting, "login_name") == 0)
				value = 18;	
			if (strcmp(setting, "proxy_Choice") == 0)
				value = 20;	
			if (strcmp(setting, "proxy_server") == 0)
				value = 21;	
			if (strcmp(setting, "proxy_port") == 0)
				value = 22;	
		
      // General
      if (strcmp(setting, "main_resizeHeight") == 0)
				if (settingValue == QString("1"))
          i_currentSettings->main_resizeHeight = true;
			  else
					i_currentSettings->main_resizeHeight = false;
			  	
      if (strcmp(setting, "main_resizeHeightMax") == 0)
       	i_currentSettings->main_resizeHeightMax = atoi(settingValue);
			if (strcmp(setting, "main_resizeHeightMin") == 0)
       	i_currentSettings->main_resizeHeightMin = atoi(settingValue);
			
			if (strcmp(setting, "main_resizeWidth") == 0){
				if (settingValue == QString("1"))
          i_currentSettings->main_resizeWidth = true;
			  else
					i_currentSettings->main_resizeWidth = false;
			}  	
      if (strcmp(setting, "main_resizeWidthMax") == 0)
       	i_currentSettings->main_resizeWidthMax = atoi(settingValue);
			if (strcmp(setting, "main_resizeWidthMin") == 0)
       	i_currentSettings->main_resizeWidthMin = atoi(settingValue);
			if (strcmp(setting, "main_loginMoveRight") == 0){
				if (settingValue == QString("1"))
          i_currentSettings->main_loginMoveRight = true;
			  else
					i_currentSettings->main_loginMoveRight = false;
			} 
      if (strcmp(setting, "main_loginMoveLeft") == 0){
				if (settingValue == QString("1"))
          i_currentSettings->main_loginMoveLeft = true;
			  else
					i_currentSettings->main_loginMoveLeft = false;
			} 
      if (strcmp(setting, "main_loginMoveNo") == 0){
				if (settingValue == QString("1"))
          i_currentSettings->main_loginMoveNo = true;
			  else
					i_currentSettings->main_loginMoveNo = false;
			} 
      if (strcmp(setting, "main_showIdle") == 0){
				if (settingValue == QString("1"))
          i_currentSettings->main_showIdle = true;
			  else
					i_currentSettings->main_showIdle = false;
			} 
			if (strcmp(setting, "main_showLastOn") == 0){
				if (settingValue == QString("1"))
          i_currentSettings->main_showLastOn = true;
			  else
					i_currentSettings->main_showLastOn = false;
			} 
      if (strcmp(setting, "main_browser") == 0)
        i_currentSettings->main_browser = settingValue;	
		  if (strcmp(setting, "main_noIcons") == 0){
				if (settingValue == QString("1"))
          i_currentSettings->main_noIcons = true;
			  else
					i_currentSettings->main_noIcons = false;
			} 
			if (strcmp(setting, "main_moveDown") == 0)
       	i_currentSettings->main_moveDown = atoi(settingValue);
			if (strcmp(setting, "main_idleLayout") == 0)
       	i_currentSettings->main_idleLayout = atoi(settingValue);
			
			// Away
			if (strcmp(setting, "away_auto_delay_minutes") == 0)
        i_currentSettings->away_auto_delay_minutes = atoi(settingValue);	
		  if (strcmp(setting, "away_auto_default_message") == 0)
        i_currentSettings->away_auto_default_message = atoi(settingValue);	
		  
			
			if (strcmp(setting, "away_enable_auto") == 0)
				value = 42;	
			if (strcmp(setting, "away_enable_ai") == 0)
				value = 43;	
			if (strcmp(setting, "away_enable_info") == 0)
				value = 44;	
			if (strcmp(setting, "away_disable_sound") == 0)
				value = 45;	
			if (strcmp(setting, "away_show_messageBox") == 0)
				value = 46;	
			if (strcmp(setting, "Config") == 0)
				value = 49;	

      // Chat
      if(strcmp(setting, "chat_raise_window") == 0)		
      	value = 50;	
      if(strcmp(setting, "chat_flash_on_new_message") == 0)
      	value = 51;	
      if(strcmp(setting, "chat_log") == 0)
      	value = 52;	
      if(strcmp(setting, "chat_log_inHtml") == 0)
      	value = 53;	
      if(strcmp(setting, "chat_show_time") == 0)
      	value = 54;	
      if(strcmp(setting, "chat_log_time") == 0)
      	value = 55;	
      if(strcmp(setting, "chat_return_send") == 0) 				
      	value = 56;	
      if(strcmp(setting, "chat_sreturn_send") == 0) 			
      	value = 57;	
      if(strcmp(setting, "chat_send_blank") == 0)
      	value = 58;	
      if(strcmp(setting, "chat_ignore_buddyhtml") == 0)
      	value = 59;	
      if(strcmp(setting, "chat_line_limit") == 0)
      	value = 60;	
      if(strcmp(setting, "chat_line_limit_number") == 0)
      	value = 61;	

      if(strcmp(setting, "view_text") == 0)
      	value = 62;	
      if(strcmp(setting, "view_chat") == 0)
      	value = 63;	
      if(strcmp(setting, "view_prev") == 0)
      	value = 64;	
      if(strcmp(setting, "loc_text") == 0)
      	value = 65;	
      if(strcmp(setting, "loc_chat") == 0)
      	value = 66;	
      if(strcmp(setting, "loc_prev") == 0)
      	value = 67;	
      if(strcmp(setting, "text_bold") == 0)
      	value = 68;	
      if(strcmp(setting, "text_italic") == 0)
      	value = 69;	
      if(strcmp(setting, "text_underline") == 0)
      	value = 70;	
      if(strcmp(setting, "text_fontdown") == 0)
      	value = 71;	
      if(strcmp(setting, "text_fontnormal") == 0)
      	value = 72;	
      if(strcmp(setting, "text_fontbig") == 0)
      	value = 73;	
      if(strcmp(setting, "text_bgcolor") == 0)
      	value = 74;	
      if(strcmp(setting, "text_pencolor") == 0)
      	value = 75;	
      if(strcmp(setting, "text_link") == 0)
      	value = 76;	
      if(strcmp(setting, "chat_clear") == 0)
      	value = 77;	
      if(strcmp(setting, "chat_save") == 0)
      	value = 78;	
      if(strcmp(setting, "chat_print") == 0)
      	value = 79;	
      if(strcmp(setting, "chat_time") == 0)
      	value = 80;	
      	
      if(strcmp(setting, "chat_sound") == 0)
      	value = 82;	
      if(strcmp(setting, "chat_remove") == 0)
      	value = 83;	
      if(strcmp(setting, "chat_warn") == 0)
      	value = 84;	
      if(strcmp(setting, "chat_block") == 0)
      	value = 85;	
      if(strcmp(setting, "chat_info") == 0)
      	value = 86;	
      if(strcmp(setting, "prev_prev") == 0)
      	value = 87;	
      if(strcmp(setting, "prev_send") == 0)
        value = 88;	
      
      if(strcmp(setting, "chat_yourText_color.R") == 0)
        value = 89;	
      if(strcmp(setting, "chat_yourText_color.G") == 0)
        value = 90;	
      if(strcmp(setting, "chat_yourText_color.B") == 0)
        value = 91;	
      
      if(strcmp(setting, "chat_yourName_color.R") == 0)
        value = 92;	
      if(strcmp(setting, "chat_yourName_color.G") == 0)
        value = 93;	
      if(strcmp(setting, "chat_yourName_color.B") == 0)
        value = 94;	   
     
      if(strcmp(setting, "chat_buddyText_color.R") == 0)
        value = 95;	
      if(strcmp(setting, "chat_buddyText_color.G") == 0)
        value = 96;	
      if(strcmp(setting, "chat_buddyText_color.B") == 0)
        value = 97;	
      
      if(strcmp(setting, "chat_buddyName_color.R") == 0)
        value = 98;	
      if(strcmp(setting, "chat_buddyName_color.G") == 0)
        value = 99;	
      if(strcmp(setting, "chat_buddyName_color.B") == 0)
        value = 100;	
      
      if(strcmp(setting, "chat_sound_enable") == 0)
        value = 101;	
      if(strcmp(setting, "chat_sound_beepInitial") == 0)
        value = 102;	
      if(strcmp(setting, "chat_sound_beepReceive") == 0)
        value = 103;	
      if(strcmp(setting, "chat_sound_beepSend") == 0)
        value = 104;	
      if(strcmp(setting, "chat_sound_playFile") == 0)
        value = 105;	
      if(strcmp(setting, "chat_sound_fileSend") == 0)
        i_currentSettings->chatSoundSettings->chat_sound_fileSend = settingValue;
      if(strcmp(setting, "chat_sound_fileReceive") == 0)
        i_currentSettings->chatSoundSettings->chat_sound_fileReceive = settingValue;	
      if(strcmp(setting, "chat_sound_fileInitial") == 0)
        i_currentSettings->chatSoundSettings->chat_sound_fileInitial = settingValue;
      
      if(strcmp(setting, "chat_log_singleFile") == 0)
        value = 108;	
      if(strcmp(setting, "chat_crypt") == 0)
        value = 109;	
      if(strcmp(setting, "chat_smallest_font_size") == 0)
        value = 110;	
      
      if(strcmp(setting, "chat_buddyBg_color.R") == 0)
        value = 111;	
      if(strcmp(setting, "chat_buddyBg_color.G") == 0)
        value = 112;	
      if(strcmp(setting, "chat_buddyBg_color.B") == 0)
        value = 113;	
      
      if(strcmp(setting, "chat_yourBg_color.R") == 0)
        value = 114;	
      if(strcmp(setting, "chat_yourBg_color.G") == 0)
        value = 115;	
      if(strcmp(setting, "chat_yourBg_color.B") == 0)
        value = 116;	    	
     
      if (value !=0 && settingValue != NULL && settingValue !=""){
				// Sound
				if (value == 1)
					i_currentSettings->SoundMode = atoi(settingValue);
				if (value == 2){
					if (settingValue == "1")
						i_currentSettings->sound_enable_login = true;
					else
						i_currentSettings->sound_enable_login = false;
				}
				if (value == 3){
					if (settingValue == "1")
						i_currentSettings->sound_enable_logout = true;
					else
						i_currentSettings->sound_enable_logout = false;
				}
				if (value == 7)
					i_currentSettings->sound_file_login = settingValue;
				if (value == 8)
					i_currentSettings->sound_file_logout = settingValue;
				
				// Network
				if (value == 12)
					i_currentSettings->login_server_toc = settingValue;
				if (value == 13)
					i_currentSettings->login_server_login = settingValue;
				if (value == 14)
					i_currentSettings->login_port_toc = settingValue;
				if (value == 15)
					i_currentSettings->login_port_login = settingValue;
				if (value == 17){
				  if (settingValue == "1")
				    i_currentSettings->login_enable_auto = true;
				  else
				    i_currentSettings->login_enable_auto = false;
				}
				if (value == 18)
				  i_currentSettings->login_name = settingValue;
				if (value == 20)
				  i_currentSettings->proxy_Choice = atoi(settingValue);
				if (value == 21)
				  i_currentSettings->proxy_server = settingValue;
				if (value == 22)
				  i_currentSettings->proxy_port = settingValue;
			
        // Away
        if (value == 42){
	        if (settingValue == "1")
	    i_currentSettings->away_enable_auto = true;
          else
	    i_currentSettings->away_enable_auto = false;
        }
	if (value == 43){
	  if (settingValue == "1")
	    i_currentSettings->away_enable_ai = true;
          else
	    i_currentSettings->away_enable_ai = false;
        }
        if (value == 44){
          if (settingValue == "1")
	    i_currentSettings->away_enable_info = true;
          else
	    i_currentSettings->away_enable_info = false;
        }
        if (value == 45){
          if (settingValue == "1")
			i_currentSettings->away_disable_sound = true;
        	else
			i_currentSettings->away_disable_sound = false;
        }
        if (value == 46){
	  	if (settingValue == "1")
				i_currentSettings->away_show_messageBox = true;
        	else
				i_currentSettings->away_show_messageBox = false;
       }
	  
		if (value == 49)
	  i_currentSettings->config = settingValue;
      }

      // Chat
      if (value == 50){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_raise_window = true;
        	else  i_currentSettings->chatSettings->chat_raise_window = false;
      }
			if (value == 51){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_flash_on_new_message = true;
        	else  i_currentSettings->chatSettings->chat_flash_on_new_message = false;
      }
			if (value == 52){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_log = true;
        	else  i_currentSettings->chatSettings->chat_log = false;
      }
			if (value == 53){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_log_inHtml = true;
        	else  i_currentSettings->chatSettings->chat_log_inHtml = false;
      }
			if (value == 54){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_show_time = true;
        	else  i_currentSettings->chatSettings->chat_show_time = false;
      }
			if (value == 55){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_log_time = true;
        	else  i_currentSettings->chatSettings->chat_log_time = false;
      }
			if (value == 56){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_return_send = true;
        	else  i_currentSettings->chatSettings->chat_return_send = false;
      }
			if (value == 57){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_sreturn_send = true;
        	else  i_currentSettings->chatSettings->chat_sreturn_send = false;
      }
			if (value == 58){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_send_blank = true;
        	else  i_currentSettings->chatSettings->chat_send_blank = false;
      }
			if (value == 59){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_ignore_buddyhtml = true;
        	else  i_currentSettings->chatSettings->chat_ignore_buddyhtml = false;
      }
			if (value == 60){
			  	if (settingValue == "1") i_currentSettings->chatSettings->chat_line_limit = true;
        	else  i_currentSettings->chatSettings->chat_line_limit = false;
      }
			if (value == 61){
			  i_currentSettings->chatSettings->chat_line_limit_number = atoi(settingValue);
                        }
			if (value == 62){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->view_text = true;
        	else  i_currentSettings->chatToolBarSettings->view_text = false;
      }
			if (value == 63){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->view_chat = true;
        	else  i_currentSettings->chatToolBarSettings->view_chat = false;
      }
			if (value == 64){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->view_prev = true;
        	else  i_currentSettings->chatToolBarSettings->view_prev = false;
      }
			if (value == 65){
			  	i_currentSettings->chatToolBarSettings->loc_text = atoi(settingValue);
      }
			if (value == 66){
			  	i_currentSettings->chatToolBarSettings->loc_chat = atoi(settingValue);
      }
			if (value == 67){
			  	i_currentSettings->chatToolBarSettings->loc_prev = atoi(settingValue);
      }
			if (value == 68){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_bold = true;
        	else  i_currentSettings->chatToolBarSettings->text_bold = false;
      }
			if (value == 69){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_italic = true;
        	else  i_currentSettings->chatToolBarSettings->text_italic = false;
      }
			if (value == 70){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_underline = true;
        	else  i_currentSettings->chatToolBarSettings->text_underline = false;
      }
			if (value == 71){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_fontdown = true;
        	else  i_currentSettings->chatToolBarSettings->text_fontdown = false;
      }
			if (value == 72){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_fontnormal = true;
        	else  i_currentSettings->chatToolBarSettings->text_fontnormal = false;
      }
			if (value == 73){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_fontbig = true;
        	else  i_currentSettings->chatToolBarSettings->text_fontbig = false;
      }
			if (value == 74){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_bgcolor = true;
        	else  i_currentSettings->chatToolBarSettings->text_bgcolor = false;
      }
			if (value == 75){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_pencolor = true;
        	else  i_currentSettings->chatToolBarSettings->text_pencolor = false;
      }
			if (value == 76){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->text_link = true;
        	else  i_currentSettings->chatToolBarSettings->text_link = false;
      }
			if (value == 77){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_clear = true;
        	else  i_currentSettings->chatToolBarSettings->chat_clear = false;
      }
			if (value == 78){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_save = true;
        	else  i_currentSettings->chatToolBarSettings->chat_save = false;
      }
			if (value == 79){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_print = true;
        	else  i_currentSettings->chatToolBarSettings->chat_print = false;
      }
			if (value == 80){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_time = true;
        	else  i_currentSettings->chatToolBarSettings->chat_time = false;
      }
		
			if (value == 82){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_sound = true;
        	else  i_currentSettings->chatToolBarSettings->chat_sound = false;
      }
			if (value == 83){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_remove = true;
        	else  i_currentSettings->chatToolBarSettings->chat_remove = false;
      }
			if (value == 84){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_warn = true;
        	else  i_currentSettings->chatToolBarSettings->chat_warn = false;
      }
			if (value == 85){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_block = true;
        	else  i_currentSettings->chatToolBarSettings->chat_block = false;
      }
			if (value == 86){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->chat_info = true;
        	else  i_currentSettings->chatToolBarSettings->chat_info = false;
      }
			if (value == 87){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->prev_prev = true;
        	else  i_currentSettings->chatToolBarSettings->prev_prev = false;
      }
			if (value == 88){
			  	if (settingValue == "1") i_currentSettings->chatToolBarSettings->prev_send = true;
        	else  i_currentSettings->chatToolBarSettings->prev_send = false;
      }
      if (value == 89)
        r = atoi(settingValue);
      if (value == 90)
        g = atoi(settingValue);
      if (value == 91){
        b = atoi(settingValue);
        i_currentSettings->chatColorSettings->chat_yourText_color.setRgb(r,g,b);
      }
      
      if (value == 92)
        r = atoi(settingValue);
      if (value == 93)
        g = atoi(settingValue);
      if (value == 94){
        b = atoi(settingValue);
        i_currentSettings->chatColorSettings->chat_yourName_color.setRgb(r,g,b);
      }
	
      if (value == 95)
        r = atoi(settingValue);
      if (value == 96)
        g = atoi(settingValue);
      if (value == 97){
        b = atoi(settingValue);
      i_currentSettings->chatColorSettings->chat_buddyText_color.setRgb(r,g,b);
      }
     
      
      if (value == 98)
        r = atoi(settingValue);
      if (value == 99)
        g = atoi(settingValue);
      if (value == 100){
        b = atoi(settingValue);
        i_currentSettings->chatColorSettings->chat_buddyName_color.setRgb(r,g,b);}    
      if (value == 101){
        if (settingValue == "1")
	  i_currentSettings->chatSoundSettings->chat_sound = true;
	else
      	  i_currentSettings->chatSoundSettings->chat_sound = false;
      }
      if (value == 102){
        if (settingValue == "1")
	  i_currentSettings->chatSoundSettings->chat_sound_beepInitial = true;
	else
      	  i_currentSettings->chatSoundSettings->chat_sound_beepInitial = false;
      }
      if (value == 103){
        if (settingValue == "1")
	  i_currentSettings->chatSoundSettings->chat_sound_beepReceive = true;
	else
      	  i_currentSettings->chatSoundSettings->chat_sound_beepReceive = false;
      }
      if (value == 104){
        if (settingValue == "1")
	  i_currentSettings->chatSoundSettings->chat_sound_beepSend = true;
	else
      	  i_currentSettings->chatSoundSettings->chat_sound_beepSend = false;
      }
      if (value == 105){
        if (settingValue == "1")
	  i_currentSettings->chatSoundSettings->chat_sound_playFile = true;
	else
      	  i_currentSettings->chatSoundSettings->chat_sound_playFile = false;
      }
      
      if (value == 108){
        if (settingValue == "1")
	  i_currentSettings->chatSettings->chat_log_singleFile = true;
	else
      	  i_currentSettings->chatSettings->chat_log_singleFile = false;
      }
      if (value == 109){
        if (settingValue == "1")
	  i_currentSettings->chatSettings->chat_crypt = true;
	else
      	  i_currentSettings->chatSettings->chat_crypt = false;
      }
      if (value == 110){
        i_currentSettings->chatSettings->chat_smallest_font_size = atoi(settingValue);
      }
      
      if (value == 111)
        r = atoi(settingValue);
      if (value == 112)
        g = atoi(settingValue);
      if (value == 113){
        b = atoi(settingValue);
        i_currentSettings->chatColorSettings->chat_buddybg_color.setRgb(r,g,b);
      }
      
      if (value == 114)
        r = atoi(settingValue);
      if (value == 115)
        g = atoi(settingValue);
      if (value == 116){
        b = atoi(settingValue);
        i_currentSettings->chatColorSettings->chat_yourbg_color.setRgb(r,g,b);
      }

    }
    file.close();
  }
  else
  {
    //error can't open file
    QMessageBox::critical(0, "ERROR","Can not open settings file.", "Ok");
  }
	
  // Read Profile
  QString profileFile = getenv("HOME");
  profileFile = profileFile + "/.kaim/";
  profileFile = profileFile + i_currentSettings->login_name + ".profile";

  QFile pfile(profileFile);
  //Opens File for general settings
  if (pfile.exists()){
  if (pfile.open(IO_ReadWrite)){
    QTextStream stream(&pfile);
    QString line = "";
    bool correctFile = false;
    while(!stream.eof()){
      line = stream.readLine();
      if (correctFile == true)
        i_currentSettings->profile->insertLine(line);
      if (line == "[Kaim Profile]"){
	i_currentSettings->profile->clear();
	correctFile = true;
      }
    }
    pfile.close();
  }
  else{
    //Error in opening file to read data.
    QMessageBox::critical(0, "ERROR","Can not open profile.", "Ok");
  }
  }	
	
  // Read Away Mesages
  QString awayMessagesFile = getenv("HOME");
  awayMessagesFile = awayMessagesFile + "/.kaim/";
  awayMessagesFile = awayMessagesFile + i_currentSettings->login_name + ".awaylist";
	
  QFile afile(awayMessagesFile);
  //Opens File for general settings
  if (afile.exists()){
  if (afile.open(IO_ReadWrite)){
    QTextStream stream(&afile);
    QString line = "", text = "";
    int numberFound = 0;
    //int found_message = 0;

		while(!stream.eof()){
			while(!stream.eof() && (line != "[Kaim Away Message]"))
	      line = stream.readLine();
			line = stream.readLine();
			text = stream.readLine();
			if(!line.isNull() && !text.isNull()){
				i_currentSettings->mapAwayMessages->insert(line, text);
				numberFound++;
			}
		}
    afile.close();
    if (numberFound == 0){
			i_currentSettings->mapAwayMessages->insert(QString("Away"), 
																		QString("I am away from my computer right now."));
    }
  }
  else{
    //Error in opening file to read data.
    QMessageBox::critical(0, "ERROR","Can not open awaylist.", "Ok");
  }
  }
}

// setup.cpp

